/*
 * *      adc.c
 * *
 * *      Copyright (C) 2012 Jiri Pittner <jiri@pittnerovi.com>
 * *
 * *      This program is free software: you can redistribute it and/or modify
 * *      it under the terms of the GNU General Public License as published by
 * *      the Free Software Foundation, either version 3 of the License, or
 * *      (at your option) any later version.
 * *
 * *      This program is distributed in the hope that it will be useful,
 * *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 * *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * *      GNU General Public License for more details.
 * *      You should have received a copy of the GNU General Public License
 * *      along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * *
 * *
 * */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include "delay.h"
#include "usart.h"
#include "adc.h"


uint32_t read_ADC(uint16_t nrep)
{
uint32_t r=0;
//discard first after the switching
//ADC_SoftwareStartConv(ADC1);
while(ADC_GetFlagStatus(ADC1, ADC_FLAG_EOC) == RESET);
ADC_GetConversionValue(ADC1);

do{
	//ADC_SoftwareStartConv(ADC1);
	while(ADC_GetFlagStatus(ADC1, ADC_FLAG_EOC) == RESET);
	r+= ADC_GetConversionValue(ADC1);
}while(--nrep);
return r;
}

void array_read_ADC(uint16_t nrep, uint16_t *store)
{
uint16_t i;
for(i=0; i<nrep; ++i)
	{
        //ADC_SoftwareStartConv(ADC1);
        while(ADC_GetFlagStatus(ADC1, ADC_FLAG_EOC) == RESET);
        store[i] = ADC_GetConversionValue(ADC1);
	}
}


uint8_t init_ADC(uint8_t channel)
{
RCC_ADCCLKConfig(RCC_PCLK2_Div4); //max 14MHz, Div2 is too fast, maybe use even div6
RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA, ENABLE);
GPIO_InitTypeDef    GPIO_InitStructure;
GPIO_InitStructure.GPIO_Pin = GPIO_Pin_1|GPIO_Pin_2|GPIO_Pin_3;
GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AN;
GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL ;
GPIO_Init(GPIOA, &GPIO_InitStructure);

RCC_APB2PeriphClockCmd(RCC_APB2Periph_ADC1, ENABLE);
ADC_DeInit(ADC1);

ADC_InitTypeDef     ADC_InitStructure;
ADC_StructInit(&ADC_InitStructure);
ADC_InitStructure.ADC_ScanConvMode = DISABLE;
ADC_InitStructure.ADC_ContinuousConvMode = ENABLE; //faster than always starting it
ADC_InitStructure.ADC_ExternalTrigConv = ADC_ExternalTrigConv_None;
ADC_InitStructure.ADC_DataAlign = ADC_DataAlign_Right;
ADC_InitStructure.ADC_NbrOfChannel = 1;
ADC_Init(ADC1, &ADC_InitStructure);

ADC_RegularChannelConfig(ADC1, ADC_Channel_Vrefint, 1, ADC_SampleTime_13Cycles5);
ADC_Cmd(ADC1, DISABLE);
ADC_TempSensorVrefintCmd(ENABLE);
delay_us(100);
ADC_Cmd(ADC1, ENABLE);

/* ADC1 calibration start */
ADC_ResetCalibration(ADC1);
while(ADC_GetResetCalibrationStatus(ADC1));
ADC_StartCalibration(ADC1);
while(ADC_GetCalibrationStatus(ADC1));
ADC_Cmd(ADC1, ENABLE);

ADC_RegularChannelConfig(ADC1, channel, 1, ADC_SampleTime_13Cycles5);
ADC_Cmd(ADC1, ENABLE);
return 0;
}


