/*
    AM2320 @ i2c2 driver

    Copyright (C) 2020 Jiri Pittner <jiri@pittnerovi.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdio.h>
#include <stdint.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "delay.h"
#include "am2320.h"

#define AM2320_ADDRESS 0x5C
#define AM2320_CMD_READREG    0x03
#define AM2320_REG_TEMP     0x02
#define AM2320_REG_HUM      0x00


//TODO - for some reason, irregularly an i2c timeout occurs 


uint16_t am2320_crc16(uint8_t *buffer, uint8_t nbytes) 
{
uint16_t crc = 0xffff;
uint8_t i;
for (i=0; i<nbytes; i++) 
  {
    uint8_t b = buffer[i];
    crc ^= b;
    uint8_t x;
    for (x=0; x<8; x++) 
      {
      if (crc & 0x0001) 
		{
		 crc >>= 1;
		 crc ^= 0xA001;
      		} 
      else 
 		crc >>= 1;
      }
  }
return crc;
}

void am2320_wakeup(void)
{
uint8_t tmp=0;
I2C2_WrBuf(AM2320_ADDRESS,&tmp,1);
}


uint16_t am2320_read(uint8_t reg)
{
am2320_wakeup();
delay_ms(1);

// send a command to read register
{
uint8_t tmp[3]={AM2320_CMD_READREG,reg,2};
I2C2_WrBuf(AM2320_ADDRESS,tmp,3);
delay_ms(1);  
}
  
//read 2 bytes preamble, 2 bytes data, 2 bytes CRC
uint8_t buffer[6];
I2C2_RdBuf(AM2320_ADDRESS,buffer,6); 

if (buffer[0] != 0x03)   return 0xFFFF; // must be 0x03 modbus reply
if (buffer[1] != 2)      return 0xFFFF; // must be 2 bytes reply
  
uint16_t the_crc = buffer[5];
the_crc <<= 8;
the_crc |= buffer[4];
uint16_t calc_crc = am2320_crc16(buffer, 4); 
if (the_crc != calc_crc) 
	{	
	printf("am2320 crc mismatch %d %d\n",the_crc,calc_crc);
	return 0xFFFF;
	}

uint16_t ret = buffer[2];
ret <<= 8;
ret |= buffer[3];

return ret;
}



float am2320_temperature(void)
{
uint16_t tmp = am2320_read(AM2320_REG_TEMP);
if(tmp==0xffff) return 0.f/0.f; 
float t = (tmp&0x8000) ? -(tmp&0x7fff) : tmp;
return 0.1f*t;
}

float am2320_humidity(void)
{
uint16_t tmp = am2320_read(AM2320_REG_HUM);
if(tmp==0xffff) return 0.f/0.f;
return 0.001f* tmp; //not percent
}


