#include <stdio.h>
#include "delay.h"
#include "avi_calibration.h"
#include "avionics.h"


float ee_magnetic_offsets[3] __attribute__ ((section(".eeprom")));
float ee_magnetic_scales[3]  __attribute__ ((section(".eeprom")));

float ee_magnetic_fit_matrix[FITVARS*FITVARS]  __attribute__ ((section(".eeprom")));
float ee_magnetic_fit_rhs[FITVARS]  __attribute__ ((section(".eeprom")));


float ee_omega_offset_x  __attribute__ ((section(".eeprom"))) = DEFAULT_OMEGA_OFFSET_X;
float ee_omega_offset_y __attribute__ ((section(".eeprom"))) = DEFAULT_OMEGA_OFFSET_Y;
float ee_omega_offset_z __attribute__ ((section(".eeprom"))) = DEFAULT_OMEGA_OFFSET_Z;

float omega_offset_x;
float omega_offset_y;
float omega_offset_z;

float ee_p_pitot_calibration_offset  __attribute__ ((section(".eeprom"))) = 0;
float ee_p_pitot2_calibration_offset  __attribute__ ((section(".eeprom"))) = 0;

float p_pitot_calibration_offset;
float p_pitot2_calibration_offset;

uint64_t eeprom_initialized  __attribute__ ((section(".eeprom")));

static uint8_t stored_eeprom_ok;


void store_magnetic_calibration(uint8_t eeprom_ok,uint8_t reset_it, float *magnetic_offsets, float *magnetic_scales, float *magnetic_fit_matrix, float *magnetic_fit_rhs)
{
if(reset_it)
	{
        for(int i=0; i<3; ++i)
                {
                magnetic_offsets[i]=0;
                magnetic_scales[i]=1;
                }
	for(int i=0; i<FITVARS*FITVARS; ++i) magnetic_fit_matrix[i]=0;
	for(int i=0; i<FITVARS; ++i) magnetic_fit_rhs[i]=0;
	}
if(eeprom_ok)
	{
        eepromWriteF(&ee_magnetic_offsets[0],magnetic_offsets[0]); delay_ms(10);
        eepromWriteF(&ee_magnetic_offsets[1],magnetic_offsets[1]); delay_ms(10);
        eepromWriteF(&ee_magnetic_offsets[2],magnetic_offsets[2]); delay_ms(10);
        eepromWriteF(&ee_magnetic_scales[0],magnetic_scales[0]); delay_ms(10);
        eepromWriteF(&ee_magnetic_scales[1],magnetic_scales[1]); delay_ms(10);
        eepromWriteF(&ee_magnetic_scales[2],magnetic_scales[2]); delay_ms(10);
	for(int i=0; i<FITVARS*FITVARS; ++i) {eepromWriteF(&ee_magnetic_fit_matrix[i],magnetic_fit_matrix[i]); delay_ms(10);}
	for(int i=0; i<FITVARS; ++i) {eepromWriteF(&ee_magnetic_fit_rhs[i],magnetic_fit_rhs[i]); delay_ms(10);}
	}
}

void read_magnetic_calibration(uint8_t eeprom_ok,float *magnetic_offsets, float *magnetic_scales, float *magnetic_fit_matrix, float *magnetic_fit_rhs)
{
if(!eeprom_ok) return;
for(int i=0; i<3; ++i)
                {
                magnetic_offsets[i]=eepromReadF(&ee_magnetic_offsets[i]);
                magnetic_scales[i]=eepromReadF(&ee_magnetic_scales[i]);
                }
for(int i=0; i<FITVARS*FITVARS; ++i) magnetic_fit_matrix[i]=eepromReadF(&ee_magnetic_fit_matrix[i]);
for(int i=0; i<FITVARS; ++i) magnetic_fit_rhs[i]=eepromReadF(&ee_magnetic_fit_rhs[i]);
}


void init_calibration(int eeprom_ok, int eeprom_reset, float *magnetic_offsets, float *magnetic_scales, float *magnetic_fit_matrix, float *magnetic_fit_rhs)
{
stored_eeprom_ok=eeprom_ok;

#if 0
//debug
while(1)
	{
	uint64_t x; x=eepromReadU64(&eeprom_initialized);
	printf("TEST %lx\n",x);
	delay_ms(1000);
	}
#endif

if(eeprom_ok)
    {
    if(eepromReadU64(&eeprom_initialized)!=MAGIC_INITIALIZED || eeprom_reset) //store factory defaults to eeprom
	{
	store_magnetic_calibration(eeprom_ok,1,magnetic_offsets,magnetic_scales, magnetic_fit_matrix, magnetic_fit_rhs);

	eepromWriteF(&ee_omega_offset_x,DEFAULT_OMEGA_OFFSET_X);
	delay_ms(10);
	eepromWriteF(&ee_omega_offset_y,DEFAULT_OMEGA_OFFSET_Y);
	delay_ms(10);
	eepromWriteF(&ee_omega_offset_z,DEFAULT_OMEGA_OFFSET_Z);
	delay_ms(10);

	eepromWriteF(&ee_p_pitot_calibration_offset,0);
        delay_ms(10);
	eepromWriteF(&ee_p_pitot2_calibration_offset,0);
        delay_ms(10);


	eepromWriteU64(&eeprom_initialized,MAGIC_INITIALIZED);
	delay_ms(10);
	printf("EEPROM was initialized to factory defaults\n");
	}
    }

if(!eeprom_ok || eepromReadU64(&eeprom_initialized)!=MAGIC_INITIALIZED) 
	{
	stored_eeprom_ok=0;
	printf("EEPROM FAILED, using standard calibration\n");

	for(int i=0; i<3; ++i)
                {
                magnetic_offsets[i]=0;
                magnetic_scales[i]=1;
                }

	omega_offset_x=DEFAULT_OMEGA_OFFSET_X;
	omega_offset_y=DEFAULT_OMEGA_OFFSET_Y;
	omega_offset_z=DEFAULT_OMEGA_OFFSET_Z;

	p_pitot_calibration_offset=0;
	p_pitot2_calibration_offset=0;
	}
else	//read calibration constants from eeprom
	{
	read_magnetic_calibration(eeprom_ok,magnetic_offsets,magnetic_scales, magnetic_fit_matrix, magnetic_fit_rhs);

	omega_offset_x=eepromReadF(&ee_omega_offset_x);
	omega_offset_y=eepromReadF(&ee_omega_offset_y);
	omega_offset_z=eepromReadF(&ee_omega_offset_z);

	p_pitot_calibration_offset=eepromReadF(&ee_p_pitot_calibration_offset);
	p_pitot2_calibration_offset=eepromReadF(&ee_p_pitot2_calibration_offset);

	printf("Calibration read from eeprom\n");
	}
}



static float omega_average_x,omega_average_y,omega_average_z;
static float p1_average,p2_average;
static uint32_t omega_average_n;

void init_omega_calibration(void)
{
omega_average_n=0;
omega_average_x=0;
omega_average_y=0;
omega_average_z=0;
p1_average=0;
p2_average=0;
}

void omega_calibration(float x, float y, float z)
{
++omega_average_n;
omega_average_x+= x;
omega_average_y+= y;
omega_average_z+= z;
float p0,p1,p2;
read_pressures(&p0,&p1,&p2);
p1_average+=p1;
p2_average+=p2;
}

void store_omega_calibration(uint8_t reset_it)
{
if(reset_it)
	{
        omega_offset_x=DEFAULT_OMEGA_OFFSET_X;
        omega_offset_y=DEFAULT_OMEGA_OFFSET_Y;
        omega_offset_z=DEFAULT_OMEGA_OFFSET_Z;
	p_pitot_calibration_offset=0;
	p_pitot2_calibration_offset=0;
	}
else
	{
	omega_offset_x = omega_average_x/omega_average_n;
	omega_offset_y = omega_average_y/omega_average_n;
	omega_offset_z = omega_average_z/omega_average_n;
	p_pitot_calibration_offset = p1_average/omega_average_n;
	p_pitot2_calibration_offset = p2_average/omega_average_n;
	}

printf("Omega calibration %f %f %f\n",omega_offset_x,omega_offset_y,omega_offset_z);
printf("Pitot calibration %f %f\n",p_pitot_calibration_offset,p_pitot2_calibration_offset);

if(stored_eeprom_ok)
	{
	eepromWriteF(&ee_omega_offset_x,omega_offset_x);
	delay_ms(10);
	eepromWriteF(&ee_omega_offset_y,omega_offset_y);
	delay_ms(10);
	eepromWriteF(&ee_omega_offset_z,omega_offset_z);
	delay_ms(10);
	eepromWriteF(&ee_p_pitot_calibration_offset,p_pitot_calibration_offset);
	delay_ms(10);
	eepromWriteF(&ee_p_pitot2_calibration_offset,p_pitot2_calibration_offset);
        delay_ms(10);

	printf("Omega calibration stored to eeprom\n");
	}
}

void print_calibration(const  float *magnetic_offsets, const float *magnetic_scales)
{
if(!stored_eeprom_ok) {printf("EEPROM error\n"); return;}

printf("Magnetic offsets %f %f %f\n",magnetic_offsets[0],magnetic_offsets[1],magnetic_offsets[2]);
printf("Magnetic scales %f %f %f\n",magnetic_scales[0],magnetic_scales[1],magnetic_scales[2]);
printf("Omega calibration %f %f %f\n",omega_offset_x,omega_offset_y,omega_offset_z);
}
