#ifndef _AVI_GUI_H
#define _AVI_GUI_H

#include <float.h>
#include "bitmap.h"

//for LCD
#include "easyT6963-master/T6963_STM32portD.h"
#include "easyT6963-master/easyT6963.h"
//#include "easyT6963-master/fonts/Imagine_FontFixed__6.h"

#define LCD_X 240
#define LCD_Y 128
#define TEXTFONT_X 6
#define TEXTFONT_Y 8

extern T6963_STM32 *mylcd;

extern bool displaymode_changed;

#define N_DISPLAY_MODE 5
typedef enum {avitext=0,gui_mode=1,magnet_cal=2,gyro_cal=3,can_debug=4} DISPLAY_MODE;

//GUI classes

//abstract base class
typedef class widget
	{
public:
	uint16_t x;
	uint16_t y;
	uint8_t sizex;
	uint8_t sizey;
	virtual void draw(bool firsttime) =0;
	virtual void activate(bool on) {};
	virtual void process_press(uint16_t ms) {};
	virtual void process_rotation(uint8_t dir) {};

	widget(uint16_t x0,uint16_t y0,  uint8_t sizex0, uint8_t sizey0)
		: x(x0), y(y0), sizex(sizex0),sizey(sizey0)
		{};
	}
WIDGET;


typedef class textwidget : public widget
	{
protected:
	const char *text;
	volatile float *number;
	float scale;
public:
	textwidget(uint16_t x0,uint16_t y0,  uint8_t sizex0, uint8_t sizey0,  const char *text0, volatile float *number0, const float scale0=1.f);
	void draw(bool firsttime);
	}
TEXTWIDGET;

	
typedef class tunable_textwidget : public textwidget
	{
	const float step;
	const float defaultv;
	const float mini;
	const float maxi;
	const bool multiplicative;
	volatile bool *changed;
public:
	tunable_textwidget(uint16_t x0,uint16_t y0,  uint8_t sizex0, uint8_t sizey0,  const char *text0, volatile float *number0, const float scale0=1.f, const float step0=1.f, const float defaultv0=0.f, const float mini0= -FLT_MAX, const float maxi0= FLT_MAX,  volatile bool *changed0=NULL, const bool multiplicative0=false);
	void process_press(uint16_t ms);
	void process_rotation(uint8_t dir);
	void activate(bool on);
	}
TUNABLE_TEXTWIDGET;

//sideslip indicator
typedef class ballwidget : public widget
        {
protected:
	BITMAP map;
	float *y_acceleration;
	float sensitivity; //defined as acceleration yielding full deflection
public:
	void draw(bool firsttime);
	ballwidget(uint16_t x0,uint16_t y0,  uint8_t sizex0, uint8_t sizey0, float *y_acceleration0, float sensitivity0);
	}
BALLWIDGET;


//artificial horizon
typedef class horizonwidget : public widget
        {
protected:
	BITMAP map;
	float *pitch;
	float *roll;
	float pitchscale; //in pixel per degree
public:
        void draw(bool firsttime);
	horizonwidget(uint16_t x0,uint16_t y0,  uint8_t sizex0, uint8_t sizey0, float *pitch0, float *roll0, float pitchscale0);
	}
HORIZONWIDGET;



//non-member global variables and functions

extern volatile  DISPLAY_MODE display_mode;

extern void init_gui(T6963_STM32 *lcd, WIDGET **gui0);
extern void update_gui(T6963_STM32 *lcd, bool firsttime);
extern void process_touch(float x, float y);

extern void xyplot(float x, float y, float scale);
#endif


