/*
 * *      avionics.cpp  ... firmware for a simple experimental avionics
 * *
 * *      Copyright (C) 2019-2020 Jiri Pittner <jiri@pittnerovi.com>
 * *
 * *      This program is free software: you can redistribute it and/or modify
 * *      it under the terms of the GNU General Public License as published by
 * *      the Free Software Foundation, either version 3 of the License, or
 * *      (at your option) any later version.
 * *
 * *      This program is distributed in the hope that it will be useful,
 * *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 * *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * *      GNU General Public License for more details.
 * *      You should have received a copy of the GNU General Public License
 * *      along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * *
 * *
 * */

//precist log z magnet kalibrace, proc byla tak velka scale y, kdyz to vypadalo jako kruh!
//@@@@@vylepsit purified gravity
//@@@cat vytvoril neexistujici soubor - problem openu na rw asi
//@@@option use  gps smer misto magnetu a zadanou deklinaci
//general approximations employed: earth-frame is considered inertial and gravity dependence  on altitude is neglected
//@@@zpozdovaci linka na acceler[0] ???
//@@@@@@rotate GPS speed from x and z direction in body axes to earth axes and compare with velocity from coordinate differences and altitude differences
//@@@@@@@@@zanedbame li sideslip, uhlovy rozdil bude mezi magnetickym a geogr  azimutem
//@@@@pozor earth z vertikalni rychlost bude obecne transf v body systemu
//@@@@@@@@gravitacni low_passed acceleration x component average over 5 seconds or so for the purpose of purified gravity
//@@@@@@wind speed and azimuth estimation
//@@@@@@@@@@@@@software AOA estimation
//@@@@@gps kurz je vlastne tratovy uhel vcetne snosu
//
//@@@@@@!!!!!! implement accelerometer and magnetometer temporary rejection as in https://github.com/xioTechnologies/Fusion
//@@@watchdog properly
//@@@CAN  interface pridat hw i sw
/*
 Acceleration rejection @@@@

The acceleration rejection feature reduces the errors that result from the accelerations of linear and rotational motion. Acceleration rejection works by comparing the instantaneous measurement of inclination provided by the accelerometer with the current measurement of inclination of the algorithm output. If the angular difference between these two inclinations is greater than a threshold then the accelerometer will be ignored for this algorithm update. This is equivalent to a dynamic gain that deceases as accelerations increase.

Prolonged accelerations from linear and rotational motion may result in the accelerometer being unusable as a measurement of inclination. This is detected by the algorithm as an acceleration rejection timeout. An acceleration rejection timeout occurs when the number of algorithm updates that have ignored the accelerometer exceeds ten times the number that have used the accelerometer within a defined period. If an acceleration rejection timeout occurs then the algorithm will reinitialise.
Magnetic rejection

The magnetic rejection feature reduces the errors that result from temporary magnetic distortions. Magnetic rejection works using the same principle as acceleration rejection operating on the magnetometer instead of the accelerometer and by comparing the measurements of heading instead of inclination. A magnetic rejection timeout will not cause the algorithm to reinitialise. If a magnetic rejection timeout occurs then the heading of the algorithm output will be set to the instantaneous measurement of heading provided by the magnetometer.

 */

//TODO hardware drivers: implement digital pressure sensor digital, bluetooth,  CAN bus (vyvest konektor)

#define GPS_HEADING_THR 3.f
#define WMM_MODULO 255
//sample every .06s and recalibrate about each  minute, only slowly filtering out old data
#define MAGNETIC_RECALIBRATION_ALPHA 0.95
#define MAGNETIC_RECALIBRATION_MODULO 0x07

#include <stdio.h>
#include <math.h>
#include <stdint.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <setjmp.h>
#include <errno.h>
#include <sys/time.h>


#include "usart123.h"
#include "delay.h"
#include "stm32f37x_gpio.h"
#include "stm32f37x_rcc.h"
#include "stm32f37x_pwr.h"
#include "stm32f37x_tim.h"
#include "stm32f37x_exti.h"
#include "stm32f37x_dma.h"
#include "stm32f37x_adc.h"
#include "stm32f37x_dac.h"
#include "stm32f37x_misc.h"
#include "stm32f37x_syscfg.h"

#include "i2c1.h"
#include "mcp24aa.h"
#include "i2c2.h"
#include "am2320.h"
#include "avi_calibration.h"
#include "magnetometer.h"

#include "rtc.h"
#include "watchdog.h"

#include "touchscreen.h"

#include "imu_sensors.h"
#include "pressure_sensors.h"
//#include "ahrs1.h"
#include "ahrs2.h"
#include "nmea.h"
#include "wmm.h"
#include "avi_gui.h"
#include "sdcard.h"
#include "fat_interface.h"


#ifdef WITH_CAN
#include "can.h"
#include "can_display.h"
#endif


#ifdef WITH_USB
#include "usb/inc/ttyacm.h"
#include "usb/inc/usbsd.h"
#endif

//flight parameters
float temperature=15.f;
float humidity=0.4f; //decimal, not in %
float density_altitude;
float altitude=0,ias=0,aoa=0,tas=0;
volatile float vario=0; //from pressure
float vario_lowpassed=0;
volatile float oldaltitude= 0;
volatile static uint8_t vario_novice=10;

#define M2FEET 3.28084f

#define VARIO_DT 2047
#define vario_alpha 0.5f

void update_vario(void)
{
float newvario=0;
if(!vario_novice) 
	{
	newvario=(altitude-oldaltitude)/(.001f*VARIO_DT);
	vario *= vario_alpha;
	vario += newvario*(1.f-vario_alpha);
	}
if(vario_novice) --vario_novice;
oldaltitude=altitude;
}



static volatile uint64_t milliseconds=0;

extern "C" { //interrupt handler must have C-linkage!
void TIM19_IRQHandler(void) __attribute__((optimize("-O1")));
void TIM19_IRQHandler(void) //this is called every millisecond to update counter
{
TIM_ClearITPendingBit(TIM19,TIM_IT_Update);
++milliseconds;
if((milliseconds&(VARIO_DT-1))==0) update_vario();
}
}


void init_mstimer(void)
{
RCC_APB2PeriphClockCmd(RCC_APB2Periph_TIM19, ENABLE);
RCC_APB2PeriphResetCmd(RCC_APB2Periph_TIM19, ENABLE);
RCC_APB2PeriphResetCmd(RCC_APB2Periph_TIM19, DISABLE);
TIM_TimeBaseInitTypeDef TIM_TimeBaseInitStructure;
TIM_TimeBaseStructInit(&TIM_TimeBaseInitStructure);
TIM_TimeBaseInitStructure.TIM_Prescaler=2;
TIM_TimeBaseInitStructure.TIM_Period = (XTAL*PLLMULT)/2/1000;
TIM_TimeBaseInitStructure.TIM_CounterMode = TIM_CounterMode_Up;
TIM_TimeBaseInit(TIM19,&TIM_TimeBaseInitStructure);
TIM_InternalClockConfig(TIM19);
TIM19->CNT=0;
TIM_ITConfig(TIM19, TIM_IT_Update, ENABLE);
TIM_Cmd(TIM19,ENABLE);


NVIC_InitTypeDef NVIC_InitStructure;
NVIC_InitStructure.NVIC_IRQChannel = TIM19_IRQn;
NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
NVIC_Init(&NVIC_InitStructure);
}


//leds - PF2,9,10, beeper PF4
void init_led_beeper(void)
{
GPIO_InitTypeDef GPIO_InitStructure;
RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOF, ENABLE);
GPIO_InitStructure.GPIO_Pin =  GPIO_Pin_2|GPIO_Pin_4|GPIO_Pin_9|GPIO_Pin_10;
GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
GPIO_Init(GPIOF, &GPIO_InitStructure);
cbi(GPIOF,2);
cbi(GPIOF,4);
cbi(GPIOF,9);
cbi(GPIOF,10);
}




void led(uint8_t which, uint8_t onoff) //which = 0-2 
{
uint8_t pin=which?8+which:2;
if(onoff) sbi(GPIOF,pin); else cbi(GPIOF,pin);
}

void beeper(uint8_t onoff) 
{
if(onoff) sbi(GPIOF,4); else cbi(GPIOF,4);
}



/*
void oldquat2ypr(const Quaternion<float> &q, Vec3<float> &ypr)
{
float s=2*q[1]*q[2]-2.f*q[0]*q[3];
float c=2*q[0]*q[0]+2*q[1]*q[1]-1.f;
ypr[0]= 180.f/M_PI*atan2f(s,c);
if(ypr[0]<0.) ypr[0]+= 360.;
float a= -180.f/M_PI*asinf(2*q[1]*q[3]+2.f*q[0]*q[2]);
float b= 180.f/M_PI*atan2f(2*q[2]*q[3]-2.f*q[0]*q[1],2*q[0]*q[0]+2*q[3]*q[3]-1.f);
ypr[1]= -c*a-s*b;
ypr[2]= s*a-c*b;
}
*/



//quaternion to yaw pitch roll in avionics convention
void quat2ypr(const Quaternion<float> &q, Vec3<float> &ypr)
{
q.conjugate().normquat2eulerzyx(ypr); //our orientation is defined inversely
ypr *= 180.f/M_PI;
if(ypr[0]<0) ypr[0]+=360.f; 
}

//and inverse to this
void ypr2quat(Quaternion<float> &q, const Vec3<float> &ypr)
{
Vec3<float> ypr2 = ypr*M_PI/180.f;
q.euler2normquat(ypr2,"zyx");
q.conjugateme();
}

void usb_connect(uint8_t onoff)
{
GPIO_InitTypeDef GPIO_InitStructure;
RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOB, ENABLE);
GPIO_InitStructure.GPIO_Pin =  GPIO_Pin_14;
GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
GPIO_Init(GPIOB, &GPIO_InitStructure);
if(onoff) cbi(GPIOB,14);
else sbi(GPIOB,14);
}



#define LOGPASS 63
#define LCDPASS 3
#define TEMP_PASS 137

//exact zeros indicate that we have to compute them as soon as possible
float declination=0;
float inclination=0;

FILE *logfile=NULL;


#define MAGIC_INITIALIZED 0x3141592653589793ULL
static uint64_t reboot_type_initialized  __attribute__ ((section(".persistent")));
static uint8_t reboot_type_requested __attribute__ ((section(".persistent")));


int main(void)
{
float continuous_magnetic_recalibration=0.f; //float in order to be applicable in a tunable widget; default off
DISPLAY_MODE old_display_mode=avitext;
displaymode_changed=true;
init_led_beeper();

FILE *us1=fopen("usart 1 115200 1","w+"); //with echo!
setlinebuf(us1);
stdout=stderr=us1;
printf("Reset!\n");

if(reboot_type_initialized==MAGIC_INITIALIZED)
        {
        reboot_type_initialized=0;//for next time
        printf("Targeted reboot %d\n",reboot_type_requested);
        }
else
        {
        reboot_type_requested=0;
        }



bool rtc_was_set=false;
rtc_init();

bool mass_storage=false;

//check for button2 pressed while power on to switch to mass storage
#ifdef WITH_USB
{
RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOC, ENABLE);
GPIO_InitTypeDef GPIO_InitStructure;
GPIO_InitStructure.GPIO_Pin = GPIO_Pin_13;
GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
GPIO_Init(GPIOC, &GPIO_InitStructure);
if(reboot_type_requested==0 && bit_is_clear(GPIOC,13)||reboot_type_requested==2)
	{
	printf("enable USB MASS storage\n");
	led(0,1);
	mass_storage=true;
	delay_ms(300);
	}
else	printf("enable USB terminal\n");
}

usb_connect(0);
delay_ms(500);
usb_connect(1);

FILE *usb=NULL;
if(!mass_storage) 
	{
	usb=fopen("usb 1","w+");
	if(usb)
	        {
	        setlinebuf(usb);
	        printf("USB TTYACM Init success!\n");
	        //does not mean it is connected to host yet!
	        }
	else    printf("USB TTYACM Init failed!\n");
	}
#endif

FILE *gps=NULL;
gps=fopen("usart 2 9600 0","w+"); //no echo!
setlinebuf(gps);
printf("GPS usart initialized\n");
//speed can be 38400 by hardware jumper on the module, but it is not critical

FILE *bt=NULL;
bt=fopen("usart 3 9600 0","w+"); //no echo!
setlinebuf(bt);
//@@@has to toggle on an enable pin to work!
printf("BlueTooth usart initialized\n");





//init SD card
{
uint8_t i;
for(i=0; i<4; ++i) //card might need some time after powerup
	{
	delay_ms(100);
	SDHC_Init();
	if(!SD_error) break;
	}
}
printf("SD init error=%d\n",SD_error);

//initialize LCD 
T6963_STM32 lcd;


bool FAT_mounted=false;
if(!SD_error) 
{
#ifdef WITH_USB
if(mass_storage)
	{
	//init the button for exiting this mode
	GPIO_InitTypeDef GPIO_InitStructure;
        RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOE, ENABLE);
        GPIO_InitStructure.GPIO_Pin = GPIO_Pin_5|GPIO_Pin_6;
        GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
        GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
        GPIO_Init(GPIOE, &GPIO_InitStructure);

	uint8_t r=usbsd_init(1);
	printf("usbd_init returned %d\n",r);
	init_gui(&lcd,NULL);
	if(r==0)
		{
		lcd.setCursor(0,5); lcd.write("USB Mass Storage mode entered...");
		while(1){watchdog(); delay_ms(1000);  if(bit_is_clear(GPIOE,5)) softreset(); }
		}
	else
		{
		lcd.setCursor(0,5); lcd.write("USB Mass Storage mode FAILED!");
		delay_ms(3000);
		}
	}
else
#endif
	{
	//initialize FAT
	fat_mount();  
	FAT_mounted=true;
	}
}

//init I2C
I2C_LowLevel_Init();
I2C2_LowLevel_Init();

//init eeprom and magnetic calibration
init_magnetic_calibration();
init_calibration(HAS_EEPROM,0,magnetic_offsets,magnetic_scales,&magnetic_fit.fitmat[0][0],&magnetic_fit.rhsmat[0][0]);

volatile bool instrument_orientation_changed=0;
volatile float instrument_orientation_ypr[3]={0,0,0};
volatile float beta;
volatile float beta2;


//GPS data
Vec3<float> previous_gps_position(0,0,0), gps_position(0,0,0), gps_velocity(0,0,0);
Vec3<float> gps_acceleration_lowpassed(0,0,0), gps_acceleration(0,0,0);
float gps_vario=0; //from gps
float gps_vario_lowpassed=0;
uint32_t gps_valid_sequence=0;
time_t gps_time=0;
time_t previous_gps_time=0;
float ground_speed=0.;
float previous_ground_speed=0.;
float gps_heading=0;
float gps_heading_magnetic=0;
int nsat=0;
float accuracy=0;
time_t last_fix=0;

Vec3<float> gravity_reference(0,0,-1);
Vec3<float> magnetic_reference(1,0,0); //assuming inclination will be filtered out by magnetometer_corrected() since calibration is done in more or less horizontal positions

float low_passed_omega2;
Vec3<float> low_passed_acceleration,low_passed_omega,low_passed_magnet,mraw;
Vec3<float>  low_passed_purified_gravity(0.,0.,-9.81f);
Vec3<float> acceleration,omega;
float sensortime,sensor_t0,dt;

//init IMU sensors
init_9dof();
printf("IMU sensor init\n");

//init pressure sensors
init_pressure();
printf("pressure sensor init\n");

init_mstimer();
printf("millisecond timer initialized\n");

//adjustable correction for positioning of the device
Quaternion<float> instrument_orientation(1,0,0,0);


//initial sensor readout
{
Vec3<float> a,o,m0;
read_9dof(a,o,m0, &sensor_t0, NULL,NULL);
Vec3<float> m=magnetometer_corrected(m0);
o[0]-= omega_offset_x;
o[1]-= omega_offset_y;
o[2]-= omega_offset_z;
instrument_orientation.rotate(low_passed_acceleration,a);
instrument_orientation.rotate(low_passed_omega,o);
low_passed_omega2=low_passed_omega[2];
instrument_orientation.rotate(low_passed_magnet,m);
}

float g_norm=low_passed_acceleration.norm(); if(low_passed_acceleration[2]>0) g_norm= -g_norm; //sign opposite than the z-component


//
const float alpha_a=0.97f; //alpha=0 .. .no lowpassing
const float alpha_o=0.9f; //alpha=0 .. .no lowpassing
const float alpha_o2=0.97f; //alpha=0 .. .no lowpassing
const float alpha_m=0.8f; //alpha=0 .. .no lowpassing
const float alpha_g=0.5f; //gps accel
const float alpha_gv=0.9f; //gps vario
const float alpha_pg=0.97f; //purified gravity in body frame
float betaenhance=0.1f;
float betadecay=0.9995f;
beta=0.0025f; //fusion parameter, smaller value delays the correction of gyroscope by gravity and magnetic references; 0.001 is too low to compensate gyro drift if omega is not very well calibrated
beta2=0.03f;

char rtc_text[24];

Vec3<float> magnet; 
Quaternion<float> orientation(1.f);
Quaternion<float> old_orientation(1.f);
Quaternion<float> orientation_derivative;

float qnh=101300.f;
Vec3<float>ypr;


//prepare GUI widgets
//@@@est. wind speed and velocity
//@@@AOA and software-AOA  and estimated windspeed and direction left
//@@@logovat (debug screen) tyto udaje

//text display widgets
TEXTWIDGET OmegaZ(18,24,24,16,"zROT\n%5.0f",&low_passed_omega2,180.f/M_PI*60.f);

TEXTWIDGET Mag_hdg(72,0,24,16,"mHDG\n%03.0f",&ypr[0]);
TEXTWIDGET Gps_Mag_hdg(102,0,24,16,"MHDG\n %03.0f",&gps_heading_magnetic);
TEXTWIDGET Gps_hdg(138,0,24,16,"gHDG\n %03.0f",&gps_heading);

TEXTWIDGET Ias(54,24,18,16,"IAS\n%3.0f",&ias,3.6f);
TEXTWIDGET Tas(54,48,18,16,"TAS\n%3.0f",&tas,3.6f);
TEXTWIDGET Ground_speed(54,72,18,16," GS\n%3.0f",&ground_speed,3.6f);
TEXTWIDGET Gforce(48,96,18,16," G\n%4.1f",&g_norm,1.f/9.80665f);

TEXTWIDGET Paltitude(168,16,30,16,"p-ALT\n%5.0f",&altitude,M2FEET);
TEXTWIDGET Galtitude(168,40,30,16,"g-ALT\n%5.0f",&gps_position[2],M2FEET);
TEXTWIDGET    Pvario(168,64,30,16,"p-VAR\n%+5.0f",&vario_lowpassed,M2FEET*60.f);
TEXTWIDGET    Gvario(168,88,30,16,"g-VAR\n%+5.0f",&gps_vario_lowpassed,M2FEET*60.f);

TEXTWIDGET Rtc(0,0,66,16,rtc_text,NULL);
TEXTWIDGET Lattitude(172,0,72,1,"GPS%9.5f",&gps_position[0]);
TEXTWIDGET Longitude(184,8,60,1,"%10.5f",&gps_position[1]);

TEXTWIDGET Dalt(210,24,30,16,"DALT\n%.0f ",&density_altitude,M2FEET);
TEXTWIDGET Temp(210,48,30,16,"Temp\n%.1f ",&temperature);
TEXTWIDGET Humid(210,72,30,16," RH\n %.0f %",&humidity,100.f);

//debug and technical info
TEXTWIDGET Dt(198,112,42,16," dt(ms)\n%7.2f",&dt,1000.f);
TUNABLE_TEXTWIDGET Mag_recal(0,96,24,16,"MCAL\n%-4.0f",&continuous_magnetic_recalibration,1.f,100.f,1000.f,0.f);

//tunable parameters
TUNABLE_TEXTWIDGET Beta(0,112,36,16,"beta\n%6.4f",&beta,1.f,.0001f,.001f,0.f);
TUNABLE_TEXTWIDGET Offset_Y(36,112,30,16,"Y-OFF\n%5.1f",&instrument_orientation_ypr[0],180.f/M_PI,.1*M_PI/180.f,0.f,-180.f,180.f,&instrument_orientation_changed);
TUNABLE_TEXTWIDGET Qnh(168,112,24,16,"QNH\n%-4.0f",&qnh,0.01f,100.f,101300.f,0.f);

//graphical widgets
HORIZONWIDGET Horizon(75,18,90,90,&ypr[1],&ypr[2],1.f);
BALLWIDGET Ball(75,111,90,17,&low_passed_acceleration[1],2.f);

WIDGET *(widgets[]) =
{
&Horizon,
&Ball,
&OmegaZ,
&Mag_hdg,
&Gps_Mag_hdg,
&Gps_hdg,
&Ias,
&Tas,
&Ground_speed,
&Paltitude,
&Galtitude,
&Pvario,
&Gvario,
&Rtc,
&Dalt,
&Temp,
&Humid,
&Lattitude,
&Longitude,
&Qnh,
&Offset_Y,
&Beta,
&Dt,
&Mag_recal,
&Gforce,
NULL //sentinel
};

//setup the whole GUI
init_gui(&lcd, widgets);


//open logfile
if(FAT_mounted)
	{
	char name[64];
	struct timeval tv;
	gettimeofday(&tv,NULL);
	struct tm *tmp;
        tmp = gmtime(&tv.tv_sec);
	strftime(name,64,"/%Y%m%d_%H%M.log",tmp);
	logfile=fopen(name,"w");
	if(logfile) printf("logifle opened\n"); 
	fputs("Avionics logfile\n",logfile); fflush(logfile);
	}

#ifdef WITH_CAN
//init CAN              
{               
uint8_t r=init_can(500000,1);
if(r) printf("init_can error %d\n",r);
else printf("init_can success\n");
}                       
#endif


//MAIN LOOP
uint32_t gps_timeout=0;
uint32_t pass=0;
printf("entering main loop\n");
while(1) {

//check actions from GUI
if(instrument_orientation_changed)
	{
	instrument_orientation.euler2normquat((const float *)instrument_orientation_ypr,"zyx");
	instrument_orientation_changed=false;
	}

//periodic sensor readout

{
Vec3<float> a,o;
read_9dof(a,o,mraw, &sensortime, &sensor_t0, &dt);
if((pass&MAGNETIC_RECALIBRATION_MODULO)==0)
	{
	if(display_mode==magnet_cal||(int)continuous_magnetic_recalibration) accumulate_magnetic_calibration(mraw,(display_mode==magnet_cal?0:(int)continuous_magnetic_recalibration),HAS_EEPROM,MAGNETIC_RECALIBRATION_ALPHA);
	}
if(display_mode==gyro_cal) omega_calibration(o[0],o[1],o[2]);

//apply magnetometer correction here
Vec3<float> m=magnetometer_corrected(mraw);

//apply omega offset corrections
o[0]-= omega_offset_x;
o[1]-= omega_offset_y;
o[2]-= omega_offset_z;

//adjust for instrument orientation
instrument_orientation.rotate(acceleration,a);
instrument_orientation.rotate(magnet,m);
instrument_orientation.rotate(omega,o);
}

//filter high freq noise
low_passed_acceleration *= alpha_a;
low_passed_acceleration += acceleration * (1.f-alpha_a);
g_norm=low_passed_acceleration.norm(); if(low_passed_acceleration[2]>0) g_norm= -g_norm; //sign opposite than the z-component
low_passed_omega *= alpha_o;
low_passed_omega += omega * (1.f-alpha_o);
low_passed_omega2 *= alpha_o2;
low_passed_omega2 += omega[2] * (1.f-alpha_o2);
low_passed_magnet *= alpha_m;
low_passed_magnet += magnet * (1.f-alpha_m);


Vec3<float> purified_gravity = low_passed_acceleration - gps_acceleration_lowpassed - low_passed_omega*gps_velocity;
low_passed_purified_gravity *= alpha_pg;
low_passed_purified_gravity += purified_gravity * (1.f-alpha_pg);

//@@@@@@compute GPS velocity and acceleration in body frame (noise filtering!)
//@@@@@actually too slow to capture acceleration
//@@@@@@@@@@@@@compute acceleration as in android as dif from long term acc average, in x direction only and this subtract!
gps_velocity[0]=ground_speed;

//controlled effectivity of the gradient step @@@check also denormalized grad corr
float betaeff=beta+betaenhance;
float beta2eff=beta2+betaenhance;
//during non-inertial movements prefer the gyroscope over accelerometers

//@@@@@use long-term average of gps_velocity,gps_acceleration,low_passed_acceleration,
//use the crossproduct for magnetic reference, so that it influences only yaw, not pitch and roll
//this is important since z-component of magnetic field fluctuates in car/plane environemnt due to EM interference
AHRSupdate(orientation,dt,betaeff,beta2eff,gps_velocity, gps_acceleration_lowpassed, low_passed_omega, low_passed_acceleration, low_passed_magnet, gravity_reference, magnetic_reference,true,true,true,&low_passed_purified_gravity);
//the original formulation  is now incompatible with our axes orientation/reference
//MadgwickAHRSupdateIMU(magorientation,dt,beta+betaenhance,gps_velocity, gps_acceleration_lowpassed,omega,low_passed_acceleration);
//MadgwickAHRSupdate(magorientation,dt,beta+betaenhance,gps_velocity, gps_acceleration_lowpassed, omega,low_passed_acceleration,magnet);

betaenhance *= betadecay; //after power on use larger beta to quickly find the right orientation from poor the guess of azimuth, the gradually lower beta

//clear display after layout change
if(display_mode!=old_display_mode)
	{
	old_display_mode=display_mode;
	lcd.clearText();
	lcd.clearGraphic();
	displaymode_changed=true;
	}

//lowpass vario
const float beta_vario=0.8;
vario_lowpassed *= beta_vario;
vario_lowpassed += vario * (1.f-beta_vario);

//PRINT to logfile and to debug screen
{
char z[100];
int n;
sprintf(z,"time %11ld sensor %7.1f %8.5f\n",milliseconds,sensortime,dt);
orientation_derivative = (orientation-old_orientation)/dt;
old_orientation=orientation;
if((pass&LCDPASS)==0&&display_mode==avitext) {lcd.setCursor(0,0); lcd.write(z);}
if(FAT_mounted&&logfile&&(pass&LOGPASS)==0) fputs(z,logfile);
sprintf(z,"gpsacc %5.2f gps GS  %5.2f\n",gps_acceleration_lowpassed[0],ground_speed*3.6f);
if((pass&LCDPASS)==0&&display_mode==avitext) {lcd.setCursor(0,1); lcd.write(z);}
if(FAT_mounted&&logfile&&(pass&LOGPASS)==0) fputs(z,logfile);
n=low_passed_acceleration.sprintf(z,"accel %6.2f  %6.2f %6.2f");
sprintf(z+n," norm %6.2f\n",g_norm);
if((pass&LCDPASS)==0&&display_mode==avitext) {lcd.setCursor(0,2); lcd.write(z);}
if(FAT_mounted&&logfile&&(pass&LOGPASS)==0) fputs(z,logfile);
n=low_passed_purified_gravity.sprintf(z,"GRAV  %6.2f  %6.2f %6.2f");
sprintf(z+n," norm %6.2f\n",low_passed_purified_gravity.norm());
if((pass&LCDPASS)==0&&display_mode==avitext) {lcd.setCursor(0,3); lcd.write(z);}
if(FAT_mounted&&logfile&&(pass&LOGPASS)==0) fputs(z,logfile);
sprintf(z,"h=%7.1fft%+5.0ffpm QNH=%4.0f IAS=%3.0fkm/h\n",altitude*M2FEET,vario*M2FEET*60.f, qnh*0.01f,ias*3.6f);
if((pass&LCDPASS)==0&&display_mode==avitext) {lcd.setCursor(0,4); lcd.write(z);}
if(FAT_mounted&&logfile&&(pass&LOGPASS)==0) fputs(z,logfile);
  {
                        struct timeval tv;
                        gettimeofday(&tv,NULL);
                        struct tm *tmp;
                        tmp = gmtime(&tv.tv_sec);
                        strftime(z,64," RTC=%Y.%m.%d_%H:%M:%S UTC\n",tmp);
			strftime(rtc_text,24,"%Y.%m.%d\n%H:%M:%SU",tmp);
  }
if((pass&LCDPASS)==0&&display_mode==avitext) {lcd.setCursor(0,5); lcd.write(z);}
if(FAT_mounted&&logfile&&(pass&LOGPASS)==0) fputs(z,logfile);
n=low_passed_omega.sprintf(z,"omega  %7.3f %7.3f %7.3f");
sprintf(z+n," N %6.3f\n",low_passed_omega.norm());
if((pass&LCDPASS)==0&&display_mode==avitext) {lcd.setCursor(0,6); lcd.write(z);}
if(FAT_mounted&&logfile&&(pass&LOGPASS)==0) fputs(z,logfile);
low_passed_magnet.sprintf(z,"magnet %7.3f %7.3f %7.3f\n");
if(FAT_mounted&&logfile&&(pass&LOGPASS)==0) fputs(z,logfile);
if((pass&LCDPASS)==0&&display_mode==avitext) {lcd.setCursor(0,7); lcd.write(z);}
if((pass&LCDPASS)==0&&display_mode==magnet_cal) 
	{
	lcd.setCursor(4,12); lcd.write(z);
	float a= -180.f/M_PI*atan2(low_passed_magnet[1],low_passed_magnet[0]);
	if(a<0) a+=360.f;
	sprintf(z,"    Direction  = %3.0f  YAW= %3.0f ",a,ypr[0]);
	lcd.setCursor(0,14); lcd.write(z);
	mraw.sprintf(z,"RAW magnet %7.3f %7.3f %7.3f \n");
	lcd.setCursor(0,13); lcd.write(z);
        a= -180.f/M_PI*atan2(mraw[1],mraw[0]);
        if(a<0) a+=360.f;
        sprintf(z,"RAW Direction  = %3.0f  GPS= %3.0f ",a,gps_heading);
        lcd.setCursor(0,15); lcd.write(z);
	}
sprintf(z,"OFFSET %6.1f %6.1f %6.1f\n",instrument_orientation_ypr[0]*180.f/M_PI,instrument_orientation_ypr[1]*180.f/M_PI,instrument_orientation_ypr[2]*180.f/M_PI);
if((pass&LCDPASS)==0&&display_mode==avitext) {lcd.setCursor(0,8); lcd.write(z);}
if(FAT_mounted&&logfile&&(pass&LOGPASS)==0) fputs(z,logfile);
quat2ypr(orientation,ypr);
#if 0
{
//debug
Quaternion<float> orientation2;
ypr2quat(orientation2,ypr);
printf("DEBUG quat2ypr inverse %f\n",(orientation-orientation2).norm());
}
#endif
ypr.sprintf(z,"Yaw %6.1f Pitch %6.1f Roll %6.1f\n");
if((pass&LCDPASS)==0&&display_mode==avitext) {lcd.setCursor(0,10); lcd.write(z);}
if(FAT_mounted&&logfile&&(pass&LOGPASS)==0) fputs(z,logfile);
sprintf(z,"HDG %6.1f M%6.1f BETA %6.3f BETA2 %6.3f\n",gps_heading,gps_heading_magnetic,betaeff,beta2);
if((pass&LCDPASS)==0&&display_mode==avitext) {lcd.setCursor(0,12); lcd.write(z);}
if(FAT_mounted&&logfile&&(pass&LOGPASS)==0) fputs(z,logfile);

//gps_time=1580633311+(pass>>7); //debug gps
if(gps_time) //have GPS data
	{
	struct tm *tmp;
        tmp = gmtime(&gps_time);
	if(!rtc_was_set)
		{
		rtc_settime(tmp);
		rtc_was_set=true;
		}
        char times[64];
        strftime(times,64,"%c %Z",tmp);
	sprintf(z,"GPS%c %s\n",(gps_time==last_fix?'=':'!'),times);
	if((pass&LCDPASS)==0&&display_mode==avitext) {lcd.setCursor(0,13); lcd.write(z);}
if(FAT_mounted&&logfile&&(pass&LOGPASS)==0) fputs(z,logfile);
	sprintf(z,"FIX %.5f %.5f ALT %.1fm\n",gps_position[0],gps_position[1],gps_position[2]);
	if((pass&LCDPASS)==0&&display_mode==avitext) {lcd.setCursor(0,14); lcd.write(z);}
if(FAT_mounted&&logfile&&(pass&LOGPASS)==0) fputs(z,logfile);
	sprintf(z,"KMH %6.2f  HDG %5.1f ACC %5.2f\n",ground_speed*3.6f,gps_heading,gps_acceleration_lowpassed[0]);
	if((pass&LCDPASS)==0&&display_mode==avitext) {lcd.setCursor(0,15); lcd.write(z);}
if(FAT_mounted&&logfile&&(pass&LOGPASS)==0) fputs(z,logfile);
	}

if((pass&LCDPASS)==0)
	{
	switch(display_mode) {
		case avitext: //text debug mode
			break;
		case gui_mode: //update all variable fields of widgets
			update_gui(&lcd,displaymode_changed);
			break;
		case magnet_cal:	//magnetometer calibration mode
			if((pass&LCDPASS)==0)
				{
				lcd.setCursor(0,0); 
				lcd.write("make 2-3 full circles");
				lcd.setCursor(0,1); 
                                lcd.write("use right button to store calibration");
                                char z[64];
                                sprintf(z,"mago %f %f %f\n",magnetic_offsets[0],magnetic_offsets[1],magnetic_offsets[2]);
				if(logfile) fputs(z,logfile);
                                lcd.setCursor(0,2);lcd.write(z);
                                sprintf(z,"mags %f %f %f\n",magnetic_scales[0],magnetic_scales[1],magnetic_scales[2]);
				if(logfile) fputs(z,logfile);
                                lcd.setCursor(0,3);lcd.write(z);
				xyplot(0,0,0);
				}
			xyplot(mraw[0],mraw[1],64.);
			break;
		case gyro_cal: //gyroscope calibration mode
			if((pass&LCDPASS)==0)
                                {
                                lcd.setCursor(0,0);
                                lcd.write("stay without motion for a minute");
                                lcd.setCursor(0,1);
                                lcd.write("use right button to store calibration");
				char z[64];
				sprintf(z,"Old omega off %f %f %f\n",omega_offset_x,omega_offset_y,omega_offset_z);
                                lcd.setCursor(0,2);lcd.write(z);
				if(logfile) fputs(z,logfile);
				sprintf(z,"Old pitot off %f %f\n",p_pitot_calibration_offset,p_pitot2_calibration_offset);
                                lcd.setCursor(0,3);lcd.write(z);
                                if(logfile) fputs(z,logfile);
                                }
			break;
		case can_debug:
			break;
		}
	displaymode_changed=false;
	}

if((pass&LCDPASS)!=0) //if not updating display, do some other useful work
	{
	if(pass % TEMP_PASS == 3) am2320_wakeup();
 	if(pass % TEMP_PASS == 4) {float tmp=am2320_humidity(); if(!isnan(tmp)) humidity=tmp;}
        if(pass % TEMP_PASS == 5) {float tmp=am2320_temperature(); if(!isnan(tmp)) temperature=tmp;}
	float p_static,p1,p2;
	read_pressures(&p_static,&p1,&p2);
	process_pressures(p_static,p1,p2,qnh,temperature, humidity,&altitude,&ias,&tas,&aoa,&density_altitude);
	float x,y;
	uint8_t touch=touched(&x,&y); 
	if(touch) process_touch(x,y); //call GUI
	//compute TAS from IAS, temperature, humidity, altitude and speed
	}
}

if(FAT_mounted&&logfile&&(pass&127)==0)
        {
        fflush(logfile);
        fsync(fileno(logfile)); //the file is never close()'d, so we have to sync it time to time
        }

#ifdef WITH_CAN
{
CAN_MESSAGE m;
uint8_t len=can_receive(&m);
if(len) //log the received CAN message
        {              
	//@@@timestamp
	print_can_message(&m,stdout);
        if(FAT_mounted&&logfile) {print_can_message(&m,logfile); fflush(logfile);}
        }
}
if(display_mode==can_debug && (pass&LCDPASS)==0) //display collected values
	{
	can_display(lcd);
	}
#endif


//check input from usart1  and/or USB
if(inuart1line
#ifdef WITH_USB
	||inusbline
#endif
	) //select() not implemented yet
	{
	char line[256];
	char *p=NULL;
	if(inuart1line) {p=fgets(line,256,us1); stdout=us1;}
#ifdef WITH_USB
	if(inusbline) {p=fgets(line,256,usb); stdout=usb;}
#endif
	if(p) 
	 {
	 int n=strlen(p);
	 if(p[n-1]=='\n') p[n-1]=0;
	 switch(line[0]) {
#ifdef WITH_USB
		case 'U':
			fprintf(usb,line);
			break;
#endif
#if 0
		case 'H': //halt the cpu
			__disable_irq();
			while(1);
			break;
#endif
		case 't': //show rtc
			 {
        		char name[64];
       			struct timeval tv;
        		gettimeofday(&tv,NULL);
        		struct tm *tmp;
        		tmp = gmtime(&tv.tv_sec);
        		strftime(name,64,"RTC=%Y%m%d_%H%M\n",tmp);
			fputs(name,stdout);
			}
			break;
		case 'T':
			{
                        time_t time;
                        unsigned long t;
                        sscanf(line+1,"%lu",&t); //numerical time
                        time=t;
			struct tm *tmp;
        		tmp = gmtime(&time);
                	rtc_settime(tmp);
                	rtc_was_set=true;
                	}
			break;
#ifdef WITH_CAN
               case 'C': //CAN init
                        {
                        uint32_t baud;
                        sscanf(line+1,"%d",&baud);
                        uint8_t r=init_can(baud,1);
                        if(r) printf("init_can error %d\n",r);
                        else printf("init_can success\n");
                        }
                        break;
                case 'c': //CAN transmit
                        {
                        if(FAT_mounted&&logfile) {fputs(line,logfile); fputs("\n",logfile);} //@@@also timestamp
                        int id,l,rtr;
                        CAN_MESSAGE m;
                        sscanf(line+1,"%x %d %d %llx",&id,&l,&rtr,&m.data.word);
                        m.id=id;
                        m.length=l;
                        can_transmit(&m,rtr);
                        }
                        break;
                case 'o': //OBD2 transmit
                        {
                        if(FAT_mounted&&logfile) {fputs(line,logfile); fputs("\n",logfile);} //@@@also timestamp
                        CAN_MESSAGE m;
                        int service,pid;
                        sscanf(line+1,"%d %x",&service,&pid);
                        create_OBD2_query(service,pid,&m);
                        can_transmit(&m,1);
                        }
                        break;
#endif
		case '@': //software reset 
                        {
                        reboot_type_initialized=MAGIC_INITIALIZED;
                        reboot_type_requested=line[1]-'0';
                        softreset();
                        }
                        break;
		case '!': //simple shell commands for the fat  NOTE use /filename
			{
			char *p=line;
                        do{if(*p=='\n') *p=0;}while(*p++);
			if(0==strncmp(line+1,"ls ",3)) {fl_listdirectory(line+4); break;}
			if(0==strncmp(line+1,"ls",2)) {fl_listdirectory("/"); break;}
			if(0==strncmp(line+1,"rm ",3)) {fl_remove(line+4); break;}
			if(0==strncmp(line+1,"cat ",4))
				{
				FILE *f=fopen(line+5,"r");
                                if(!f) 
                                        {
                                        printf("cannot open %s for reading (forgotten /?)\n",line+5);
                                        break;
                                        }
				char tmp[256];
				while(fgets(tmp,256,f)) fputs(tmp,stdout);
				fclose(f);
				break;
				}
			}
			break;
		case 'E': //reset eeprom calibration
			init_magnetic_calibration();
			init_calibration(HAS_EEPROM,1,magnetic_offsets,magnetic_scales,&magnetic_fit.fitmat[0][0],&magnetic_fit.rhsmat[0][0]);
			break;
		case 'e':
			print_calibration(&magnetic_offsets[0],&magnetic_scales[0]);
			break;
		default:
			printf("Unknown command %s\n",line);
			break;	
	}}
	}

//check input from usart2 (GPS)
if(inuart2line)
	{
	char line[256];
	char *p=NULL;
	p=fgets(line,256,gps);
	if(p) 
		{
		int r=nmea_process(line,NULL,&last_fix,&gps_time,gps_position,&ground_speed,&gps_heading,&nsat,&accuracy);
		if(r) goto uart2_done;
		if(ground_speed>GPS_HEADING_THR) //update gps_heading_magnetic
			{
			Vec3<float> magfield;
			//it is costly do compute wmm and changes very slowly with position, so update it only sporadically
			if((gps_valid_sequence&WMM_MODULO)==0 || declination==0&&inclination==0)
				{
				int r=wmm(gps_position[0], gps_position[1], gps_position[2], time2year(gps_time),&magfield[0],&magfield[1],&magfield[2],&declination,&inclination);
				magfield *= 0.00001; //convert from nanotesla to gauss
				//we have now theoretical magnetic field in the earth axis system and declination and inclination
				}
			gps_heading_magnetic = gps_heading-declination;
			if(gps_heading_magnetic<=0) gps_heading_magnetic += 360.f;
			if(gps_heading_magnetic>360.f) gps_heading_magnetic -= 360.f;
			}
		if(last_fix==gps_time && last_fix!=0) 
			{
			if(previous_gps_time!=gps_time) ++gps_valid_sequence; 
			gps_timeout=300;
			} 
		else gps_valid_sequence=0;
		if(gps_valid_sequence>=2 && previous_gps_time!=gps_time) //compute acceleration and vario, assuming body-x-direction
			{
			gps_vario = (gps_position[2]-previous_gps_position[2])/(gps_time-previous_gps_time);
			gps_vario_lowpassed *= alpha_gv;
			gps_vario_lowpassed += gps_vario * (1.f-alpha_gv);
			gps_acceleration[0] = (ground_speed-previous_ground_speed)/(gps_time-previous_gps_time);
			gps_acceleration_lowpassed[0] *= alpha_g;
			gps_acceleration_lowpassed[0] += gps_acceleration[0] * (1.f-alpha_g);
			}
		if(gps_valid_sequence>=1  && previous_gps_time!=gps_time)
			{
			previous_gps_position=gps_position;
			previous_ground_speed=ground_speed;
			previous_gps_time=gps_time;
			}
		}
	uart2_done:;
	}

//check input from usart3 (BT)
if(inuart3line)
        {
        char line[256];
        char *p=NULL;
        p=fgets(line,256,bt);
        if(p) {printf("BT: "); printf(line);}
        }

++pass;
if((pass&16383)==0) rtc_was_set=0; //set RTC via GPS periodically about once a minute
if(gps_timeout) --gps_timeout;
if(gps_timeout==0)
	{
	gps_velocity[0]=0;
	gps_acceleration[0]= gps_acceleration_lowpassed[0]= 0;
	}
}//main loop
}
