/*
    Copyright (C) 2020 Jiri Pittner <jiri@pittnerovi.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include <stdio.h>
#include "bitmap.h"

bitmap::bitmap(uint16_t n, uint16_t m)
{
nn=n;
mm=m;
ldim=(nn+7)/8;
data= new uint8_t[mm*ldim];
if(data==NULL) printf("bitmap allocation failure\n");
}

void bitmap::resize(uint16_t n, uint16_t m)
{
if(data) delete[] data;
nn=n;
mm=m;
ldim=(nn+7)/8;
data= new uint8_t[mm*ldim];
if(data==NULL) printf("bitmap allocation failure\n");
}


bitmap::~bitmap()
{
if(data) delete[] data;
}

bool bitmap::operator() (uint16_t n, uint16_t m) const
{
return (data[m*ldim+(n>>3)] >>(n&7)) &1;
}

void  bitmap::assign(uint16_t n, uint16_t m, bool value)
{
uint8_t *p = data + m*ldim+(n>>3);
if(value) *p |= (1<<(n&7));
else *p &= ~(1<<(n&7));
}
