/*
 * *      caninterface.c  ... firmware for CAN interface and datalogger and OBD2 diagnostic tool 
 *        based on  STM32F373VCT6 
 * *
 * *      Copyright (C) 2022 Jiri Pittner <jiri@pittnerovi.com>
 * *
 * *      This program is free software: you can redistribute it and/or modify
 * *      it under the terms of the GNU General Public License as published by
 * *      the Free Software Foundation, either version 3 of the License, or
 * *      (at your option) any later version.
 * *
 * *      This program is distributed in the hope that it will be useful,
 * *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 * *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * *      GNU General Public License for more details.
 * *      You should have received a copy of the GNU General Public License
 * *      along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * *
 * *
 * */
//HARDWARE: caninterface.pro, without ethernet and lcd WILL RUN ALSO ON avionics.pro

//TODO: DTC codes readout was not properly tested


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <setjmp.h>
#include <errno.h>
#include <math.h>
#include "usart123.h"
#include "delay.h"
#include "lcd.h"
#include "stm32f37x_gpio.h"
#include "stm32f37x_rcc.h"
#include "stm32f37x_pwr.h"
#include "stm32f37x_tim.h"
#include "stm32f37x_exti.h"
#include "stm32f37x_dma.h"
#include "stm32f37x_adc.h"
#include "stm32f37x_dac.h"
#include "stm32f37x_misc.h"
#include "stm32f37x_syscfg.h"
#include "stm32f37x_can.h"

#include "can.h"
#include "can_devind.h"
#include "obd2.h"
#define XSTR(s) STR(s)  
#define STR(s) #s
#ifdef WITH_DBC 
#include XSTR(DBC_INCLUDE)
#endif



#include "i2c1.h" 
#include "mcp24aa.h"
#include "i2c2.h" 

#include "rtc.h"
#include "sdcard.h"
#include "fat_interface.h"

#ifdef WITH_GPS
#include "nmea.h"
#endif

#ifdef WITH_USB
#include "usb/inc/ttyacm.h"
#include "usb/inc/usbsd.h"
#endif


#ifdef WITH_ETHERNET

uint8_t lan_buf[LAN_BUFFER_SIZE+3];
#include "lan.h"

//LAN parameters
uint8_t mymac[6] = {0x44,0xa5,0x2a,0x42,0x10,0x27}; //set some arbitrary one for DHCP
uint8_t dstmac[6] = {0,0,0,0,0,0};
uint8_t myip[4] = {0,0,0,0}; //use DHCP
uint8_t mygwip[4] = {192,168,0,6};
uint8_t dnsip2[4] = {0,0,0,0};
uint8_t dhcpsrvip[4] = {0,0,0,0};
uint8_t mynetmask[4] = {255,255,255,0};
uint8_t destip[4] = {0,0,0,0};
short dst_port = 10000; //send answers to this port of host; we listen on all ports
short src_port = 20000; //our from port
//UDP pseudo-shell: nc -u caninterface_ip any_port_num
//and  nc -l -u -p 10000
//or smsm2 -S caninterface_ip any_port_num 10000

#define N_WHITE_IP 1
static const uint8_t ip_whitelist[N_WHITE_IP][4]={
        {192,168,0,20},
        }; 

#endif

volatile uint32_t randomseed  __attribute__ ((section(".persistent")));

uint32_t truerandom(void) 
{
return randomseed;
} 


volatile uint32_t milliseconds=0;
void TIM4_IRQHandler(void) __attribute__((optimize("-O1")));
void TIM4_IRQHandler(void)
{
TIM_ClearITPendingBit(TIM4,TIM_IT_Update);
++milliseconds;
}

//for milliseconds
void init_tim4(void)
{
RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM4, ENABLE);
RCC_APB1PeriphResetCmd(RCC_APB1Periph_TIM4, ENABLE);
RCC_APB1PeriphResetCmd(RCC_APB1Periph_TIM4, DISABLE);
TIM_TimeBaseInitTypeDef TIM_TimeBaseInitStructure;
TIM_TimeBaseStructInit(&TIM_TimeBaseInitStructure);
TIM_TimeBaseInitStructure.TIM_Prescaler=1;
TIM_TimeBaseInitStructure.TIM_Period = (XTAL*PLLMULT)/2/1000;
TIM_TimeBaseInitStructure.TIM_CounterMode = TIM_CounterMode_Up;
TIM_TimeBaseInit(TIM4,&TIM_TimeBaseInitStructure);
TIM_InternalClockConfig(TIM4);
TIM4->CNT=0;
TIM_ITConfig(TIM4, TIM_IT_Update, ENABLE);
TIM_Cmd(TIM4,ENABLE);
NVIC_InitTypeDef NVIC_InitStructure;
NVIC_InitStructure.NVIC_IRQChannel = TIM4_IRQn;
NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
NVIC_Init(&NVIC_InitStructure);
}






//leds - PF2,9,10, beeper PF4
void init_led_beeper(void)
{
GPIO_InitTypeDef GPIO_InitStructure;
RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOF, ENABLE);
GPIO_InitStructure.GPIO_Pin =  GPIO_Pin_2|GPIO_Pin_4|GPIO_Pin_9|GPIO_Pin_10;
GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
GPIO_Init(GPIOF, &GPIO_InitStructure);
cbi(GPIOF,2);
cbi(GPIOF,4);
cbi(GPIOF,9);
cbi(GPIOF,10);
}




void led(uint8_t which, uint8_t onoff) //which = 0-2 
{
uint8_t pin=which?8+which:2;
if(onoff) sbi(GPIOF,pin); else cbi(GPIOF,pin);
}

void beeper(uint8_t onoff) 
{
if(onoff) sbi(GPIOF,4); else cbi(GPIOF,4);
}

void usb_connect(uint8_t onoff)
{
GPIO_InitTypeDef GPIO_InitStructure;
RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOB, ENABLE);
GPIO_InitStructure.GPIO_Pin =  GPIO_Pin_14;
GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
GPIO_Init(GPIOB, &GPIO_InitStructure);
if(onoff) cbi(GPIOB,14);
else sbi(GPIOB,14);
}

FILE *logfile=NULL;


#define MAGIC_INITIALIZED 0x3141592653589793ULL
static uint64_t reboot_type_initialized  __attribute__ ((section(".persistent")));
static uint8_t reboot_type_requested __attribute__ ((section(".persistent")));

uint8_t reboot_allowed=0;
void reboot(uint8_t type)
{
fclose(logfile);
reboot_type_initialized=MAGIC_INITIALIZED;
reboot_type_requested=type+1;
delay_ms(5);
softreset();
}

#define N_CAN_BAUD_SEL 6
volatile int8_t can_baud_sel=0;
static const uint32_t can_bauds[N_CAN_BAUD_SEL]={500000UL,250000UL,100000UL,50000UL,25000UL,10000UL};

#define N_WORKMODES 9
typedef enum {
setup=0,
receive=1,
service1=2,
service2=3,
service3=4,
service4=5,
service9=6,
service1all=7,
service2all=8,
} WORKMODE;

volatile WORKMODE workmode,old_workmode;
volatile uint8_t selected_pid, old_pid, continuous_mode;
volatile uint8_t can_tx_pending;
volatile uint8_t vin_request_pending=0;
volatile uint8_t  group_query_request=0;
volatile uint8_t lcd_clear_requested;
volatile CAN_MESSAGE can_tx_message;

uint8_t workmode_service[N_WORKMODES] = {0,0,1,2,3,4,9,1,2};
#define DISPLAYMAX 21
char workmode_names[N_WORKMODES][DISPLAYMAX] ={
"SETUP",
"CAN RECEIVE",
"OBD2 S1",
"OBD2 S2",
"OBD2 S3 DTC CODES",
"OBD2 S4 CLEAR DTC",
"OBD2 S9",
"OBD2 S1 all",
"OBD2 S2 all",
};


#define N_SETUPMODES 3
typedef enum {
showtime=0,
showlan=1,
canspeed=2,
} SETUPMODE;

volatile SETUPMODE setupmode,old_setupmode;
volatile uint8_t setup_item_selected;
volatile uint8_t request_lan_reinit, request_usb_reinit;

char setupmode_names[N_SETUPMODES][DISPLAYMAX] ={
"DATE AND TIME",
"LAN INFO",
"CAN BAUD RATE",
};


//user interface
//buttons
void process_button(uint8_t which, uint16_t ms)
{
//printf("button %d pressed for %dms\n",which,ms);
switch(which)
	{
	case 2: //black
		old_workmode = workmode;
		workmode = (WORKMODE) (workmode+1)%N_WORKMODES;
		continuous_mode=0;
		group_query_request=0;
		setupmode=setup_item_selected=0;
		old_pid=255; selected_pid=0;
		break;
	case 0: //green
		if(ms>=1000 && reboot_allowed) reboot(0);
		else request_lan_reinit=1;
		break;
	case 1: //red
		if(ms>=1000 && reboot_allowed) reboot(1);
		else request_usb_reinit=1;
		break;
	}
}

//rotcoder
void process_rotation(uint8_t dir)
{
switch(workmode)
	{
	case setup:
		{
		if(setup_item_selected)
			{
			switch(setupmode)
				{
				case canspeed:
					can_baud_sel += (dir?1: -1);
					if(can_baud_sel<0) can_baud_sel=0;
					if(can_baud_sel>=N_CAN_BAUD_SEL) can_baud_sel=N_CAN_BAUD_SEL-1;
					break;
				case showtime:
				case showlan:
				default: 
					break;
				}
			}
		else //rotated setup items
			{
			old_setupmode=setupmode;
			int16_t tmp = setupmode;
                	tmp += (dir?1: -1);
                	if(tmp<0) tmp=0;
                	if(tmp>=N_SETUPMODES) tmp=N_SETUPMODES-1;
			setupmode=tmp;
			}
		}
		break;
        case receive: //no effect
                break;
        case service1: 
        case service2: 
		{
		old_pid=selected_pid;
		int16_t tmp = selected_pid;
		tmp += (dir?1: -1);
		if(tmp<0) tmp=0;
		if(tmp>=NUMBER_OF_ALL_PIDS) tmp=NUMBER_OF_ALL_PIDS-1;
		selected_pid=tmp;
		continuous_mode=0;
		lcd_clear_requested=1;
		}
                break;
        case service3: //no effect
        case service4: //no effect
        case service9: //no effect
        case service1all: //no effect
	case service2all: //no effect
	default: //no effect
                break;
	};
}


void process_press(uint16_t ms)
{
switch(workmode)
        {
	case setup:
		if(ms<300) setup_item_selected ^= 1; //short press (de)selects item
		else //long press
			switch(setupmode)
                                {
				case canspeed: //init can
					{
					uint8_t r=init_can(can_bauds[can_baud_sel],1); 
					if(r) printf("init_can error %d\n",r);
					else printf("init_can success\n");

					}
					break;
				case showtime:
				case showlan:
				default:
					break;
				};
		break;
        case receive: //no transmitting, receive all
		can_tx_pending=0;
                continuous_mode=0;
                break;
        case service1:  
        case service2: //send obd2 message, optionally indicate cont. mode
		create_OBD2_query(workmode_service[workmode],selected_pid,&can_tx_message);
                can_tx_pending=1;
		if(continuous_mode) continuous_mode=0;
		else if(ms>300)  continuous_mode = 1;
                break; 
        case service3: 
		create_OBD2_query(workmode_service[workmode],0,&can_tx_message);
		can_tx_pending=1;
		if(continuous_mode) continuous_mode=0;
                else if(ms>300)  continuous_mode = 1;
                break; 
        case service4: 
		create_OBD2_query(workmode_service[workmode],0,&can_tx_message);
                can_tx_pending=1;
		continuous_mode=0;
                break; 
        case service9: 
		vin_request_pending=1;
		can_tx_pending=0;
                continuous_mode=0;
                break;
        case service1all: 
	case service2all:
		if(group_query_request) group_query_request=0;
		else group_query_request= (ms>300?2:1);
		can_tx_pending=0;
		selected_pid=0;
		continuous_mode=0;
                break;
        };      

}




int8_t get_vin(char *vin)
{
//timeout via watchdog

//set hw filtering
can_rx_filter(0x7e8,0x7e8);
CAN_MESSAGE m;
create_OBD2_query(9,2,&m); //initial request
uint8_t r=can_transmit(&m,0);
if(r) return 1;
uint8_t len;
do
	{
	len=can_receive(&m);
	}
while(len==0 || m.id!=0x7e8);
print_can_message(&m,stdout,0);
if(m.data.byte[0]!=0x10 || m.data.byte[2]!=0x49|| m.data.byte[3]!=2) return 2;
uint8_t extralen=m.data.byte[1];
vin[0]=m.data.byte[5];
vin[1]=m.data.byte[6];
vin[2]=m.data.byte[7];
//send continuation request
delay_ms(60);
m.id=0x7e0;
m.data.byte[0]=0x30;
memset(m.data.byte+1,0,7);
r=can_transmit(&m,0);
if(r) return 3;
do
        {
        len=can_receive(&m);
        }
while(len==0 || m.id!=0x7e8);
print_can_message(&m,stdout,0);
if(m.data.byte[0]!=0x21) return 4;
memcpy(vin+3,m.data.byte+1,7);
do
        {
        len=can_receive(&m);
        }
while(len==0 || m.id!=0x7e8);
print_can_message(&m,stdout,0);
if(m.data.byte[0]!=0x22) return 5;
memcpy(vin+10,m.data.byte+1,7);
vin[17]=0;
return 0;
}


uint32_t pid_supported[(NUMBER_OF_ALL_PIDS+32)/32];

uint8_t pid_is_unsupported(uint8_t pid)
{
if(pid==0) return 0;
return (pid_supported[(pid - 1) / 0x20] & ((1UL << (31 - ((pid - 1) % 32))))) ? 0 : 1;
}



#define GPSPASS 127
#define LCDPASS 3
//must not send more freequently than each 60ms
#define CANSENDPASS 63
#define TIMESTAMPPASS 16383


int main(void)
{
request_usb_reinit=request_lan_reinit=0;
group_query_request=0;
reboot_allowed=0;
old_workmode=0;
workmode=0;
setupmode=old_setupmode=0;
setup_item_selected=0;
selected_pid=0;
old_pid=0;
continuous_mode=0;
lcd_clear_requested=0;
init_led_beeper();
init_tim4();

FILE *us1=fopen("usart 1 115200 1","w+"); setlinebuf(us1);
stdout=stderr=us1;
printf("Reset!\n");

if(reboot_type_initialized==MAGIC_INITIALIZED)
	{
	reboot_type_initialized=0;//for next time
	printf("Targeted reboot %d\n",reboot_type_requested);
	}
else
	{
        reboot_type_requested=0;
	}



lcd_init(); 
lcd_light(1);
lcd_print("Reset");

printf("Init done!\n");

watchdog_init(120000); watchdog();
printf("Watchdog activated!\n");

uint8_t mass_storage_mode=0; 

#ifdef WITH_USB
{
RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOC, ENABLE);
GPIO_InitTypeDef GPIO_InitStructure;
GPIO_InitStructure.GPIO_Pin = GPIO_Pin_13;
GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
GPIO_Init(GPIOC, &GPIO_InitStructure);
delay_ms(5);
if(reboot_type_requested==0 && bit_is_clear(GPIOC,13)||reboot_type_requested==2)
        {
        printf("enable USB MASS storage\n");
        led(2,1);
        mass_storage_mode=1;
        delay_ms(300);
        }
else    printf("enable USB terminal\n");
}

usb_connect(1);

FILE *usb=NULL;
if(!mass_storage_mode) 
        {
        usb=fopen("usb 1","w+");
        if(usb)
                {
                setlinebuf(usb);
                printf("USB TTYACM Init success!\n");
                //does not mean it is connected to host yet!
                }
         else    printf("USB TTYACM Init failed!\n");
         }
#endif

uint8_t rtc_was_set=0;
rtc_init();

static char timestamp[24];



//init SD card
{
uint8_t i;
for(i=0; i<4; ++i) //card might need some time after powerup
        { 
        delay_ms(100);
        SDHC_Init();
        if(!SD_error) break; 
        }
}
printf("SD init error=%d\n",SD_error); 

//for user interface
delay_ms(10);
init_rotcoder_buttons(process_press,process_rotation,process_button);
delay_ms(10);
reboot_allowed=1;
request_lan_reinit= request_usb_reinit=0;

uint8_t FAT_mounted=0;
if(!SD_error) 
{
#ifdef WITH_USB
if(mass_storage_mode)
	{
	lcd_cursor(0,0); lcd_print("MASS STORAGE");
	uint8_t r=usbsd_init(1);
	printf("usbd_init returned %d\n",r);
	if(r==0)
		{
		lcd_cursor(1,0); lcd_print("MODE ENTERED");
		while(1){watchdog(); delay_ms(1000);}
		}
	else
		{
		lcd_cursor(1,0); lcd_print("FAILED");
		delay_ms(3000);
		}
	}
else
#endif
	{
	//initialize FAT
	fat_mount();  
	FAT_mounted=1;
	}
printf("FAT mounted\n");
}

//open log file
if(FAT_mounted)
        {
        char name[64];
        struct timeval tv;
        gettimeofday(&tv,NULL);
        struct tm *tmp;
        tmp = gmtime(&tv.tv_sec);
        strftime(name,64,"/%Y%m%d_%H%M.log",tmp);
        logfile=fopen(name,"w");
        if(logfile) printf("logfile %s created\n",name);
        fputs("CAN logfile\n",logfile); fflush(logfile);
        }


//init I2C
I2C_LowLevel_Init(); //eeprom is here
I2C2_LowLevel_Init();
printf("I2C initialized\n");

#ifdef WITH_ETHERNET
{
watchdog();
uint8_t r=init_lan(1); //DHCP
if(r==0)
        { 
        puts("LAN INIT OK\n");
	lcd_cursor(1,0);
	char z[24];
	sprintf(z,"IP=%d.%d.%d.%d",myip[0],myip[1],myip[2],myip[3]);
	lcd_cursor(1,0);
        lcd_printn(z,20);	
        }
 else 
        {
        puts("LAN INIT FAILED\n");
	lcd_cursor(1,0);
	lcd_printn("DHCP failed",20);
        }
watchdog();
delay_ms(1000);
watchdog();
}
FILE *udp=fopen("udp","w+");
if(udp)
        {
        setlinebuf(udp);
        printf("UDP open success\n");
        }
else    printf("UDP open failed\n");
#endif


#ifdef WITH_GPS
FILE *gps=NULL;
gps=fopen("usart 2 9600 0","w+"); //no echo!
setlinebuf(gps);
printf("GPS usart initialized\n");
//speed can be 38400 by hardware jumper on the module, but it is not critical

float gps_position[3]={0,0,0};
float gps_vario=0; //from gps
float gps_vario_lowpassed=0;
uint32_t gps_valid_sequence=0; 
time_t gps_time=0;
uint32_t gps_timeout=0;
float ground_speed=0.; 
float gps_heading=0;
int nsat=0;
float accuracy=0;               
time_t last_fix=0;

#endif

//init CAN
{
can_baud_sel=0;
uint8_t r=init_can(can_bauds[can_baud_sel],1); //works better with auto retransmission
if(r) printf("init_can error %d\n",r);
else printf("init_can success\n");
}


lcd_cursor(0,0);
lcd_print("READY");
printf("READY\n");


//allocate memory for message decoding
CAN_VALUE canval[MAXSIG];
char groupnames[MAXSIG*SIGNAME_LEN];
char signames[MAXSIG*SIGNAME_LEN];
char units[MAXSIG*UNIT_NAME_LEN];
{
uint8_t  i;
for(i=0; i<MAXSIG; ++i)
        {
        canval[i].groupname=groupnames+SIGNAME_LEN*i;
        canval[i].signame=signames+SIGNAME_LEN*i;
        canval[i].unit=units+UNIT_NAME_LEN*i;
        }
}

////////////////////////////////main loop
int8_t can_rx_pending=0;
can_tx_pending=0;
reboot_allowed=1;
uint32_t pass=0;
//LAN processing done analogously to routerwatch.c on LPC
while(1)
{
//generate time stamp
{     
struct timeval tv;
gettimeofday(&tv,NULL);
struct tm *tmp;
tmp = gmtime(&tv.tv_sec);
strftime(timestamp,24,"%Y.%m.%d %H:%M:%SU",tmp);
timestamp[20]=0;
}     
if((pass&TIMESTAMPPASS)==0) 
	{
	fputs(timestamp,stdout); fputs("\n",stdout);
	if(FAT_mounted&&logfile)
		{
		fputs(timestamp,logfile); fputs("\n",logfile);
		}
	}


//hardware can filtering is important, otherwise some responses get missed
//in the flow of orrelevant messages
uint8_t obd2only= (workmode!=receive);
if(obd2only) can_rx_filter(0x7e8,0x7e8);
else can_rx_filter(0,0);
watchdog();

//periodically flush logfile
if(FAT_mounted&&logfile&&(pass&1023)==0)
        {
        fflush(logfile);
        fsync(fileno(logfile)); //the file is never close()'d, so we have to sync it time to time
        }

#ifdef WITH_GPS
if((pass&16383)==0) rtc_was_set=0; //set RTC via GPS periodically about once a minute
if(gps_timeout) --gps_timeout;
#endif
if(gps_timeout==0)
        {
        ground_speed=0;
        }

//check for incoming CAN traffic
{
CAN_MESSAGE m;
uint8_t len=can_receive(&m);
if(len) //process CAN message
	{
	randomseed <<=2; randomseed ^= TIM4->CNT;
	if((!obd2only) || (m.id >= 0x7e8 && m.id <=0x7ef))  //filter
		{
		led(1,1);
		fputs(timestamp,stdout);
		print_can_message(&m,stdout,obd2only);
		if(FAT_mounted&&logfile) 
			{
			 fputs(timestamp,logfile);
			print_can_message(&m,logfile,obd2only);
			}
		//filter by service and pid
		if(m.data.byte[1]!= 0x40+workmode_service[workmode])
			{
			fputs("RX message mismatch 1\n",stdout);
			if(FAT_mounted&&logfile)  fputs("RX message mismatch 1\n",logfile);
			goto can_rx_done;
			}
		if(workmode==service1 || workmode==service2 || workmode==service1all || workmode==service2all)
			{
			if(m.data.byte[2]!= selected_pid && (workmode==service1 || workmode==service2)) //in group modes pid has been incremented meanwhile
				{
	                        fputs("RX message mismatch 2\n",stdout);
       		                if(FAT_mounted&&logfile)  fputs("RX message mismatch 2\n",logfile);
				goto can_rx_done;
       		                }
				if((m.data.byte[2]&0x1f)==0)//PID for supported PIDs
					{
					pid_supported[m.data.byte[2] / 0x20] = ((uint32_t)m.data.byte[3] << 24 | (uint32_t)m.data.byte[4] << 16 | (uint32_t)m.data.byte[5] << 8 | (uint32_t)m.data.byte[6]);
					}
			}
		can_rx_pending=decode_can_message(&m,&canval);
		if(FAT_mounted&&logfile) fprintf(logfile,"OBD2 decode returned %d\n",can_rx_pending);
		if(can_rx_pending>0)
			{
			output_can_values(&canval,can_rx_pending,stdout);
			if(FAT_mounted&&logfile)  output_can_values(&canval,can_rx_pending,logfile);
			}
		}
	}
can_rx_done:;
}

//handle group query mode (on first pass, it will read only the PIDs encoding supported pids, then all supported pids
if(group_query_request && (!can_tx_pending) &&(workmode==service1all||workmode==service2all)) //generate new tx request
	{
	//generate new tx request
	fprintf(stdout,"query S%d pid %03d\n",workmode_service[workmode],selected_pid);
	create_OBD2_query(workmode_service[workmode],selected_pid,&can_tx_message);
	can_tx_pending=1;
	//increment pids, skip unsupported pids except for support encoding pids
	do{++selected_pid;} while(selected_pid<=NUMBER_OF_ALL_PIDS && pid_is_unsupported(selected_pid) && ((selected_pid&0x1f)!=0));
	//stop group_query_request if all done 
	if(selected_pid >= NUMBER_OF_ALL_PIDS)
		{
		selected_pid=0;
		if(group_query_request==1) group_query_request=0;
		}
	}

//transmit pending CAN message
if(can_tx_pending && (pass&CANSENDPASS)==0)
	{
	led(0,1);
	uint8_t rtr=0; //MUST be 0
	uint8_t r=can_transmit(&can_tx_message,rtr); 
	 if(FAT_mounted&&logfile)   {fprintf(logfile,"%s RTR %d ",r?"FAILSENT":"SENT",rtr); print_can_message(&can_tx_message,logfile,0);}
	if(!continuous_mode) can_tx_pending=0;
	}


//display handling here
if((pass&LCDPASS)==0)
    {
    if(old_workmode!=workmode)
	{
	lcd_clear();
	lcd_clear_requested=0;
	old_workmode=workmode;
	selected_pid=0; old_pid=255;
	memset(pid_supported,0,sizeof(uint32_t)*(NUMBER_OF_ALL_PIDS+32)/32);
	fprintf(stdout,"Entering mode %s\n",workmode_names[workmode]);
	if(FAT_mounted&&logfile)   {fprintf(logfile,"Entering mode %s\n",workmode_names[workmode]);}
	}
    else if(lcd_clear_requested || selected_pid!=old_pid && (workmode == service1 || workmode == service2)) {lcd_clear(); lcd_clear_requested=0;}

    lcd_cursor(0,0);
    lcd_print(workmode_names[workmode]);
    switch(workmode) 
	{
	case setup:
		{
		lcd_cursor(1,0);
		lcd_printn(setupmode_names[setupmode],20);
		if(setupmode!=old_setupmode) 
			{
			fprintf(stdout,"Entering setup mode %s\n",setupmode_names[setupmode]);
        		if(FAT_mounted&&logfile)   {fprintf(logfile,"Entering setupmode %s\n",setupmode_names[setupmode]);}
			lcd_cursor(2,0); lcd_printn("",20);
			lcd_cursor(3,0); lcd_printn("",20);
			old_setupmode=setupmode;
			}
		if(setup_item_selected)
		    {
		    lcd_cursor(2,0);
		    char z[21];
		    switch(setupmode)
			{
			case canspeed:
				{
				sprintf(z,"%ld",can_bauds[can_baud_sel]);
				lcd_printn(z,20);
				}
				break;
			case showtime:
				lcd_printn(timestamp,20);
				if(gps_time) 
					{
					lcd_cursor(3,0); lcd_printn("GPS LOCKED",20);
					}
				break;
			case showlan:
				{
#ifdef WITH_ETHERNET
				sprintf(z,"IP=%d.%d.%d.%d",myip[0],myip[1],myip[2],myip[3]);
                                lcd_printn(z,20);
#endif
				}
				break;
			default:
				break;
			};
		    }
		}
		break;
	case receive: 
		lcd_cursor(1,0);
		lcd_print("Logging all CAN msgs");
		break;
	case service1: 
	case service2: 
		{
		lcd_cursor(0,1+strlen(workmode_names[workmode]));
		char z[21];
		sprintf(z,"PID %03d%c%4s",selected_pid,(pid_is_unsupported(selected_pid)?'x':' '),(continuous_mode?"CONT":"    "));
		lcd_print(z);
		//get pid name even if nothing received yet
		if(can_rx_pending<=0)
			{
			lcd_cursor(1,0);
			CAN_MESSAGE m; //create a dummy message
			m.id=0x7e8;
			m.length=8;
			m.data.byte[1]=workmode_service[workmode]+0x40;
			m.data.byte[0]=6;
			m.data.byte[2]=selected_pid;
			int8_t r=decode_can_message(&m,&canval); 
			if(r>=1)
				{
				canval[0].signame[30]=0;
				lcd_printn(canval[0].signame+10,20);
				}
			else
				lcd_print("                    ");
			}
		//print at most first two signals of that message
		if(can_rx_pending>=1)
			{
			canval[0].signame[30]=0;
			lcd_cursor(1,0);
			lcd_printn(canval[0].signame+10,20);
			}
		char line[64];
		line[0]=0;
		if(can_rx_pending>=1)
                        {
			sprintf_can_value(&canval[0],line);
			line[20]=0;
			lcd_cursor(2,0);
			lcd_printn(line,20);
                        }
		line[0]=0;
		if(can_rx_pending>=2)
			{
			sprintf_can_value(&canval[1],line);
                        line[20]=0;
			lcd_cursor(3,0);
			lcd_printn(line,20);
			}

		old_pid=selected_pid;
		}
		break;
	case service3: 
		{
		if(can_rx_pending==3)
			{
			char line[16];
			lcd_cursor(1,0);
			sprintf_can_value(&canval[0],line); lcd_printn(line,20);
			lcd_cursor(2,0);
			sprintf_can_value(&canval[1],line); lcd_printn(line,20);
                	lcd_cursor(3,0);
                	sprintf_can_value(&canval[2],line); lcd_printn(line,20);
			}
		}
		break;
	case service4: 
		lcd_cursor(1,0);
		lcd_printn("press to erase DTC",20);
		break;
	case service9:
		{
		char vin[32];
		int8_t r= -1;
		if(vin_request_pending) r=get_vin(vin);
		vin_request_pending=0;
		lcd_cursor(1,0);
		lcd_printn("VIN read",20);
		if(r>=0)
			{
			if(r) sprintf(vin,"Error %d",r);
			lcd_cursor(2,0);
		 	lcd_printn(vin,20);
			}
		}
		break;
	case service1all: 
	case service2all: 
		lcd_cursor(0,strlen(workmode_names[workmode]));
		if(group_query_request==2) lcd_print("cont"); else lcd_print("    ");
		lcd_cursor(1,0);
		char z[32];
		sprintf(z,"reading %03d",selected_pid);
		lcd_printn(z,20);
		break;
	}
    can_rx_pending=0;
    }

//if reinits were requested
#ifdef WITH_USB
if(request_usb_reinit)
{
watchdog();
puts("USB reconnect\n");
usb_connect(0);
delay_ms(500);
usb_connect(1);
request_usb_reinit=0;
}
#endif
#ifdef WITH_ETHERNET
if(request_lan_reinit)
{
watchdog();
uint8_t r=init_lan(1); //DHCP
if(r==0)
        { 
        puts("LAN INIT OK\n");
        }
 else 
        {
        puts("LAN INIT FAILED\n");
        }
request_lan_reinit=0;
}
#endif


//check input from usart1  and USB and UDP 
if(inuart1line
#ifdef WITH_USB
	||inusbline
#endif
#ifdef WITH_ETHERNET
         ||inudpline
#endif
	) //select() not implemented yet
	{
	randomseed <<=4; randomseed ^= TIM4->CNT;
	char line[256];
	char *p=NULL;
	if(inuart1line) {p=fgets(line,256,us1); fflush(stdout); stdout=us1;}
#ifdef WITH_USB
	if(inusbline) {p=fgets(line,256,usb); fflush(stdout); stdout=usb;}
#endif
#ifdef WITH_ETHERNET
         if(inudpline) {p=fgets(line,256,udp); fflush(stdout); stdout=udp;}
#endif
	if(p) 
	 {
	 int n=strlen(p);
	 if(p[n-1]=='\n') p[n-1]=0;
	 switch(line[0]) {
#ifdef WITH_USB
		case 'U': //enforce new enumeration
			usb_connect(0);
			delay_ms(500);
			usb_connect(1);
			break;
		case 'u':
			fprintf(usb,line);
			break;
#endif
		case 'C': //CAN init
			{
			uint32_t can_baud;
			sscanf(line+1,"%d",&can_baud);
			uint8_t r=init_can(can_baud,1);
			if(r) printf("init_can error %d\n",r);
			else printf("init_can success\n");
			}
			break;
		case 'c': //CAN transmit
			{
			if(FAT_mounted&&logfile) {fputs(timestamp,logfile); fputs(line,logfile); fputs("\n",logfile);} 
                        int id,l,rtr;
                        sscanf(line+1,"%x %d %d %llx",&id,&l,&rtr,&can_tx_message.data.word);
                        can_tx_message.id=id;
                        can_tx_message.length=l;
			can_tx_pending=1;
			}
			break;
		case 'o': //OBD2 transmit
			{
			if(FAT_mounted&&logfile) {fputs(timestamp,logfile); fputs(line,logfile); fputs("\n",logfile);} 
			int service,pid;
			sscanf(line+1,"%d %x",&service,&pid);
			create_OBD2_query(service,pid,&can_tx_message);
			can_tx_pending=1;
			}
			break;
                case 't': //show rtc
                         {
                        char name[64];
                        struct timeval tv;
                        gettimeofday(&tv,NULL);
                        struct tm *tmp;
                        tmp = gmtime(&tv.tv_sec);
                        strftime(name,64,"RTC=%Y%m%d_%H%M\n",tmp);
                        fputs(name,stdout);
                        }
                        break;
                case 'T':
                        {
                        time_t time;
			unsigned long t;
			sscanf(line+1,"%lu",&t); //numerical time
			time=t;
                        struct tm *tmp;
                        tmp = gmtime(&time);
                        rtc_settime(tmp);
                        rtc_was_set=1;
                        }
                        break;
		case '@': //software reset
			{
			reboot(line[1]-'0');
			}
			break;
                case '!': //simple shell commands for the fat NOTE use /filename
                        {
			char *p=line;
			do{if(*p=='\n') *p=0;}while(*p++);	
                        if(0==strncmp(line+1,"ls ",3)) {fl_listdirectory(line+4); break;}
                        if(0==strncmp(line+1,"ls",2)) {fl_listdirectory("/"); break;}
                        if(0==strncmp(line+1,"rm ",3)) {fl_remove(line+4); break;}
                        if(0==strncmp(line+1,"cat ",4))
                                {
                                FILE *f=fopen(line+5,"r");
                                if(!f) 
					{
					printf("cannot open %s for reading (forgotten /?)\n",line+5);
					break;
					}
                                char tmp[256];
                                while(fgets(tmp,256,f)) fputs(tmp,stdout);
                                fclose(f);
                                break;
                                }
                        }
                        break;
		case 'm':
			workmode=(WORKMODE) line[1]-'0';
			break;
		case 'P':
			{
			int n;
			sscanf(line+1,"%d",&n);
			selected_pid=n;
			}
			break;
		case 's':
			continuous_mode = line[1]-'0';
			break;
		case 'S':
			setupmode  = line[1]-'0';
			break;
		case 'g':
			group_query_request  = line[1]-'0';
                        break;
		case 'p':
			process_press(line[1]-'0'?500:50);
			break;	
		default:
			printf("Unknown command %s\n",line);
			break;	
	}}
	}

#ifdef WITH_ETHERNET
//check for incoming UDP packets from whitelisted addresses, ping ICMP reply
watchdog();
char *udpin = read_udp(N_WHITE_IP,ip_whitelist);
if(udpin)
        {
	randomseed <<=8; randomseed ^= TIM4->CNT;
        //printf("UDP IN %s\n",udpin); 
        }

#endif

#ifdef WITH_GPS
//check input from usart2 (GPS)
if(inuart2line)
	{
	randomseed <<=4; randomseed ^= TIM4->CNT;
	char line[256];
	char *p=NULL;
	p=fgets(line,256,gps);
	if(p) 
		{
		int r=nmea_process(line,NULL,&last_fix,&gps_time,gps_position,&ground_speed,&gps_heading,&nsat,&accuracy);
		if(r) goto uart2_done;
		//process GPS data, update RTC
		gps_timeout=300;
		}
	uart2_done:;
	}
if(gps_time) //have GPS data
        {
        struct tm *tmp;
        tmp = gmtime(&gps_time);
        if(!rtc_was_set)
                {
                rtc_settime(tmp);
                rtc_was_set=1;
                }
        char times[64];
        strftime(times,64,"%c %Z",tmp);
	char z[256];
        sprintf(z,"GPS%c %s\n",(gps_time==last_fix?'=':'!'),times);
	if(FAT_mounted&&logfile&&(pass&GPSPASS)==0) fputs(z,logfile);
        sprintf(z,"FIX %.5f %.5f ALT %.1fm\n",gps_position[0],gps_position[1],gps_position[2]);
	if(FAT_mounted&&logfile&&(pass&GPSPASS)==0) fputs(z,logfile);
        sprintf(z,"KMH %6.2f  HDG %5.1f\n",ground_speed*3.6f,gps_heading);
	if(FAT_mounted&&logfile&&(pass&GPSPASS)==0) fputs(z,logfile);
        //if((pass&LCDPASS)==0) {}
        }

#endif
//end GPS

delay_ms(1);
led(0,0);led(1,0);led(2,0);
++pass;
}//while
}//main

