#ifndef delay_h_included
#define delay_h_included

#include <stdint.h>

#ifndef SYSTEM_CLOCK
#define SYSTEM_CLOCK (XTAL * PLLMULT)
#endif



//one step takes 3 clock cycles !!! check this on cortex-m0/m3!!!
//possibly longer when executed from FLASH, since flash reading requires more cycles on M3 running at top speed
//SOLVED by defining experimentally obtained value DELAY_CYCLES in Makefile
//NOTE we use a fixed register to prevent compiler from using special registers, which might be slower/faster leading to irregular performance of the inlined instantiations
//NOTE: timing is not completely reliable, there can be flash read delays and instruction times are thus not
//fully predictable
//prevent unpredictable flash delays by placing it into RAM - section .data with extra .ramfunc (maybe not necessary)
//linker generates "veneers" for long branches and the code gets loaded to RAM together with initialized data by the startup routine
//not to waster RAM, this should be moved to a .c file and added into OBJS in the Makefile, inlining here makes no sense

#if defined(__cplusplus) || defined(c_plusplus)
extern "C" {
#endif

#ifdef DELAY_DEBUG
inline void Delay(const uint32_t n_clock3);
#else
extern void Delay(const uint32_t n_clock3);
#endif

#if defined(__cplusplus) || defined(c_plusplus)
}
#endif


#define delay_us(s) Delay((s)*SYSTEM_CLOCK/DELAY_CYCLES/1000000)
#define delay_ms(s) Delay((s)*SYSTEM_CLOCK/DELAY_CYCLES/1000)
#define delay_xs(s) Delay((s)*SYSTEM_CLOCK/DELAY_CYCLES/2000000)


#endif

