/*
 *
 *
 * Ported to STM32F37, direct connection to GPIO port D; (c) 2019 Jiri Pittner <jiri@pittnerovi.com>
 * Code contributed under the GPL v3 license or higher
 *
 *
 *
 *
 ___  _   _  _ __ ___    ___  | |_  ___   _   _ 
/ __|| | | || '_ ` _ \  / _ \ | __|/ _ \ | | | |
\__ \| |_| || | | | | || (_) || |_| (_) || |_| |
|___/ \__,_||_| |_| |_| \___/  \__|\___/  \__, |
                                          |___/ 
										  
	easyT6963 - A Library & Hardware for drive Graphic LCD displays drived by T6963 controller with a GPIO (4 wires)
	------------- GPIO Low Level Routines -------------------
---------------------------------------------------------------------------------------------------------------------
Version history:
0.1b3:First working version, tested with Mega2560
---------------------------------------------------------------------------------------------------------------------
		Copyright (c) 2014, s.u.m.o.t.o.y [sumotoy(at)gmail.com]
---------------------------------------------------------------------------------------------------------------------

    easyT6963 Library is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version. You cannot use for commercial producs
	or military purposes!

    easyT6963 Library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Foobar.  If not, see <http://www.gnu.org/licenses/>.

	Small code portions from Adafruit (Adafruit_GFX).
	https://github.com/adafruit/Adafruit-GFX-Library
	Originated from the library created by Radosław Kwiecień
	http://code.google.com/p/arduino-t6963c/
	Bitmap Byte technique by http://www.holger-klabunde.de
	+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	Version:0.1b3
	+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	Attention! This library needs sumotoy gpio_expander library
	>>>>>>>>> https://github.com/sumotoy/gpio_expander <<<<<<<<<<
	-------------------------------------------------------------
	Hardware needed:
	schematics coming soon...
	-------------------------------------------------------------
	Pins used:
	LCD SIDE			STM32F37 PORT D 
	D0					I/O:0
	D1					I/O:1
	D2					I/O:2
	D3					I/O:3
	D4					I/O:4
	D5					I/O:5
	D6					I/O:6
	D7					I/O:7
	CE					I/O:8
	CD					I/O:9
	WR					I/O:10 
	RD					I/O:11 *note 0
	FS					I/O:12 *note 1
	RV					I/O:13 *note 2
	RST					I/O:14
	Backlight			        I/O:15
*note 0: In fast mode, only write to LCD is used, MCU never reads from LCD
         This allows to employ simpler one-direction 3V3->5V level shifter between MCU and LCD
	 RD pin on the display is then tied to high level.
*note 1: If your LCD doesn't have this pin, forget it.
*note 2: If your LCD doesn't have this pin, forget it.
------------------------------------------------------------------
LCD Contrast: Every LCD has it own way so refer to datasheet, btw most LCD based on T6963 uses a negative offset voltage and unfortunatly sometimes it -12 or -15 so you will need
a StepUp negative circuit since you are probably driving everithing at +5V!
*/

#ifndef T6963_STM32_incl
#define T6963_STM32_incl

#include "delay.h"
#include <inttypes.h>
#include "easyT6963.h"
#include "utility/_gpioConstants.h"//load gpio constants
#define nop asm volatile ("nop\n\t")

class T6963_STM32 : public easyT6963 {
	public:
		T6963_STM32();
		virtual void 		begin(uint8_t width, uint8_t height, uint8_t charsize = T6963_6x8DOTS,uint8_t memsize = 32);
		virtual void 		setBacklight(uint8_t value);
		virtual void		hardwareFontSelect(bool val);//1:6*8, 0:8*8
		virtual void		hardwareScreenReverse(bool val);//1:reverse
	protected:
		virtual void 		sendCommand(byte command);
		virtual void 		sendData(byte data);
		virtual uint8_t 	readData(bool do_CD=1);
		virtual uint8_t 	readData(void) {return readData(1);};
		virtual uint8_t 	checkState(void);
	private:
		void				chipWaitState();
		void 				hardwareReset();
		inline void		T6963_WAIT_DELAY() { delay_us(4); } //slower delay works on tabletop but leads to bad pixels in car - maybe shielded cable is needed; it helps also to lower the edge speeds on the gpio to 10MHz
};


#endif
