/*
    Copyright (C) 2020 Jiri Pittner <jiri@pittnerovi.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/



#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>


#include "fat_interface.h"
#include "sdcard.h"


int media_read(unsigned long sector, unsigned char *buffer, unsigned long sector_count)
{
    unsigned long i;

    for (i=0;i<sector_count;i++)
    {
	SD_readblock_a(sector,buffer);
        sector ++;
        buffer += 512;
    }

    return 1;
}

int media_write(unsigned long sector, unsigned char *buffer, unsigned long sector_count)
{
    unsigned long i;

    for (i=0;i<sector_count;i++)
    {
	SD_writeblock_a(sector,buffer);
        sector ++;
        buffer += 512;
    }

    return 1;
}

FL_FILE *fat_file[FAT_MAXOPEN]; 

void fat_mount(void)
{
    FL_FILE *file;

    // Initialise File IO Library
    fl_init();

    // Attach media access functions to library
    if (fl_attach_media(media_read, media_write) != FAT_INIT_OK)
    {
        printf("ERROR: FAT mount failed\n");
        return; 
    }

uint8_t i;
for(i=0; i<FAT_MAXOPEN; ++i) fat_file[i]=NULL;
}

void fat_umount(void)
{
uint8_t i;
for(i=0; i<FAT_MAXOPEN; ++i) 
	{
	if(fat_file[i]) fl_fclose(fat_file[i]);
	fat_file[i]=NULL;
	}
fl_shutdown();
}



//return FAT_HANDLE_OFFSET + file number, or -1 on error
int _open_fat(const char *pathname, int flags, mode_t mode)
{
uint8_t i;
for(i=0; i<FAT_MAXOPEN; ++i) if(!fat_file[i]) break;
if(i==FAT_MAXOPEN) return -1;
//debug
//printf("_open_fat %s flags %08x\n",pathname,flags);
fat_file[i] = fl_fopen(pathname,(flags&O_APPEND)?"a+":(flags&O_RDONLY?"r":(flags&O_RDWR?"r+":"w+")));
if(!fat_file[i]) return -1;
return i+FAT_HANDLE_OFFSET;
}

ssize_t _write_fat(int fd, const void *buf, size_t count)
{
return fl_fwrite(buf,1,count,fat_file[fd-FAT_HANDLE_OFFSET]);
}

ssize_t _read_fat(int fd, const void *buf, size_t count)
{
return fl_fread(buf,1,count,fat_file[fd-FAT_HANDLE_OFFSET]);
}

int _close(int fd)
{
if(fat_file[fd-FAT_HANDLE_OFFSET]) fl_fclose(fat_file[fd-FAT_HANDLE_OFFSET]);
return 0;
}

off_t _lseek(int fd, off_t offset, int whence)
{
return fl_fseek(fat_file[fd-FAT_HANDLE_OFFSET],offset,whence);
}


int _fstat(int fd, struct stat *statbuf)
{
return -1; //not implemented
}

int fsync(int fd)
{
FL_FILE *file = fat_file[fd-FAT_HANDLE_OFFSET];
if(file) fl_fsync(file);
return 0;
}
