/*
 *     iaploader.c ... STM32F37 IAP USART1 loader
 *
 *      This file GPLv3 Copyright (C) 2014 Jiri Pittner <jiri@pittnerovi.com>
 *      Calls parts of STM example code under their license in separate files
 *
 *      This program is free software: you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation, either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      You should have received a copy of the GNU General Public License
 *      along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include "delay.h"
#include "stm32f37x_gpio.h"
#include "stm32f37x_rcc.h"
#include "stm32f37x_pwr.h"
#include "stm32f37x_conf.h" 

#include "ymodem.h"



void printsimple(USART_TypeDef* USARTx, char *t)
{
while(*t)
	{
	//assert_param(IS_USART_ALL_PERIPH(USARTx));
	//  assert_param(IS_USART_DATA(Data));
	while((USARTx->ISR & USART_ISR_TC)==0);
  	USARTx->TDR = *t++;
	}
}


void scan(USART_TypeDef* USARTx, char *string)
{
char c;
        do      {
                do {
			while((USARTx->ISR & USART_ISR_RXNE)==0);
                        c = USARTx->RDR;
                        } while ((USARTx->ISR & USART_ISR_FE)==1);
                *string++ = c;
		while((USARTx->ISR & USART_ISR_TC)==0);
		USARTx->TDR = c;
                } while ( c != '\r' );
	while((USARTx->ISR & USART_ISR_TC)==0);
	USARTx->TDR = '\n';
        string[-1]=0;
}

#if !defined(USART1_ON_PE01) && !defined(USART1_ON_PA910)
#error CHOOSE USART1 PINOUT
#endif

#if defined(USART1_ON_PE01) &&  defined(USART1_ON_PA910)
#error CHOOSE A UNIQUE USART1 PINOUT
#endif


void usart1_init(uint32_t baud)
{
GPIO_InitTypeDef GPIO_InitStructure;
USART_InitTypeDef USART_InitStructure;

#ifdef USART1_ON_PE01
RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOE, ENABLE);
#endif
#ifdef USART1_ON_PA910
RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA, ENABLE);
#endif

RCC_APB2PeriphClockCmd(RCC_APB2Periph_USART1, ENABLE); 

#ifdef USART1_ON_PE01
/* GPIOE Configuration:  USART1 TX on PE0 RX on PE1 */
        GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0;
        GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
        GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
        GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
        GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL ;
        GPIO_Init(GPIOE, &GPIO_InitStructure);

        GPIO_InitStructure.GPIO_Pin = GPIO_Pin_1;
        GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
        GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
        GPIO_Init(GPIOE, &GPIO_InitStructure);

        GPIO_PinAFConfig(GPIOE, GPIO_PinSource0, GPIO_AF_7);
        GPIO_PinAFConfig(GPIOE, GPIO_PinSource1, GPIO_AF_7);
#endif
#ifdef USART1_ON_PA910
        GPIO_InitStructure.GPIO_Pin = GPIO_Pin_9;
        GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
        GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
        GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
        GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL ;
        GPIO_Init(GPIOA, &GPIO_InitStructure);

        GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10;
        GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
        GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
        GPIO_Init(GPIOA, &GPIO_InitStructure);

        GPIO_PinAFConfig(GPIOA, GPIO_PinSource9, GPIO_AF_7);
        GPIO_PinAFConfig(GPIOA, GPIO_PinSource10, GPIO_AF_7);
#endif


	USART_InitStructure.USART_BaudRate = baud;
	USART_InitStructure.USART_WordLength = USART_WordLength_8b;
	USART_InitStructure.USART_StopBits = USART_StopBits_1;
	USART_InitStructure.USART_Parity = USART_Parity_No;
	USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
	USART_InitStructure.USART_Mode = USART_Mode_Rx |USART_Mode_Tx;
	USART_Init(USART1, &USART_InitStructure);

	USART_Cmd(USART1, ENABLE); // enable USART1

}



int main(void)
{
//test PA0
RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA, ENABLE);
GPIO_InitTypeDef GPIO_InitStructure;
GPIO_InitStructure.GPIO_Pin =  GPIO_Pin_0;
GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_DOWN;
GPIO_Init(GPIOA, &GPIO_InitStructure);

uint8_t wasiap=0;

if(GPIO_InputDataBit(GPIOA,GPIO_Pin_0)) //activate bootloader
	{
	wasiap=1;
	usart1_init(115200);
	printsimple(USART1, "IAP loader\r\n"); 

	FLASH_If_Init();

#ifdef USE_CRC_PERIPH
RCC_AHBPeriphClockCmd(RCC_AHBPeriph_CRC, ENABLE);
  /* DeInit CRC peripheral */
  CRC_DeInit();
  /* Init the INIT register */
  CRC_SetInitRegister(0);
  /* Select 16-bit pol size */
  CRC_PolynomialSizeSelect(CRC_PolSize_16);
  /* Set the polynomial coefficients: x16 + x12 + x5 + 1 used in ymodem file transfer protocol */
  CRC_SetPolynomial(0x1021);
#endif /* USE_CRC_PERIPH */

	//FLASH_If_DisableWriteProtection();
	//FLASH_OB_Launch();

	printsimple(USART1, "Transmit the file via ymodem now (press 'a' to abort)\r\n");
	int32_t Size = 0;
	uint8_t buf[1024] ={0};
	Size = Ymodem_Receive(buf);
	if(Size<0)
		{
		if (Size == -1) printsimple(USART1, "Not enough memory\r\n");
		else if (Size == -2) printsimple(USART1, "Verification failed\r\n");
		else if (Size == -3)  printsimple(USART1, "Aborted by user\r\n");
		else 	 printsimple(USART1, "Failed for some other reason\r\n");
		while(1);
		}
	else printsimple(USART1, "IAP successfull\r\n");
	}

delay_ms(5);
//test whether the application code has been loaded
if (((*(__IO uint32_t*)APPLICATION_ADDRESS) & 0x2FFE0000 ) == 0x20000000) //jump to application (address defined in flash_if.h)
	{
	if(wasiap)
		{
		usart1_init(115200);
		printsimple(USART1, "Start application now\r\n");
		delay_ms(5);
		}
	uint32_t addr =  *(__IO uint32_t*) (APPLICATION_ADDRESS + 4);
	void (*Jump_To_Application) () = (void (*)()) addr;
	__set_MSP(*(__IO uint32_t*) APPLICATION_ADDRESS); //init SP
	Jump_To_Application(); //jump
	}

//if all fails
usart1_init(115200);
printsimple(USART1, "IAP: No application code found!\r\n");
while(1);
}

