/*
    Hardware-dependent sensor routines for avionics

    Copyright (C) 2020 Jiri Pittner <jiri@pittnerovi.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdio.h>
#include <stdint.h>
#include <math.h>
#include "delay.h"
#include "imu_sensors.h"

//hardware setup:
#define MAGNET_HMC5883

//@@@@@check HMC5883 orientation and correct

//@@@@@@@@@@@@@@@flipped y,z axes of BNI088, jak je to s magneto? - correct here
//@@@@@@pak opravit gravity reference a magnetic reference
//@@@@@@@opravit nejdriv toto, pak  znova zkontrolovat ahrs2!!!


#ifdef MAGNET_HMC5883
//HMC5883 routines (I2C1)
#define HMC5883_I2C 0x1e
void HMC5883_init(void)
{
I2C_Wr_byte(HMC5883_I2C,0x0,0x14); //conf A
I2C_Wr_byte(HMC5883_I2C,0x1,0x20); //range 2.5Ga
I2C_Wr_byte(HMC5883_I2C,0x2,0x00); //continuous mode
}

void HMC5883_read(float *m)
{
uint8_t buf[6];
I2C_RdData(HMC5883_I2C,0x03,1,buf,6);
uint8_t i;
for(i=0; i<3; ++i)
	{
	//range 0x20 1090 lsb/gauss
	//range 0x60 660 lsb/gauss
	int16_t tmp=buf[2*i]<<8|buf[2*i+1];
	m[i] = tmp/1090.f;
	}
float tmp; tmp=m[1]; m[1]=m[2]; m[2]=tmp; //swap from the xzy ordering of registers
m[1]= -m[1]; m[2]= -m[2]; //depends on how the sensor is mounted
}
#endif

//BMP180 routines (I2C1)
#define BMP180_I2C 0x77
//see datasheet for the calibration and wait time, slow!

//AM2320 routines (I2C2)
#define AM2320_I2C 0x5c

void AM2320_read(float *t, float *h)
{
uint8_t buf[4];
I2C2_RdData(AM2320_I2C,0x030004,3,buf,4);
int16_t t0=buf[0]<<8|buf[1];
*t = 0.1f*t0;
int16_t h0=buf[2]<<8|buf[3];
*h = 0.1f*h0;
}

//BNO055 routines
#ifdef MAGNET_BNO055
#define BNO055_I2C 0x29
//default units
//100lsb=1ms-2
//1uT=16lsb
//1dps=15lsb
//1degree=16lsb
//1 Quaternion = 2^14 lsb
//1Celsius = 1 lsb
//
void BNO055_init_raw(void)
{
uint8_t chip_id;
I2C_Wr_byte(BNO055_I2C,0x07,0);//page 0
I2C_Wr_byte(BNO055_I2C,0x3d,0);//config mode
I2C_Wr_byte(BNO055_I2C,0x07,1);//page 1
I2C_Wr_byte(BNO055_I2C,0x08,0x12);//8G
I2C_Wr_byte(BNO055_I2C,0x0a,0x03);//gyro_config0
I2C_Wr_byte(BNO055_I2C,0x09,0x19);//mag_config
I2C_Wr_byte(BNO055_I2C,0x07,0);//page 0
I2C_Wr_byte(BNO055_I2C,0x3b,0);//units
I2C_Wr_byte(BNO055_I2C,0x3d,0x07);//oper mode
I2C_RdData(BNO055_I2C,0,1,&chip_id,1); printf("BNO055 chip id should be A0 is %02x\n",chip_id);
}

void BNO055_init_fusion(void)
{
I2C_Wr_byte(BNO055_I2C,0x07,0);//page 0
I2C_Wr_byte(BNO055_I2C,0x3d,0);//config mode
uint8_t chip_id;
I2C_RdData(BNO055_I2C,0,1,&chip_id,1);
printf("BNO055 chip id should A0 is %02x\n",chip_id);
I2C_Wr_byte(BNO055_I2C,0x07,1);//page 1
I2C_Wr_byte(BNO055_I2C,0x08,0x12);//8G
I2C_Wr_byte(BNO055_I2C,0x07,0);//page 0
I2C_Wr_byte(BNO055_I2C,0x3b,0);//units
I2C_Wr_byte(BNO055_I2C,0x3d,0x0c);//oper mode
}

void BNO055_read_raw(float *acc, float *gyro, float *mag)
{
int16_t buf[3][3]; //a,m,g, xyz
I2C_RdData(BNO055_I2C,0x08,1,(uint8_t *)&buf[0][0],3*3*sizeof(int16_t));
uint8_t i;
printf("BNO055 ");
for(i=0; i<3; ++i)
	{
	printf("%6d %6d %6d ",buf[i][0],buf[i][1],buf[i][2]);
	}
printf("\n");
for(i=0; i<3; ++i)
	{
	if(buf[0][i]<30000) acc[i] = buf[0][i] * 0.01f; 
	if(buf[1][i]<30000) mag[i] = buf[1][i]/16.f/100.f; //convert from microtesla to gauss
	if(buf[2][i]<30000) gyro[i] = buf[2][i]/15.f;
	}
}

void BNO055_read_euler(float *e)
{
int16_t buf[3];
I2C_RdData(BNO055_I2C,0x1a,1,(uint8_t *)&buf[0],3*sizeof(int16_t));
printf("BNO055 %d %d %d\n",buf[0],buf[1],buf[2]);
uint8_t i;
for(i=0; i<3; ++i) e[i] = buf[i] /16.f;
}

void BNO055_read_quaternion(float *q)
{
int16_t buf[4];
I2C_RdData(BNO055_I2C,0x20,1,(uint8_t *)&buf[0],4*sizeof(int16_t));
printf("BNO055 %d %d %d %d\n",buf[0],buf[1],buf[2],buf[3]);
uint8_t i;
for(i=0; i<4; ++i) q[i] = buf[i] /16384.f;
}

#endif


//BMI088 routines, module's default I2C addresses on I2C2
#define BMI088_I2C_accel 0x19 
#define BMI088_I2C_gyro 0x69 
static uint8_t BMI088_a_range, BMI088_g_range;

void BMI088_init(uint8_t a_range, uint8_t g_range)
{
BMI088_a_range=a_range;
BMI088_g_range=g_range;
delay_ms(1);
I2C_Wr_byte(BMI088_I2C_accel,0x7d,4);
delay_ms(50);

//set ranges, leave default filters
I2C_Wr_byte(BMI088_I2C_accel,0x40,0x88); //lowest bandwidth, 100Hz output rate
I2C_Wr_byte(BMI088_I2C_accel,0x41,a_range);
I2C_Wr_byte(BMI088_I2C_gyro,0x10,0x07); //lowest bandwidth to prevent fluctuations
I2C_Wr_byte(BMI088_I2C_gyro,0x0f,g_range);

//gyro selftest
I2C_Wr_byte(BMI088_I2C_gyro,0x3c,1);
uint8_t st;
delay_ms(100);
I2C_RdData(BMI088_I2C_gyro,0x3c,1,&st,1);
//printf("gyro selftest register %02x should be 0x12\n",st);
}

//returns in g's and omega in rad/sec and sensortime in secs
void BMI088_read(float *accel, float *omega, float *stime)
{
//assuming little endian processor, reading LSB,MSB order into shorts
struct
	{
	int16_t a[3];
	uint32_t t;
	} __attribute__((packed)) adata;
adata.t=0;
int16_t g[3];
uint8_t selftesta;

I2C_RdData(BMI088_I2C_accel,0x12,1,(uint8_t *)&adata,sizeof(adata)-1);
I2C_RdData(BMI088_I2C_accel,0x0,1,&selftesta,1);
I2C_RdData(BMI088_I2C_gyro,0x02,1,(uint8_t *)g,3*sizeof(int16_t));

uint8_t selftest;
I2C_RdData(BMI088_I2C_gyro,0x0,1,&selftest,1);
uint8_t i;
for(i=0; i<3; ++i)
	{
	accel[i] = G_normal * 3.f*(1<<BMI088_a_range)*adata.a[i]/32768.f;
	//printf("accel test %x %05d %05d %05d ",selftest,adata.a[0],adata.a[1],adata.a[2]);
	omega[i] = M_PI/180.*2000.f*g[i]/(float)(1UL<<15+BMI088_g_range); 
	//printf("gyro test %x %x %x %x\n",selftest,g[0],g[1],g[2]);
	}
//this depends on how the chip is mounted in the device!!!
accel[1] = -accel[1];
accel[2] = -accel[2];
omega[1] = -omega[1];
omega[2] = -omega[2];
*stime=adata.t/25600.f;
}

/*test BNO055
{
Vec3<float> acc,gyro,mag,eul;
Quaternion<float> quat;
//BNO055_read_raw(acc,gyro,mag);
BNO055_read_euler(eul);
uint8_t chip_id;
I2C_RdData(BNO055_I2C,0,1,&chip_id,1);
printf("BNO055 chip id should A0 is %02x euler %6.2f %6.2f %6.2f\n",chip_id,eul[0],eul[1],eul[2]);
}
*/

/*
test AM2320
{
float t,h;
AM2320_read(&t,&h);
printf("temp %f humidity %f\n",t,h);
}
*/




//high level hardware independent routines

void init_9dof(void)
{
BMI088_init(1,2);
#ifdef MAGNET_HMC5883
HMC5883_init();
#endif
#ifdef MAGNET_BNO055
BNO055_init_raw();
#endif
}




void read_9dof(float *acceleration, float *omega, float *magnetic, float *time, float *t0, float *dt)
{
BMI088_read(acceleration,omega,time);
if(t0)
	{
	*dt= *time - *t0;
	if(*dt <0) *dt += 655.36f; //counter overflow
	*t0 = *time;
	}
#ifdef MAGNET_HMC5883
HMC5883_read(magnetic);
#endif
#ifdef MAGNET_BNO055
{
float bno_acc[3],bno_gyro[3];
BNO055_read_raw(bno_acc,bno_gyro,magnetic);
}
#endif
}
