/*
 * *      kbd_spi1.c
 * *
 * *      Copyright (C) 2012 Jiri Pittner <jiri@pittnerovi.com>
 * *
 * *      This program is free software: you can redistribute it and/or modify
 * *      it under the terms of the GNU General Public License as published by
 * *      the Free Software Foundation, either version 3 of the License, or
 * *      (at your option) any later version.
 * *
 * *      This program is distributed in the hope that it will be useful,
 * *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 * *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * *      GNU General Public License for more details.
 * *      You should have received a copy of the GNU General Public License
 * *      along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * *
 * *
 * */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include "usart.h"
#include "delay.h"
#include "kbd_spi1.h"

static volatile uint8_t kbd_line;

void kbd_spi1_init(void)
{
initSpi1();
MCP23S17_Setiocon(1,0);
MCP23S17_Write(1,0x00);
MCP23S17_Setpullup(1,0xf0);
//setting for interrupt generation by MCP23S17
MCP23S17_Setdefault(1,0xf0);
MCP23S17_Setinterrupt(1,0xf0);
MCP23S17_Setdirection(1,1<<(kbd_line=3));
}


static volatile uint8_t kbd_line_pressed;
static volatile uint8_t inter=0;
static volatile void (*kbd_spi1_callback)(void)=NULL;

void EXTI9_5_IRQHandler(void)
{
EXTI_ClearFlag(EXTI_Line7);
if(inter==0)
	{
	kbd_line_pressed=kbd_line;
	inter=1;
	if(kbd_spi1_callback) (*kbd_spi1_callback)();
	}
}

static const char keymap[4][4]=
{{'1','2','3','A'},
{'4','5','6','B'},
{'7','8','9','C'},
{'*','0','#','D'},
};

char decode_char(uint8_t line, uint8_t x)
{
x = ~x >> 4;
uint8_t lx;
for(lx=0; lx<4; ++lx) if(x&1) break; else x>>=1;
return keymap[line][lx];
}

char kbd_spi1_getchar(void)
{
inter=0;
MCP23S17_Readinterrupt(1);

while(1)
    	{
	uint8_t tmp=(kbd_line+1)&3;
	NVIC->ICER[EXTI9_5_IRQn >> 0x05] = (uint32_t)0x01 << (EXTI9_5_IRQn & (uint8_t)0x1F); //disable interrupt
	MCP23S17_Setdirection(1,1<<tmp); 
	MCP23S17_Setdirection(1,1<<tmp); 
	delay_us(100);
	kbd_line = tmp;
	NVIC->ISER[EXTI9_5_IRQn >> 0x05] = (uint32_t)0x01 << (EXTI9_5_IRQn & (uint8_t)0x1F); //enable interrupt
	delay_ms(1);
	if(inter) break;
    	}
return decode_char(kbd_line_pressed,MCP23S17_Readinterrupt(1));
}

void kbd_trap_line(uint8_t line, void (*callback)(void))
{
kbd_spi1_callback = callback;
NVIC->ICER[EXTI9_5_IRQn >> 0x05] = (uint32_t)0x01 << (EXTI9_5_IRQn & (uint8_t)0x1F); //disable interrupt
inter=0;
kbd_line = line;
MCP23S17_Setdirection(1,1<<line);
MCP23S17_Setdirection(1,1<<line);
delay_us(100);
NVIC->ISER[EXTI9_5_IRQn >> 0x05] = (uint32_t)0x01 << (EXTI9_5_IRQn & (uint8_t)0x1F); //enable interrupt
MCP23S17_Readinterrupt(1);
}


#ifdef standalone

void pressed(void)
{
print(USART1,"!");
}

int main(void)
{
usart1_init(115200);
MCP23S17_reset();
delay_ms(5);
lcd_spi1_init();
kbd_spi1_init();
delay_ms(20);
lcd_spi1_clear();
lcd_spi1_print("hello world");
print(USART1, "hello world\n"); 

kbd_trap_line(3,pressed);
delay_ms(3000);
if(inter)
	{
	char c[20]="  pressed\n";
	c[0] = decode_char(kbd_line_pressed,MCP23S17_Readinterrupt(1));
	print(USART1, c);
	}

uint8_t ii=0;
  while (1)
   {
lcd_spi1_cursor(0,0);
lcd_spi1_print("kbd read         ");
char c[2]=" ";
c[0]=kbd_spi1_getchar();
print(USART1,c);
lcd_spi1_cursor(1,ii&0x0f);
lcd_spi1_print(c);
delay_ms(150);
++ii;
  }
}

#endif
