/*
 *      lcd_spi1.c ... HD44780 LCD for STM32F37 via spi1 and MCP23S17 port A
 *
 *      Copyright (C) 2012 Jiri Pittner <jiri@pittnerovi.com>
 *
 *      This program is free software: you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation, either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      You should have received a copy of the GNU General Public License
 *      along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 */


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include "lcd_spi1.h"




// LCD routines
/*located at port A of MCP23S17
        A0-3=DB4-7 VIA 1k resistors, since the display is powered to 5V for the case we would read to prevent 5V go through internal diode to 3V3!!!
        A4=E
        A5=RW
        A6=RS 
        A7=light (via NPN transistor - 150mA)
*/


#define lcd_spi1_delay 50

volatile uint8_t spi1_light=0;

void lcd_spi1_w4bit(uint8_t rs, uint8_t x)
{
uint8_t val=1<<4|spi1_light|rs<<6;
MCP23S17_Write(0,val);
delay_xs(lcd_spi1_delay);
val |= (x&0x0f);
MCP23S17_Write(0,val);
delay_xs(lcd_spi1_delay);
MCP23S17_ResetBits(0, 1<<4);
delay_xs(lcd_spi1_delay);
MCP23S17_SetBits(0, 1<<4);
delay_xs(lcd_spi1_delay);
}

void lcd_spi1_init4bit(void)
{
delay_xs(20000);
lcd_spi1_w4bit(0,3);
delay_xs(10000);
lcd_spi1_w4bit(0,3);
delay_xs(500);
lcd_spi1_w4bit(0,3);
lcd_spi1_w4bit(0,2);
}

void lcd_spi1_wbyte(uint8_t rs, uint8_t x)
{
lcd_spi1_w4bit(rs,x>>4);
lcd_spi1_w4bit(rs,x);
}

void lcd_spi1_clear(void)
{
lcd_spi1_wbyte(0,0x01);
delay_xs(30000);
}


void lcd_spi1_init(void)
{
//hardware init
initSpi1();
MCP23S17_Setdirection(0,0xff);

//software init
lcd_spi1_init4bit();
lcd_spi1_wbyte(0,0x28);
lcd_spi1_wbyte(0,0x08);
lcd_spi1_clear();
lcd_spi1_wbyte(0,0x06);
lcd_spi1_wbyte(0,0x0c);
}

void lcd_spi1_print(char *t)
{
while(*t) if(*t=='\n') {++t; lcd_spi1_wbyte(0,0xc0);} else lcd_spi1_wbyte(1,*t++);
}



void lcd_spi1_printn(char *t, uint8_t n)
{
uint8_t l=0;
while(*t) {lcd_spi1_wbyte(1,*t++); ++l;}
while(l<n) {lcd_spi1_wbyte(1,' '); ++l;}
}

void lcd_spi1_cursor(uint8_t r, uint8_t c)
{
lcd_spi1_wbyte(0,0x80+c+(r<<6));
}


#ifdef standalone


int main(void)
{
usart1_init(115200);

lcd_spi1_init();

//spi1_light=1<<7;

char c[64];
print(USART1, "lcd_spi1 testik\n"); 
print(USART1, "lcd_spi1 init done\n");

  while (1)
  {
lcd_spi1_cursor(0,0);
lcd_spi1_print("milliohmmeter\nhello world!\n");
print(USART1, "hello world!\n");
delay_ms(1000);
  }
}

#endif

