/*
 * *      mcp23017.c
 * *
 * *      Copyright (C) 2015 Jiri Pittner <jiri@pittnerovi.com>
 * *
 * *      This program is free software: you can redistribute it and/or modify
 * *      it under the terms of the GNU General Public License as published by
 * *      the Free Software Foundation, either version 3 of the License, or
 * *      (at your option) any later version.
 * *
 * *      This program is distributed in the hope that it will be useful,
 * *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 * *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * *      GNU General Public License for more details.
 * *      You should have received a copy of the GNU General Public License
 * *      along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * *
 * *
 * */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include "mcp23017.h"

#define MCP_read 1

//TODO: support interrupt which is connected to a GPIO PIN
//single chip with address 0 assumed

#define MCP_ADDR 0x20
//gets left-shifted by I2C routines, addresses MCP23017 chip with a0-2 bits =0

static uint8_t MCP23017_out[2]; //cache to avoid readbacks //would NOT work for multiple addresses

void I2C_Wr3(uint8_t byte2, uint8_t byte3)
{
uint8_t msg[2];
msg[0]=byte2;
msg[1]=byte3;
uint32_t r=I2C_WrBuf(MCP_ADDR,msg,2);
if(r) printf("I2C MCP23017 communication failed\n");
}

void MCP23017_Setdirection(uint8_t port, uint8_t bits) //1==out
{
I2C_Wr3(port,~bits);
}

void MCP23017_Setpullup(uint8_t port, uint8_t bits)
{
I2C_Wr3(0x0c+port,bits);
}

void MCP23017_Setpolarity(uint8_t port, uint8_t bits)
{
I2C_Wr3(0x02+port,bits);
}

void MCP23017_Setiocon(uint8_t port, uint8_t bits)
{
I2C_Wr3(0x0a+port,bits);
}

void MCP23017_Setdefault(uint8_t port, uint8_t bits)
{
I2C_Wr3(0x06+port,bits);
I2C_Wr3(0x08+port,bits);
}

void MCP23017_Setinterrupt(uint8_t port, uint8_t bits)
{
I2C_Wr3(0x04+port,bits);
}


void MCP23017_push(uint8_t port)
{
I2C_Wr3(0x12+port,MCP23017_out[port]);
}

void MCP23017_Write(uint8_t port, uint8_t bits)
{
MCP23017_out[port]=bits;
MCP23017_push(port);
}

void MCP23017_ResetBits(uint8_t port, uint8_t bits)
{
MCP23017_out[port] &= ~bits;
MCP23017_push(port);
}

void MCP23017_SetBits(uint8_t port, uint8_t bits)
{
MCP23017_out[port] |= bits;
MCP23017_push(port);
}


void MCP23017_XorBits(uint8_t port, uint8_t bits)
{
MCP23017_out[port] ^= bits;
MCP23017_push(port);
}

uint8_t MCP23017_Read0(uint8_t reg)
{
uint8_t buf[4];
I2C_RdData(MCP_ADDR,reg,1,buf,1);
return buf[0];
}

uint8_t MCP23017_Read(uint8_t port)
{
return MCP23017_Read0(0x12+port);
}


uint8_t MCP23017_Readinterrupt(uint8_t port)
{
return MCP23017_Read0(0x10+port);
}


void MCP23017_reset(void)
{
// i2c init has to be called extra
//reset pin connected to MCU reset, not to gpio
}
