/*
 * *      mcp23s17.c
 * *
 * *      Copyright (C) 2012 Jiri Pittner <jiri@pittnerovi.com>
 * *
 * *      This program is free software: you can redistribute it and/or modify
 * *      it under the terms of the GNU General Public License as published by
 * *      the Free Software Foundation, either version 3 of the License, or
 * *      (at your option) any later version.
 * *
 * *      This program is distributed in the hope that it will be useful,
 * *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 * *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * *      GNU General Public License for more details.
 * *      You should have received a copy of the GNU General Public License
 * *      along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * *
 * *
 * */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include "mcp23s17.h"

#define MCP_addr 0x40
#define MCP_read 1

static uint8_t MCP23S17_out[2]; //cache to avoid readbacks

void MCP23S17_Setdirection(uint8_t port, uint8_t bits) //1==out
{
uint8_t msg[3] = {MCP_addr,port,~bits};
spi1_write(msg,3);
}

void MCP23S17_Setpullup(uint8_t port, uint8_t bits)
{
uint8_t msg[3] = {MCP_addr,0x0c+port,bits};
spi1_write(msg,3);
}

void MCP23S17_Setiocon(uint8_t port, uint8_t bits)
{
uint8_t msg[3] = {MCP_addr,0x0a+port,bits};
spi1_write(msg,3);
}

void MCP23S17_Setdefault(uint8_t port, uint8_t bits)
{
uint8_t msg[3] = {MCP_addr,0x06+port,bits};
spi1_write(msg,3);
uint8_t msg2[3] = {MCP_addr,0x08+port,bits};
spi1_write(msg2,3);

}

void MCP23S17_Setinterrupt(uint8_t port, uint8_t bits)
{
uint8_t msg[3] = {MCP_addr,0x04+port,bits};
spi1_write(msg,3);
}




void MCP23S17_push(uint8_t port)
{
uint8_t msg[3] = {MCP_addr,0x12+port,MCP23S17_out[port]};
spi1_write(msg,3);
}

void MCP23S17_Write(uint8_t port, uint8_t bits)
{
MCP23S17_out[port]=bits;
MCP23S17_push(port);
}

void MCP23S17_ResetBits(uint8_t port, uint8_t bits)
{
MCP23S17_out[port] &= ~bits;
MCP23S17_push(port);
}

void MCP23S17_SetBits(uint8_t port, uint8_t bits)
{
MCP23S17_out[port] |= bits;
MCP23S17_push(port);
}



void MCP23S17_XorBits(uint8_t port, uint8_t bits)
{
MCP23S17_out[port] ^= bits;
MCP23S17_push(port);
}

uint8_t MCP23S17_Read(uint8_t port)
{
uint8_t msg[4] = {MCP_addr|MCP_read,0x12+port,0,0};
spi1_write(msg,4);
//printf("RX %x %x %x %x\n",msg[0],msg[1],msg[2],msg[3]);
return msg[3];
}

uint8_t MCP23S17_Readinterrupt(uint8_t port)
{
uint8_t msg[4] = {MCP_addr|MCP_read,0x10+port,0,0};
spi1_write(msg,4);
return msg[3];
}

void MCP23S17_reset(void)
{
RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOB, ENABLE);
GPIO_InitTypeDef GPIO_InitStruct;
    GPIO_StructInit(&GPIO_InitStruct);
    GPIO_InitStruct.GPIO_OType = GPIO_OType_OD; //ext. pullup
    GPIO_InitStruct.GPIO_PuPd = GPIO_PuPd_NOPULL;
    GPIO_InitStruct.GPIO_Speed = GPIO_Speed_50MHz;
GPIO_InitStruct.GPIO_Mode = GPIO_Mode_OUT;
    GPIO_InitStruct.GPIO_Pin = GPIO_Pin_8;
    GPIO_Init(GPIOB, &GPIO_InitStruct);

GPIO_ResetBits(GPIOB, GPIO_Pin_8);
delay_ms(10);
GPIO_SetBits(GPIOB, GPIO_Pin_8);
delay_ms(10);
}
