#ifndef _MCP24AA_H_
#define _MCP24AA_H_

#include <stdint.h>
#include "i2c1.h"

#if defined(__cplusplus) || defined(c_plusplus)
extern "C" {
#endif

//here the rw bit is shifted in so we shift right
#define MCP24AA_ADDR    (0xA0>>1)          // 10100000

//define this 2 for larger eeprom which needs 2-byte address
#define EEPROM_ADDR_SIZE 1

//for checking allowed address range
#define MCP24AA_MAXADDR 0xFFF


typedef enum
{
  MCP24AA_ERROR_OK = 0,               // Everything executed normally
  MCP24AA_ERROR_I2CINIT,              // Unable to initialise I2C
  MCP24AA_ERROR_I2CBUSY,              // I2C already in use
  MCP24AA_ERROR_ADDRERR,              // Address out of range
  MCP24AA_ERROR_BUFFEROVERFLOW,       // Max 8 bytes can be read/written in one operation
  MCP24AA_ERROR_LAST
}
mcp24aaError_e;

mcp24aaError_e mcp24aaInit (void);
mcp24aaError_e mcp24aaReadBuffer (uint16_t address, void *buffer, uint32_t bufferLength);
mcp24aaError_e mcp24aaWriteBuffer (uint16_t address, void *buffer, uint32_t bufferLength);
mcp24aaError_e mcp24aaReadByte (uint16_t address, uint8_t *buffer);
mcp24aaError_e mcp24aaWriteByte (uint16_t address, uint8_t value);

#if defined(__cplusplus) || defined(c_plusplus)
}
#endif

#endif
