/*
 * *      spi.c
 * *
 * *      Copyright (C) 2012 Jiri Pittner <jiri@pittnerovi.com>
 * *
 * *      This program is free software: you can redistribute it and/or modify
 * *      it under the terms of the GNU General Public License as published by
 * *      the Free Software Foundation, either version 3 of the License, or
 * *      (at your option) any later version.
 * *
 * *      This program is distributed in the hope that it will be useful,
 * *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 * *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * *      GNU General Public License for more details.
 * *      You should have received a copy of the GNU General Public License
 * *      along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * *
 * *
 * */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include "milliohm_spi.h"
#include "delay.h"

//SPI1 AF5 - piny  SCK=PB3, miso=pb4, mosi=pb5, cs=pb6 (not AF), interrupt=pb7
//
void initSpi1(void) {
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_SPI1, ENABLE);
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOB, ENABLE);

    SPI_I2S_DeInit(SPI1);

    GPIO_InitTypeDef GPIO_InitStruct;
    GPIO_StructInit(&GPIO_InitStruct);
    GPIO_InitStruct.GPIO_OType = GPIO_OType_PP;
    GPIO_InitStruct.GPIO_PuPd = GPIO_PuPd_NOPULL;
    GPIO_InitStruct.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStruct.GPIO_Pin = GPIO_Pin_3|GPIO_Pin_4|GPIO_Pin_5;
    GPIO_InitStruct.GPIO_Mode = GPIO_Mode_AF;
    GPIO_Init(GPIOB, &GPIO_InitStruct);
    GPIO_InitStruct.GPIO_Mode = GPIO_Mode_OUT;
    GPIO_InitStruct.GPIO_Pin = GPIO_Pin_6;
    GPIO_Init(GPIOB, &GPIO_InitStruct);
    GPIO_InitStruct.GPIO_Mode = GPIO_Mode_IN;
    GPIO_InitStruct.GPIO_Pin = GPIO_Pin_7;
    GPIO_Init(GPIOB, &GPIO_InitStruct);


    GPIO_PinAFConfig(GPIOB, GPIO_PinSource3, GPIO_AF_5);
    GPIO_PinAFConfig(GPIOB, GPIO_PinSource4, GPIO_AF_5);
    GPIO_PinAFConfig(GPIOB, GPIO_PinSource5, GPIO_AF_5);

    SPI_InitTypeDef SPI_InitStructure;
    SPI_InitStructure.SPI_Direction = SPI_Direction_2Lines_FullDuplex;
    SPI_InitStructure.SPI_DataSize = SPI_DataSize_8b;
    SPI_InitStructure.SPI_CPOL = SPI_CPOL_Low;
    SPI_InitStructure.SPI_CPHA = SPI_CPHA_1Edge;
    SPI_InitStructure.SPI_NSS = SPI_NSS_Soft;
    SPI_InitStructure.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_32; //set speed here
    SPI_InitStructure.SPI_FirstBit = SPI_FirstBit_MSB;
    SPI_InitStructure.SPI_CRCPolynomial = 7;
    SPI_InitStructure.SPI_Mode = SPI_Mode_Master;
    SPI_Init(SPI1, &SPI_InitStructure);

    SPI_Cmd(SPI1, ENABLE);
//	SPI_SSOutputCmd(SPI1,ENABLE);

//interrupt on pb7
RCC_APB2PeriphClockCmd(RCC_APB2Periph_SYSCFG, ENABLE);
SYSCFG_EXTILineConfig(EXTI_PortSourceGPIOB, EXTI_PinSource7);
EXTI_InitTypeDef EXTI_InitStruct;
  EXTI_InitStruct.EXTI_Line = EXTI_Line7;
  EXTI_InitStruct.EXTI_Mode = EXTI_Mode_Interrupt;
  EXTI_InitStruct.EXTI_Trigger = EXTI_Trigger_Falling;
  EXTI_InitStruct.EXTI_LineCmd = ENABLE;
EXTI_Init(&EXTI_InitStruct);

NVIC_InitTypeDef NVIC_InitStructure;
NVIC_InitStructure.NVIC_IRQChannel = EXTI9_5_IRQn;
        NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
        NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
        NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
        NVIC_Init(&NVIC_InitStructure);

}

u8 spi1_GetByte(void) {
    u32 LIS302DLTimeout = 0x1000;
    /* Wait to receive a Byte */
    while (SPI_I2S_GetFlagStatus(SPI1,  SPI_I2S_FLAG_RXNE) == RESET) {
        if((LIS302DLTimeout--) == 0) return 0x00;
    }

    /* Return the Byte read from the SPI bus */
    uint8_t r= (u8)SPI_ReceiveData8(SPI1);
return r;
}


void spi1_SendByte(u8 byte) {
    u32 LIS302DLTimeout = 0x1000;

    while (SPI_I2S_GetFlagStatus(SPI1, SPI_I2S_FLAG_TXE) == RESET) {
        if((LIS302DLTimeout--) == 0) return;
    }

    /* Send a Byte through the SPI peripheral */
    SPI_SendData8(SPI1, byte);
}


void spi1_write(uint8_t *msg, uint8_t len) 
{
GPIO_ResetBits(GPIOB, GPIO_Pin_6);
delay_us(1);
while (SPI_I2S_GetFlagStatus(SPI1,  SPI_I2S_FLAG_RXNE) == SET) SPI_ReceiveData8(SPI1); //flush rx fifo
while(len--) 
	{
	spi1_SendByte(*msg);
	*msg++ = spi1_GetByte();
	}
FlagStatus s;
do {
        s= SPI_I2S_GetFlagStatus(SPI1,SPI_I2S_FLAG_BSY);
        }while(s==SET);
delay_us(3);
GPIO_SetBits(GPIOB, GPIO_Pin_6);
}



#ifdef standalone

#include "usart.h"
#include "delay.h"

int main(void)
{
usart2_init(115200);
inuart2line=inuart2len=0;


RCC_ClocksTypeDef RCC_ClocksStatus;

print(USART2, "\nspi testik\n"); 
initSpi1();
print(USART2, "init done\n"); 

uint8_t msg1[3] = {0x40,0x01,0xfe};
uint8_t msg2[3] = {0x40,0x13,0x1};
uint8_t msg3[3] = {0x40,0x13,0x0};

        spi1_write(msg1,3);
	delay_ms(1000);

while (1)
  {
  print(USART2, "toggle bit\n");
  	spi1_write(msg2,3);
        delay_ms(500);
	spi1_write(msg3,3);
        delay_ms(500);
    
  }
}

#endif
