/*
    rotational encoder at PE2,3,4 and using timer5

    Copyright (C) 2020 Jiri Pittner <jiri@pittnerovi.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdio.h>
#include <stdint.h>

#include "stm32f37x_gpio.h"
#include "stm32f37x_rcc.h"
#include "stm32f37x_pwr.h"
#include "stm32f37x_tim.h"
#include "stm32f37x_exti.h"
#include "stm32f37x_misc.h"
#include "stm32f37x_syscfg.h"


#include "delay.h"


/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

//rotcoder and pushbuttons routines
//
//right:
////I0:01
////I1:00
////I0:10
////I1:11
////
////left:
////I1:10
////I0:00
////I1:01
////I0:11
////



static void (*process_press)(uint16_t millisec);
static void (*process_rotation)(uint8_t dir);
static void (*process_button)(uint8_t, uint16_t);


volatile static int8_t rotcount=0;
void rotcoder(uint8_t interrupt, uint8_t pins)
{
if(interrupt)
        {
        if(pins==0 || pins==3) ++rotcount;
        else --rotcount;
        }
else
        {
        if(pins==0 || pins==3) --rotcount;
        else ++rotcount;
        }
if(rotcount==4) {(*process_rotation)(1); rotcount=0;}
if(rotcount== -4) {(*process_rotation)(0); rotcount=0;}
}




void init_rotcoder_buttons(void (*process_press0)(uint16_t), void (*process_rotation0)(uint8_t), void (*process_button0)(uint8_t, uint16_t)  )
{
//setup callbacks
process_press=process_press0;
process_rotation=process_rotation0;
process_button=process_button0;

//init gpio
GPIO_InitTypeDef GPIO_InitStructure;
RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOE, ENABLE);

GPIO_InitStructure.GPIO_Pin = GPIO_Pin_2|GPIO_Pin_3|GPIO_Pin_4|GPIO_Pin_5|GPIO_Pin_6;
GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
GPIO_Init(GPIOE, &GPIO_InitStructure);

RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOC, ENABLE);
GPIO_InitStructure.GPIO_Pin = GPIO_Pin_13;
GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
GPIO_Init(GPIOC, &GPIO_InitStructure);

delay_ms(1);

//init interrupts
RCC_APB2PeriphClockCmd(RCC_APB2Periph_SYSCFG, ENABLE);

SYSCFG_EXTILineConfig(EXTI_PortSourceGPIOE,EXTI_PinSource2);
SYSCFG_EXTILineConfig(EXTI_PortSourceGPIOE,EXTI_PinSource3);
SYSCFG_EXTILineConfig(EXTI_PortSourceGPIOE,EXTI_PinSource4);
SYSCFG_EXTILineConfig(EXTI_PortSourceGPIOE,EXTI_PinSource5);
SYSCFG_EXTILineConfig(EXTI_PortSourceGPIOE,EXTI_PinSource6);
SYSCFG_EXTILineConfig(EXTI_PortSourceGPIOC,EXTI_PinSource13);

EXTI_InitTypeDef EXTI_InitStructure;
EXTI_InitStructure.EXTI_Line = EXTI_Line2|EXTI_Line3|EXTI_Line4|EXTI_Line5|EXTI_Line6|EXTI_Line13;
EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Rising_Falling;
EXTI_InitStructure.EXTI_LineCmd = ENABLE;
EXTI_Init(&EXTI_InitStructure);

NVIC_InitTypeDef NVIC_InitStructure;
NVIC_InitStructure.NVIC_IRQChannel = EXTI2_TS_IRQn;
NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0x0f;
NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0x0F;
NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
NVIC_Init(&NVIC_InitStructure);
NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0x0e;
NVIC_InitStructure.NVIC_IRQChannel = EXTI3_IRQn;
NVIC_Init(&NVIC_InitStructure);
NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0x0d;
NVIC_InitStructure.NVIC_IRQChannel = EXTI4_IRQn;
NVIC_Init(&NVIC_InitStructure);
NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0x0c;
NVIC_InitStructure.NVIC_IRQChannel = EXTI9_5_IRQn;
NVIC_Init(&NVIC_InitStructure);
NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0x0b;
NVIC_InitStructure.NVIC_IRQChannel = EXTI15_10_IRQn;
NVIC_Init(&NVIC_InitStructure);



//init timer 5 for button press
RCC_APB1PeriphResetCmd(RCC_APB1Periph_TIM5, ENABLE);
RCC_APB1PeriphResetCmd(RCC_APB1Periph_TIM5, DISABLE);
RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM5, ENABLE);
TIM_TimeBaseInitTypeDef TIM_TimeBaseInitStructure;
TIM_TimeBaseStructInit(&TIM_TimeBaseInitStructure);
TIM_TimeBaseInitStructure.TIM_Prescaler=1;
TIM_TimeBaseInit(TIM5,&TIM_TimeBaseInitStructure);
TIM_InternalClockConfig(TIM5);
TIM_Cmd(TIM5,ENABLE);

}

void EXTI2_TS_IRQHandler(void)
{
EXTI_ClearITPendingBit(EXTI_Line2); 
rotcoder(0,(GPIO_ReadInputData(GPIOE)>>2)&3);
}

void EXTI3_IRQHandler(void) 
{
EXTI_ClearITPendingBit(EXTI_Line3);
rotcoder(1,(GPIO_ReadInputData(GPIOE)>>2)&3);
}

static volatile uint32_t rotpressed;

void EXTI4_IRQHandler(void) //rotate push
{
EXTI_ClearITPendingBit(EXTI_Line4);
if(GPIO_InputDataBit(GPIOE,GPIO_Pin_4)) (*process_press)((uint32_t)(TIM5->CNT-rotpressed)/(SystemCoreClock/1000));
else rotpressed=TIM5->CNT;
}

static volatile uint32_t butpressed[3];

void EXTI9_5_IRQHandler(void) //button0,1
{
if(EXTI_GetITStatus(EXTI_Line5)==SET)
	{
	EXTI_ClearITPendingBit(EXTI_Line5);
	if(GPIO_InputDataBit(GPIOE,GPIO_Pin_5)) (*process_button)(0,(uint32_t)(TIM5->CNT-butpressed[0])/(SystemCoreClock/1000));
	else butpressed[0]=TIM5->CNT;
	}
if(EXTI_GetITStatus(EXTI_Line6)==SET)
        {
        EXTI_ClearITPendingBit(EXTI_Line6);
        if(GPIO_InputDataBit(GPIOE,GPIO_Pin_6)) (*process_button)(1,(uint32_t)(TIM5->CNT-butpressed[1])/(SystemCoreClock/1000));       
        else butpressed[1]=TIM5->CNT;
        }
}

void EXTI15_10_IRQHandler(void) //button2
{
if(EXTI_GetITStatus(EXTI_Line13)==SET)
	{
	EXTI_ClearITPendingBit(EXTI_Line13);
	if(GPIO_InputDataBit(GPIOC,GPIO_Pin_13)) (*process_button)(2,(uint32_t)(TIM5->CNT-butpressed[2])/(SystemCoreClock/1000));
	else butpressed[2]=TIM5->CNT;
	}
}



