/*
    Copyright (C) 2010-20 Jiri Pittner <jiri@pittnerovi.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

//TODO: implement block read and write using DMA and first copy to extra buffer
//TODO@@@@ implement asking for capacity

#include "sdcard.h"

#ifdef AVR_SOURCE
void SPIinit(void) {
        DDRB &= ~(1 << SPIDI);  // set port B SPI data input to input
        DDRB |= (1 << SPICLK);  // set port B SPI clock to output
        DDRB |= (1 << SPIDO);   // set port B SPI data out to output
        DDRB |= (1 << SPICS);   // set port B SPI chip select to output
        SPCR = (1 << SPE) | (1 << MSTR) | (1 << SPR1) | (1 << SPR0);
        cbi(SPSR,0); //SPI2X slower speed
        PORTB &= ~(1 << SPICS); // set chip select to low (active)
}

void SPIhigh(void) {
        //sbi(SPSR,0);
        cbi(SPCR,SPR1); cbi(SPCR,SPR0);
}

uint8_t SPI(uint8_t d) {  // send character over SPI
        char received = 0;
        SPDR = d;
        while(!(SPSR & (1<<SPIF)));
        received = SPDR;
        return (received);
}

#define CSPASSIVE sbi(PORTB,SPICS)
#define CSACTIVE cbi(PORTB,SPICS)

volatile uint8_t buffer[SD_BUFSIZE];  //has to be interpreted as unsigned
volatile uint16_t inptr;
volatile uint16_t n_read;
volatile uint8_t opmode;
#endif


///////////////////////////////////////////////////////////////////
#ifdef ARM_SOURCE37
//PIN connections: SPI1 as AF5: CS=PC6, SCK=PC7, MISO=PC8, MOSI=PC9
#include "stm32f37x.h"
#include "stm32f37x_conf.h"
#include "stm32f37x_spi.h"
#include "delay.h"
#define delayMicroseconds(x) delay_us(x)

#define SPI_FLAG_TXE SPI_I2S_FLAG_TXE
#define SPI_FLAG_RXNE SPI_I2S_FLAG_RXNE
#define SPI_FLAG_BSY SPI_I2S_FLAG_BSY

#define CSACTIVE GPIO_ResetBits(GPIOC, GPIO_Pin_6)
#define CSPASSIVE GPIO_SetBits(GPIOC, GPIO_Pin_6)
#define waitspi() while(SPI_I2S_GetFlagStatus(SPI1,SPI_I2S_FLAG_BSY)==SET)

void SPI1_SendByte(u8 byte) {
u32 LIS302DLTimeout = 0x1000;

while (SPI_I2S_GetFlagStatus(SPI1, SPI_FLAG_TXE) == RESET) 
{
//@@@@@if((LIS302DLTimeout--) == 0) return;
}
/* Send a Byte through the SPI peripheral */
SPI_SendData8(SPI1, byte);
}

uint8_t SPI1_GetByte() {
u32 LIS302DLTimeout = 0x1000;
/* Wait to receive a Byte */
while (SPI_I2S_GetFlagStatus(SPI1,  SPI_FLAG_RXNE) == RESET) {
                                    if((LIS302DLTimeout--) == 0) return 0x00; //really needed at least on the 37!!!!
                                        }
/* Return the Byte read from the SPI bus */
uint8_t r= SPI_ReceiveData8(SPI1);
return r;
}

uint8_t SPI(uint8_t d) //WR one byte 
{  
while (SPI_I2S_GetFlagStatus(SPI1, SPI_FLAG_TXE) == RESET);
SPI_SendData8(SPI1, d);
u32 LIS302DLTimeout = 0x1000;
while(SPI_I2S_GetFlagStatus(SPI1,  SPI_FLAG_RXNE) == RESET) {
                                    if((LIS302DLTimeout--) == 0) return 0x00; //really needed at least on the 37!!!!
                                        }
uint8_t r= SPI_ReceiveData8(SPI1);
//printf("SPI in %02x out %02x\n",d,r);
return r;
}


void spi_writeread(uint8_t *msg, uint16_t len, uint8_t release)
{
CSACTIVE;
while (SPI_I2S_GetFlagStatus(SPI1,  SPI_I2S_FLAG_RXNE) == SET) SPI_ReceiveData8(SPI1); //flush rx fifo
while(len--)
        {
        SPI1_SendByte(*msg);
        *msg++ = SPI1_GetByte();
        }
FlagStatus s;
do {
        s= SPI_I2S_GetFlagStatus(SPI1,SPI_I2S_FLAG_BSY);
        }while(s==SET);
delay_us(3);
if(release) CSPASSIVE;
}


void spi_writeonly(uint8_t *msg, uint16_t len, uint8_t release)
{
CSACTIVE;
while (SPI_I2S_GetFlagStatus(SPI1,  SPI_I2S_FLAG_RXNE) == SET) SPI_ReceiveData8(SPI1); //flush rx fifo
uint8_t dummy;
while(len--)
        {
        SPI1_SendByte(*msg++);
        dummy = SPI1_GetByte();
        }
FlagStatus s;
do {
        s= SPI_I2S_GetFlagStatus(SPI1,SPI_I2S_FLAG_BSY);
        }while(s==SET);
delay_us(3);
if(release) CSPASSIVE;
}


void SPIinit(void)
{
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_SPI1, ENABLE);
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOC, ENABLE);

    SPI_I2S_DeInit(SPI1);

    GPIO_InitTypeDef GPIO_InitStruct;
    GPIO_StructInit(&GPIO_InitStruct);
    GPIO_InitStruct.GPIO_OType = GPIO_OType_PP;
    GPIO_InitStruct.GPIO_PuPd = GPIO_PuPd_NOPULL;
    GPIO_InitStruct.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStruct.GPIO_Pin = GPIO_Pin_8|GPIO_Pin_7|GPIO_Pin_9;
    GPIO_InitStruct.GPIO_Mode = GPIO_Mode_AF;
    GPIO_Init(GPIOC, &GPIO_InitStruct);

    //CS pin is not AF but regular output
    GPIO_InitStruct.GPIO_Pin = GPIO_Pin_6;
    GPIO_InitStruct.GPIO_Mode = GPIO_Mode_OUT;
    GPIO_Init(GPIOC, &GPIO_InitStruct);
    CSPASSIVE;
    //while(1) {GPIO_SetBits(GPIOC, GPIO_Pin_7); delay_ms(1); GPIO_ResetBits(GPIOC, GPIO_Pin_7); delay_ms(1);}

    GPIO_PinAFConfig(GPIOC, GPIO_PinSource7, GPIO_AF_5);
    GPIO_PinAFConfig(GPIOC, GPIO_PinSource8, GPIO_AF_5);
    GPIO_PinAFConfig(GPIOC, GPIO_PinSource9, GPIO_AF_5);


    SPI_Cmd(SPI1, DISABLE);
    SPI_I2S_DeInit(SPI1);
    SPI_InitTypeDef SPI_InitStructure;
    SPI_StructInit(&SPI_InitStructure);
    SPI_InitStructure.SPI_Direction = SPI_Direction_2Lines_FullDuplex;
    SPI_InitStructure.SPI_DataSize = SPI_DataSize_8b;
    SPI_InitStructure.SPI_CPOL = SPI_CPOL_Low;
    SPI_InitStructure.SPI_CPHA = SPI_CPHA_1Edge;
    SPI_InitStructure.SPI_NSS = SPI_NSS_Soft;
    SPI_InitStructure.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_2; //2 is OK for 72mhz main clock and Kingston SDHC
    SPI_InitStructure.SPI_FirstBit = SPI_FirstBit_MSB;
    SPI_InitStructure.SPI_CRCPolynomial = 0;
    SPI_InitStructure.SPI_Mode = SPI_Mode_Master;
    SPI_Init(SPI1, &SPI_InitStructure);

    //needed on the F37 architecture, does not exist of F4!!!
    SPI_RxFIFOThresholdConfig(SPI1, SPI_RxFIFOThreshold_QF);

    SPI_Cmd(SPI1, ENABLE);

}

#endif 
///////////////////////////////////////////////////////////////////

uint8_t SDCommand(uint8_t cmd, uint32_t *adr, uint8_t crc)
{	
	//SPI(0xFF);
	SPI(cmd);
	uint8_t *addr = (uint8_t *) adr;
	if(adr)
		{
#ifdef little_endian
		SPI(addr[3]);SPI(addr[2]);SPI(addr[1]);SPI(addr[0]);
#else
		SPI(addr[0]);SPI(addr[1]);SPI(addr[2]);SPI(addr[3]);
#endif
		}
else //adr is NULL
		{
		SPI(0); SPI(0); SPI(0); SPI(0);
		}
	SPI(crc);
	uint8_t i=0,r;
	do{
		r=SPI(0xFF);
		++i;
	  }
	while(r&0x80 && i<16);
	return r;	// return the last received character
}

int SD_Init(void) 
{ // init SDCARD
	uint8_t i;
	SPIinit();
  	CSPASSIVE;
	for(i=0; i < 16; i++) SPI(0xFF); // send >= 80 clock pulses
	CSACTIVE;
	for(i=0; i < 2; i++) SPI(0xFF); //16 pulses (really needed?????)

	if (SDCommand(0x40,NULL,0x95) != 1) {CSPASSIVE; return 0xff;} //reset command
	
	{
	uint16_t ix=0;
	do
		{
		i=SDCommand(0x41,NULL,0xFF);
		if(++ix>50000) break;
		}
	while(i); //wait while idle
	if(i) {CSPASSIVE; return i;}
	}

	//go fast now @@@@@@@ARM37?
#ifdef AVR_SOURCE
	SPIhigh();
#endif
	for(i=0; i < 10; i++) SPI(0xFF); 

	CSPASSIVE; return 0;
}

uint8_t SD_error=1;

//LEAVES CS passive after done
int SDHC_Init(void) 
{ // init SDCARD
	uint8_t i;
	SPIinit();
  	CSPASSIVE;
	for(i=0; i < 16; i++) SPI(0xFF); // send >= 80 clock pulses
	CSACTIVE;
	for(i=0; i < 2; i++) SPI(0xFF); //16 pulses (really needed?????)
	if (SDCommand(0x40,NULL,0x95) != 1) {CSPASSIVE; return 0xff;} //reset command
//cmd8
      SPI(0x48);    // CMD8
      SPI(0x00);    //
      SPI(0x00);
      SPI(0x01);    // HVS
      SPI(0xAA);    // test pattern
      SPI(0x87);    // right checksum for CMD

for(i=0; i < 10; i++) SPI(0xFF); 

uint8_t test;
  for (test = 0; test < 100; test++)
   {
CSACTIVE;

       SPI(0xFF);
       SPI(0x77);    // CMD55
       SPI(0x00);   
       SPI(0x00);
       SPI(0x00);
       SPI(0x00);
       SPI(0xFF);

       SPI(0xFF);
       SPI(0xFF);

CSPASSIVE;
       SPI(0xFF);
CSACTIVE;

       SPI(0xFF);
       SPI(0x69);    // CMD41
       SPI(0x40);    // HCS=1
       SPI(0x00);
       SPI(0x00);
       SPI(0x00);
       SPI(0xFF);

	uint8_t j;
       for (j = 0; j < 150; j++)
       {
           // waiting for 00h from card
           uint8_t a1 = SPI(0xFF);
           if (a1 == 0x00)
           {
               // card init OK
               SPI(0xFF);
		CSPASSIVE;
		SD_error=0;
               return 0;
           }
       }

   }


	
	for(i=0; i < 10; i++) SPI(0xFF); 
	CSPASSIVE;
	return 1;
}


static volatile uint16_t outptr=0;
static volatile uint8_t *buf_to_flush=NULL;

uint8_t SD_writeblock_a(uint32_t addr, uint8_t *ptr) { 
//printf("in SD_writeblock_a\n");
	int i;
	uint8_t c;

	// 512 byte-write-mode
	uint16_t ix;
	CSACTIVE;
	char r1 =  SDCommand(0x58,&addr,0xFF);
	for (ix = 0; ix < 50000; ix++) {
		if (r1 == 0) break;
		r1 = SPI(0xFF);
	}
	if (r1) {CSPASSIVE; return r1;}

	SPI(0xFF);
	SPI(0xFF);
	SPI(0xFE);

	// write ram buffer to card
	if(ptr) for (i=0;i<SD_BLOCKSIZE;i++) SPI(*ptr++);
	else {if(buf_to_flush) for (i=0;i<SD_BLOCKSIZE;i++) {SPI(buf_to_flush[outptr++]); if(outptr==SD_BUFSIZE) outptr=0;}}

	// at the end, send 2 dummy bytes
	SPI(0xFF);
	SPI(0xFF);

	c = SPI(0xFF);
	c &= 0x1F;

	if (c != 0x05) {CSPASSIVE; return 0xff;}

	// wait until the card is not busy anymore
	while(SPI(0xFF) != 0xFF);
	CSPASSIVE;
	return 0;
}


uint8_t SD_readblock_a(uint32_t addr, uint8_t *buf) { 
	int i;
	uint16_t ix;
	CSACTIVE;
	char r1 =  SDCommand(0x51,&addr,0xFF);
	for (ix = 0; ix < 50000; ix++) {
		if (r1 == (char)0x00) break;
		r1 = SPI(0xFF);
	}
	if (r1) {CSPASSIVE; return r1; }

	// wait for 0xFE - start of any transmission
	ix=0;
	while(SPI(0xFF) != 0xFE) {if(++ix > 50000) {CSPASSIVE; return 0xff;}};

	for(i=0; i < SD_BLOCKSIZE; i++) *buf++ = SPI(0xFF); 

	// at the end, send 2 dummy bytes
	SPI(0xFF); // actually this returns the CRC/checksum byte
	SPI(0xFF);
	CSPASSIVE;
	return 0;
}


