/*
 * *      adc.c
 * *
 * *      Copyright (C) 2012 Jiri Pittner <jiri@pittnerovi.com>
 * *
 * *      This program is free software: you can redistribute it and/or modify
 * *      it under the terms of the GNU General Public License as published by
 * *      the Free Software Foundation, either version 3 of the License, or
 * *      (at your option) any later version.
 * *
 * *      This program is distributed in the hope that it will be useful,
 * *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 * *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * *      GNU General Public License for more details.
 * *      You should have received a copy of the GNU General Public License
 * *      along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * *
 * *
 * */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include "delay.h"
#include "touchscreen.h"
#include "stm32f37x.h"
#include "stm32f37x_rcc.h"
#include "stm32f37x_gpio.h"
#include "stm32f37x_syscfg.h"
#include "stm32f37x_misc.h"
#include "stm32f37x_pwr.h"
#include "stm32f37x_adc.h"



uint32_t read_ADC_C(uint16_t nrep)
{
uint32_t r=0;
//discard first after the switching
//ADC_SoftwareStartConv(ADC1);
while(ADC_GetFlagStatus(ADC1, ADC_FLAG_EOC) == RESET);
ADC_GetConversionValue(ADC1);

do{
	//ADC_SoftwareStartConv(ADC1);
	while(ADC_GetFlagStatus(ADC1, ADC_FLAG_EOC) == RESET);
	r+= ADC_GetConversionValue(ADC1);
}while(--nrep);
return r;
}

void array_read_ADC_C(uint16_t nrep, uint16_t *store)
{
uint16_t i;
for(i=0; i<nrep; ++i)
	{
        //ADC_SoftwareStartConv(ADC1);
        while(ADC_GetFlagStatus(ADC1, ADC_FLAG_EOC) == RESET);
        store[i] = ADC_GetConversionValue(ADC1);
	}
}


uint8_t init_ADC_C(uint8_t channel, uint32_t gpio_pins)
{
RCC_ADCCLKConfig(RCC_PCLK2_Div4); //max 14MHz, Div2 is too fast, maybe use even div6
RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOC, ENABLE);
GPIO_InitTypeDef    GPIO_InitStructure;
GPIO_InitStructure.GPIO_Pin = gpio_pins;
GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AN;
GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL ;
GPIO_Init(GPIOC, &GPIO_InitStructure);

RCC_APB2PeriphClockCmd(RCC_APB2Periph_ADC1, ENABLE);
ADC_DeInit(ADC1);

ADC_InitTypeDef     ADC_InitStructure;
ADC_StructInit(&ADC_InitStructure);
ADC_InitStructure.ADC_ScanConvMode = DISABLE;
ADC_InitStructure.ADC_ContinuousConvMode = ENABLE; //faster than always starting it
ADC_InitStructure.ADC_ExternalTrigConv = ADC_ExternalTrigConv_None;
ADC_InitStructure.ADC_DataAlign = ADC_DataAlign_Right;
ADC_InitStructure.ADC_NbrOfChannel = 1;
ADC_Init(ADC1, &ADC_InitStructure);

ADC_RegularChannelConfig(ADC1, ADC_Channel_Vrefint, 1, ADC_SampleTime_13Cycles5);
ADC_Cmd(ADC1, DISABLE);
ADC_TempSensorVrefintCmd(ENABLE);
delay_us(100);
ADC_Cmd(ADC1, ENABLE);

/* ADC1 calibration start */
ADC_ResetCalibration(ADC1);
while(ADC_GetResetCalibrationStatus(ADC1));
ADC_StartCalibration(ADC1);
while(ADC_GetCalibrationStatus(ADC1));
ADC_Cmd(ADC1, ENABLE);

ADC_RegularChannelConfig(ADC1, channel, 1, ADC_SampleTime_13Cycles5);
ADC_Cmd(ADC1, ENABLE);
return 0;
}


//connected to PC0-PC4, ADC channel 10-14
//PC0-PC2 = vertical y
//PC1-PC3 = horizontal x
//PC4 = power sense
//

#define nsamples 32
//calibration has to be performed for a given sensor!
#define XMIN 300
#define XMAX 3790
#define YMIN 640
#define YMAX 3440

//return nonzero if a touch is detected 
uint8_t touched(float *x, float *y)
{
GPIO_InitTypeDef    GPIO_InitStructure;

//test for cross-contact of x and y
init_ADC_C(10,GPIO_Pin_0|GPIO_Pin_1|GPIO_Pin_4);
GPIO_InitStructure.GPIO_Pin = GPIO_Pin_2|GPIO_Pin_3;
GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL ;
GPIO_Init(GPIOC, &GPIO_InitStructure);
GPIO_SetBits(GPIOC,GPIO_Pin_2); GPIO_ResetBits(GPIOC,GPIO_Pin_3);
delay_us(2);
uint32_t ch10=read_ADC_C(nsamples);
init_ADC_C(11,GPIO_Pin_0|GPIO_Pin_1|GPIO_Pin_4);
uint32_t ch11=read_ADC_C(nsamples);
//printf(" ch10 and 11 %d %d\n",ch10,ch11);
if(ch10>nsamples*3900 && ch11 <nsamples*160) return 0; //no touch
GPIO_ResetBits(GPIOC,GPIO_Pin_2); GPIO_ResetBits(GPIOC,GPIO_Pin_3);

//x coordinate polarize x readout y
init_ADC_C(10,GPIO_Pin_0|GPIO_Pin_2|GPIO_Pin_4);

GPIO_InitStructure.GPIO_Pin = GPIO_Pin_1|GPIO_Pin_3;
GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL ;
GPIO_Init(GPIOC, &GPIO_InitStructure);

GPIO_SetBits(GPIOC,GPIO_Pin_1); GPIO_ResetBits(GPIOC,GPIO_Pin_3);
delay_us(2);
uint32_t readout10=read_ADC_C(nsamples);
GPIO_ResetBits(GPIOC,GPIO_Pin_1); GPIO_SetBits(GPIOC,GPIO_Pin_3);
delay_us(2);
uint32_t readout01=read_ADC_C(nsamples);
GPIO_ResetBits(GPIOC,GPIO_Pin_1); GPIO_ResetBits(GPIOC,GPIO_Pin_3);

//y coordinate polarize y readout x
init_ADC_C(11,GPIO_Pin_1|GPIO_Pin_3|GPIO_Pin_4);

GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0|GPIO_Pin_2;
GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL ;
GPIO_Init(GPIOC, &GPIO_InitStructure);

GPIO_SetBits(GPIOC,GPIO_Pin_0); GPIO_ResetBits(GPIOC,GPIO_Pin_2);
delay_us(2);
uint32_t readout10y=read_ADC_C(nsamples);
GPIO_ResetBits(GPIOC,GPIO_Pin_0); GPIO_SetBits(GPIOC,GPIO_Pin_2);
delay_us(2);
uint32_t readout01y=read_ADC_C(nsamples);
GPIO_ResetBits(GPIOC,GPIO_Pin_0); GPIO_ResetBits(GPIOC,GPIO_Pin_2);

//all pins analog 
init_ADC_C(10,GPIO_Pin_0|GPIO_Pin_1|GPIO_Pin_2|GPIO_Pin_3|GPIO_Pin_4);

float x1= readout01-XMIN*nsamples;
x1/= (XMAX-XMIN)*nsamples;
float x2= readout10-XMIN*nsamples;
x2/= (XMAX-XMIN)*nsamples;
*x = 0.5f*(x1+ 1.f-x2);

float y1= readout01y-YMIN*nsamples;
y1/= (YMAX-YMIN)*nsamples;
float y2= readout10y-YMIN*nsamples;
y2/= (YMAX-YMIN)*nsamples;
*y = 0.5f*(y1+ 1.f-y2);

if(*x<0 || *x>1||*y <0||*y>1)
	{
	printf("touchscreen calibration error %f  %d %d %f %d %d \n",*x,readout01,readout10,*y,readout01y,readout10y);
	return 0;
	}

return 1;
}
