/*
 *      usart.c ... libc stdin/stdout to USART1 interface for STM32F37
 *
 *      Copyright (C) 2012 Jiri Pittner <jiri@pittnerovi.com>
 *
 *      This program is free software: you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation, either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      You should have received a copy of the GNU General Public License
 *      along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 */


#include "stm32f37x_conf.h" 
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include "usart.h"
#include "stm32f37x_rcc.h"
#include "stm32f37x_dma.h"
#include "stm32f37x_gpio.h"



void printsimple(USART_TypeDef* USARTx, char *t)
{
while(*t)
	{
	//assert_param(IS_USART_ALL_PERIPH(USARTx));
	//  assert_param(IS_USART_DATA(Data));
	while((USARTx->ISR & USART_ISR_TC)==0);
  	USARTx->TDR = *t++;
	}
}


#define INUART1MAX 128
volatile uint8_t inuart1len=0;
volatile uint8_t inuart1line=0;
volatile char inuart1[INUART1MAX];


void USART1_IRQHandler(void){
	char c = USART1->RDR; 
	if(c=='\r') c='\n';
	while((USART1->ISR & USART_ISR_TC)==0); USART1->TDR = c; //echo
	if(!inuart1line) 
		{
		inuart1[inuart1len++] = c;
		if(c=='\n' || inuart1len == INUART1MAX) 
#ifdef oldversion
			{inuart1[inuart1len-1] = 0; inuart1line=inuart1len; 
#else
			{inuart1[inuart1len] = 0; inuart1line=inuart1len; //do not substract newlines
#endif
			}
		}
}

#define MAXUSART1OUT 256
static char outuart1[MAXUSART1OUT];
static DMA_InitTypeDef DMA_InitStruct;
static uint8_t uart1virgin=1;

void print(USART_TypeDef* USARTx, char *t)
{
if(USARTx!=USART1) printsimple(USARTx,t);
if(*t == 0) return;
while((USART1->ISR & USART_ISR_TC)==0); //wait for non-dma tx completion
if(!uart1virgin) while(DMA_GetFlagStatus(DMA1_FLAG_TC4)==0 ); //wait for dma tx completion
uint16_t len=0;
while(*t) {outuart1[len++]= *t++; if(len==MAXUSART1OUT-1) {outuart1[len++]=0; break;}}
DMA_Cmd(DMA1_Channel4, DISABLE);
DMA_InitStruct.DMA_BufferSize = DMA1_Channel4->CNDTR = len;
DMA_ClearFlag(DMA1_FLAG_TC4); //clear transfer complete to allow next one
DMA_ClearFlag(DMA1_FLAG_TE4); //clear transfer error
DMA_Cmd(DMA1_Channel4, ENABLE);
uart1virgin=0;
}


void printn(USART_TypeDef* USARTx, const char *t, int len)
{
//only usart1 supported now
if(len <= 0) return;
while((USART1->ISR & USART_ISR_TC)==0); //wait for non-dma tx completion
if(!uart1virgin) while(DMA_GetFlagStatus(DMA1_FLAG_TC4)==0); //wait for dma tx completion
uint16_t l=0;
while(l<len) {outuart1[l++]= *t++; if(l==MAXUSART1OUT-1) {outuart1[l++]=0; break;}}
DMA_Cmd(DMA1_Channel4, DISABLE);
DMA_InitStruct.DMA_BufferSize = DMA1_Channel4->CNDTR = len;
DMA_ClearFlag(DMA1_FLAG_TC4); //clear transfer complete to allow next one
DMA_ClearFlag(DMA1_FLAG_TE4); //clear transfer error
DMA_Cmd(DMA1_Channel4, ENABLE);
uart1virgin=0;
}

#ifndef oldversion
ssize_t _write(int fd, const void *buf, size_t count)
{
/*
{
print(USART1,"write ");
char zz[16];
sprintf(zz,"%d",fd);
print(USART1,zz);
print(USART1,"\n");
}
*/
if(fd!=1 && fd!=2) return -1;
printn(USART1,buf,count);
return count>MAXUSART1OUT ? MAXUSART1OUT : count;
}

ssize_t _read(int fd, const void *buf, size_t count)
{
/*
{
print(USART1,"read ");
char z[2]="0";
z[0]+=fd;
print(USART1,z);
print(USART1,"\n");
}
*/
if(fd!=0) return -1;
while(!inuart1line); //blocking read
int len= count<inuart1len?count:inuart1len;
memcpy(buf,inuart1,len);
if(count<inuart1len)
	{
	inuart1len-=len;
	memmove(inuart1,inuart1+len,inuart1len);
	}
else
	inuart1line=inuart1len=0;
return len;
}

#endif

void scan(USART_TypeDef* USARTx, char *string)
{
char c;
        do      {
                do {
			while((USARTx->ISR & USART_ISR_RXNE)==0);
                        c = USARTx->RDR;
                        } while ((USARTx->ISR & USART_ISR_FE)==1);
                *string++ = c;
		while((USARTx->ISR & USART_ISR_TC)==0);
		USARTx->TDR = c;
                } while ( c != '\r' );
	while((USARTx->ISR & USART_ISR_TC)==0);
	USARTx->TDR = '\n';
        string[-1]=0;
}


void usart1_init(uint32_t baud)
{
GPIO_InitTypeDef GPIO_InitStructure;
USART_InitTypeDef USART_InitStructure;

inuart1line=inuart1len=0;

RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA, ENABLE);
RCC_APB2PeriphClockCmd(RCC_APB2Periph_USART1, ENABLE); 

/* GPIOA clock enable */
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_USART1, ENABLE);

/* GPIOA Configuration:  USART1 TX on PA9, RX on PA10 */
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_9;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL ;
	GPIO_Init(GPIOA, &GPIO_InitStructure); 

	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_Init(GPIOA, &GPIO_InitStructure); 


/* Connect USART1 pins to AF */ 
	// TX = PA9, RX = PA10
 	GPIO_PinAFConfig(GPIOA, GPIO_PinSource9, GPIO_AF_7);
 	GPIO_PinAFConfig(GPIOA, GPIO_PinSource10, GPIO_AF_7);
	USART_InitStructure.USART_BaudRate = baud;
	USART_InitStructure.USART_WordLength = USART_WordLength_8b;
	USART_InitStructure.USART_StopBits = USART_StopBits_1;
	USART_InitStructure.USART_Parity = USART_Parity_No;
	USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
	USART_InitStructure.USART_Mode = USART_Mode_Rx |USART_Mode_Tx;
	USART_Init(USART1, &USART_InitStructure);

	USART_Cmd(USART1, ENABLE); // enable USART1

  RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1, ENABLE);
//DMA USART1 TX = dma1 channel 4 
  DMA_StructInit(&DMA_InitStruct);
  DMA_InitStruct.DMA_PeripheralBaseAddr = (uint32_t)&(USART1->TDR);
  DMA_InitStruct.DMA_MemoryBaseAddr = (uint32_t)outuart1;
  DMA_InitStruct.DMA_DIR = DMA_DIR_PeripheralDST;
  DMA_InitStruct.DMA_BufferSize = 0;
  DMA_InitStruct.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
  DMA_InitStruct.DMA_MemoryInc = DMA_MemoryInc_Enable;
  DMA_InitStruct.DMA_PeripheralDataSize = DMA_PeripheralDataSize_Byte;
  DMA_InitStruct.DMA_MemoryDataSize = DMA_MemoryDataSize_Byte;
  DMA_InitStruct.DMA_Mode = DMA_Mode_Normal;
  DMA_InitStruct.DMA_Priority = DMA_Priority_Medium;
  DMA_InitStruct.DMA_M2M = DMA_M2M_Disable;
  DMA_Init(DMA1_Channel4, &DMA_InitStruct);

USART_DMACmd(USART1, USART_DMAReq_Tx, ENABLE);

//for rx
USART_ITConfig(USART1, USART_IT_RXNE, ENABLE); 
NVIC_InitTypeDef NVIC_InitStructure;
NVIC_InitStructure.NVIC_IRQChannel = USART1_IRQn;		 
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;		
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;	
	NVIC_Init(&NVIC_InitStructure);				
}


