/*
 *      usart.c ... libc stdin/stdout to USART1,2,3 interface for STM32F37
 *
 *      Copyright (C) 2014 Jiri Pittner <jiri@pittnerovi.com>
 *
 *      This program is free software: you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation, either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      You should have received a copy of the GNU General Public License
 *      along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 */


#include "stm32f37x_conf.h" 
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "usart123.h"
#include "stm32f37x_rcc.h"
#include "stm32f37x_dma.h"
#include "stm32f37x_gpio.h"

#ifdef WITH_USB
#include "usb/inc/ttyacm.h"
#endif

#ifdef WITH_FAT
#include "fat_interface.h"
#endif

#ifdef WITH_ETHERNET
#include "lan.h"
#endif


/* PINOUT
 *
 * #ifdef USART1_ON_PE01
 * PE0=USART1TX
 * PE1=USART1RX
 * #ifdef USART1_ON_PA910
 * PA9=USART1TX
 * PA10=USART1RX
 *
 * PA2 = USART2TX
 * PA3 = USART2RX
 *
 * PC10=USART3TX
 * PC11=USART3RX
 *
 */

#if !defined(USART1_ON_PE01) && !defined(USART1_ON_PA910)
#error CHOOSE USART1 PINOUT
#endif

#if defined(USART1_ON_PE01) &&  defined(USART1_ON_PA910)
#error CHOOSE A UNIQUE USART1 PINOUT
#endif



void printsimple(USART_TypeDef* USARTx, char *t)
{
while(*t)
	{
	//assert_param(IS_USART_ALL_PERIPH(USARTx));
	//  assert_param(IS_USART_DATA(Data));
	while((USARTx->ISR & USART_ISR_TC)==0);
  	USARTx->TDR = *t++;
	}
}


volatile uint8_t inuart1len=0;
volatile uint8_t inuart1line=0;
volatile char inuart1[INUARTMAX+1];

volatile uint8_t inuart2len=0;
volatile uint8_t inuart2line=0;
volatile char inuart2[INUARTMAX+1];

volatile uint8_t inuart3len=0;
volatile uint8_t inuart3line=0;
volatile char inuart3[INUARTMAX+1];

uint8_t usart1echo=1;
uint8_t usart2echo=1;
uint8_t usart3echo=1;



void USART1_IRQHandler(void){
       USART_ClearITPendingBit(USART1,USART_IT_RTO);
        USART_ClearITPendingBit(USART1,USART_IT_ORE);
        USART_ClearITPendingBit(USART1,USART_IT_FE);
        USART_ClearITPendingBit(USART1,USART_IT_PE);

//NO HANGS if(!USART_GetITStatus(USART1, USART_ISR_RXNE)) return;
	char c = USART1->RDR; 
        if(c=='\r') c='\n'; //implement option for conversion if needed...
	if(usart1echo) {while((USART1->ISR & USART_ISR_TC)==0); USART1->TDR = c;}
	if(inuart1len<INUARTMAX) 
		{
		inuart1[inuart1len++] = c;
		if(c=='\n' || inuart1len == INUARTMAX) 
			{inuart1[inuart1len] = 0; inuart1line=1; //do not substract newlines
			}
		}
}

void USART2_IRQHandler(void){
	USART_ClearITPendingBit(USART2,USART_IT_RTO);
	USART_ClearITPendingBit(USART2,USART_IT_ORE);
	USART_ClearITPendingBit(USART2,USART_IT_FE);
	USART_ClearITPendingBit(USART2,USART_IT_PE);
//NO HANGS if(!USART_GetITStatus(USART2, USART_ISR_RXNE)) return;
	char c = USART2->RDR; 
	if(c=='\r') c='\n'; //implement option for conversion if needed...
	if(usart2echo) {while((USART2->ISR & USART_ISR_TC)==0); USART2->TDR = c;}
	if(inuart2len<INUARTMAX) 
		{
		inuart2[inuart2len++] = c;
		if(c=='\n' || inuart2len == INUARTMAX) 
			{inuart2[inuart2len] = 0; inuart2line=1; //do not substract newlines
			}
		}
}

void USART3_IRQHandler(void){
        USART_ClearITPendingBit(USART3,USART_IT_RTO);
        USART_ClearITPendingBit(USART3,USART_IT_ORE);
        USART_ClearITPendingBit(USART3,USART_IT_FE);
        USART_ClearITPendingBit(USART3,USART_IT_PE);
//NO HANGS if(!USART_GetITStatus(USART3, USART_ISR_RXNE)) return;
	char c = USART3->RDR; 
        if(c=='\r') c='\n'; //implement option for conversion if needed...
	if(usart3echo) {while((USART3->ISR & USART_ISR_TC)==0); USART3->TDR = c;}
	if(inuart3len<INUARTMAX) 
		{
		inuart3[inuart3len++] = c;
		if(c=='\n' || inuart3len == INUARTMAX) 
			{inuart3[inuart3len] = 0; inuart3line=1; //do not substract newlines
			}
		}
}


#define MAXUSARTOUT 256
static char outuart1[MAXUSARTOUT];
static DMA_InitTypeDef DMA_InitStruct1;
static uint8_t uart1virgin=1;

static char outuart2[MAXUSARTOUT];
static DMA_InitTypeDef DMA_InitStruct2;
static uint8_t uart2virgin=1;


static char outuart3[MAXUSARTOUT];
static DMA_InitTypeDef DMA_InitStruct3;
static uint8_t uart3virgin=1;



void printn1(const char *t, int len)
{
if(len <= 0) return;
while((USART1->ISR & USART_ISR_TC)==0); //wait for non-dma tx completion
if(!uart1virgin) while(DMA_GetFlagStatus(DMA1_FLAG_TC4)==0); //wait for dma tx completion
uint16_t l=0;
while(l<len) {outuart1[l++]= *t++; if(l==MAXUSARTOUT-1) {outuart1[l++]=0; break;}}
DMA_Cmd(DMA1_Channel4, DISABLE);
DMA_InitStruct1.DMA_BufferSize = DMA1_Channel4->CNDTR = len;
DMA_ClearFlag(DMA1_FLAG_TC4); //clear transfer complete to allow next one
DMA_ClearFlag(DMA1_FLAG_TE4); //clear transfer error
DMA_Cmd(DMA1_Channel4, ENABLE);
uart1virgin=0;
}


void printn2(const char *t, int len)
{
if(len <= 0) return;
while((USART2->ISR & USART_ISR_TC)==0); //wait for non-dma tx completion
if(!uart2virgin) while(DMA_GetFlagStatus(DMA1_FLAG_TC7)==0); //wait for dma tx completion
uint16_t l=0;
while(l<len) {outuart2[l++]= *t++; if(l==MAXUSARTOUT-1) {outuart2[l++]=0; break;}}
DMA_Cmd(DMA1_Channel7, DISABLE);
DMA_InitStruct2.DMA_BufferSize = DMA1_Channel7->CNDTR = len;
DMA_ClearFlag(DMA1_FLAG_TC7); //clear transfer complete to allow next one
DMA_ClearFlag(DMA1_FLAG_TE7); //clear transfer error
DMA_Cmd(DMA1_Channel7, ENABLE);
uart2virgin=0;
}

void printn3(const char *t, int len)
{
if(len <= 0) return;
while((USART3->ISR & USART_ISR_TC)==0); //wait for non-dma tx completion
if(!uart3virgin) while(DMA_GetFlagStatus(DMA1_FLAG_TC2)==0); //wait for dma tx completion
uint16_t l=0;
while(l<len) {outuart3[l++]= *t++; if(l==MAXUSARTOUT-1) {outuart3[l++]=0; break;}}
DMA_Cmd(DMA1_Channel2, DISABLE);
DMA_InitStruct3.DMA_BufferSize = DMA1_Channel2->CNDTR = len;
DMA_ClearFlag(DMA1_FLAG_TC2); //clear transfer complete to allow next one
DMA_ClearFlag(DMA1_FLAG_TE2); //clear transfer error
DMA_Cmd(DMA1_Channel2, ENABLE);
uart3virgin=0;
}

ssize_t _write(int fd, const void *buf, size_t count)
{
#ifdef WITH_FAT
if(fd>=FAT_HANDLE_OFFSET) return _write_fat(fd,buf,count);
#endif

switch(fd) {
case 3:
	printn1(buf,count); break; 
case 4: printn2(buf,count); break;
case 5: printn3(buf,count); break;
#ifdef WITH_USB
case 6:	if(ttyacm_Transmit(buf,count)) return -1; break;
#endif
#ifdef WITH_ETHERNET
case 7: 
	//fprintf(stderr,"_write udp\n"); fflush(stderr);
	write_udp(buf,count);  break;
#endif
default: 
	//printn3(buf,count); break; //fallback for printf to stdout
	return -1;
}
return count>MAXUSARTOUT ? MAXUSARTOUT : count;
}

ssize_t _read(int fd, const void *buf, size_t count)
{
#ifdef WITH_FAT
if(fd>=FAT_HANDLE_OFFSET) return _read_fat(fd,buf,count);
#endif
int len;
switch(fd) {
case 1:
case 2:
case 3:
	while(!inuart1line); //blocking read
	len= count<inuart1len?count:inuart1len;
	memcpy(buf,inuart1,len);
	if(count<inuart1len) //actually we should disable interrupt or do some locking here
		{
		inuart1len-=len;
		memmove(inuart1,inuart1+len,inuart1len);
		inuart1line= (strchr(inuart1,'\n')!=NULL);
		}
	else
		inuart1line=inuart1len=0;
	break;
case 4:
	while(!inuart2line); //blocking read
	len= count<inuart2len?count:inuart2len;
	memcpy(buf,inuart2,len);
	if(count<inuart2len) //actually we should disable interrupt or do some locking here
		{
		inuart2len-=len;
		memmove(inuart2,inuart2+len,inuart2len);
		inuart2line= (strchr(inuart2,'\n')!=NULL);
		}
	else
		inuart2line=inuart2len=0;
	break;
case 5:
	while(!inuart3line); //blocking read
	len= count<inuart3len?count:inuart3len;
	memcpy(buf,inuart3,len);
	if(count<inuart3len) //actually we should disable interrupt or do some locking here
		{
		inuart3len-=len;
		memmove(inuart3,inuart3+len,inuart3len);
		inuart3line= (strchr(inuart3,'\n')!=NULL);
		}
	else
		inuart3line=inuart3len=0;
	break;
#ifdef WITH_USB
case 6:
        while(!inusbline); //blocking read
        len= count<inusblen?count:inusblen;
        memcpy(buf,inusb,len);
        if(count<inusblen)
                {
                inusblen-=len;
                memmove(inusb,inusb+len,inusblen);
                }
        else
                inusbline=inusblen=0;
        break;
#endif
#ifdef WITH_ETHERNET
//NOTE regular polling has to be done using read_udp() in program's main loop, is not interrupt driven
case 7:
        if(!inudpline) return 0; //cannot be blocking read for the above reason
        len= count<inudplen?count:inudplen;
        memcpy(buf,inudp,len);
        if(count<inudplen)
                {
                inudplen-=len;
                memmove(inudp,inudp+len,inudplen);
                }
        else
                inudpline=inudplen=0;
        break;
#endif

default: return -1;
}
return len;
}


void scan(USART_TypeDef* USARTx, char *string)
{
char c;
        do      {
                do {
			while((USARTx->ISR & USART_ISR_RXNE)==0);
                        c = USARTx->RDR;
                        } while ((USARTx->ISR & USART_ISR_FE)==1);
                *string++ = c;
		while((USARTx->ISR & USART_ISR_TC)==0);
		USARTx->TDR = c;
                } while ( c != '\r' );
	while((USARTx->ISR & USART_ISR_TC)==0);
	USARTx->TDR = '\n';
        string[-1]=0;
}


void usart1_init(uint32_t baud)
{
GPIO_InitTypeDef GPIO_InitStructure;
USART_InitTypeDef USART_InitStructure;

inuart1line=inuart1len=0;

#ifdef USART1_ON_PE01
RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOE, ENABLE);
#endif
#ifdef USART1_ON_PA910
RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA, ENABLE);
#endif

RCC_APB2PeriphClockCmd(RCC_APB2Periph_USART1, ENABLE);

#ifdef USART1_ON_PE01
/* GPIOE Configuration:  USART1 TX on PE0 RX on PE1 */
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL ;
	GPIO_Init(GPIOE, &GPIO_InitStructure); 

	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_1;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_Init(GPIOE, &GPIO_InitStructure); 

 	GPIO_PinAFConfig(GPIOE, GPIO_PinSource0, GPIO_AF_7);
 	GPIO_PinAFConfig(GPIOE, GPIO_PinSource1, GPIO_AF_7);
#endif
#ifdef USART1_ON_PA910
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_9;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL ;
	GPIO_Init(GPIOA, &GPIO_InitStructure); 

	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_Init(GPIOA, &GPIO_InitStructure); 

 	GPIO_PinAFConfig(GPIOA, GPIO_PinSource9, GPIO_AF_7);
 	GPIO_PinAFConfig(GPIOA, GPIO_PinSource10, GPIO_AF_7);
#endif


	USART_InitStructure.USART_BaudRate = baud;
	USART_InitStructure.USART_WordLength = USART_WordLength_8b;
	USART_InitStructure.USART_StopBits = USART_StopBits_1;
	USART_InitStructure.USART_Parity = USART_Parity_No;
	USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
	USART_InitStructure.USART_Mode = USART_Mode_Rx |USART_Mode_Tx;
	USART_Init(USART1, &USART_InitStructure);

	USART_Cmd(USART1, ENABLE); // enable USART1

  RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1, ENABLE);
//DMA USART1 TX = dma1 channel 4 
  DMA_StructInit(&DMA_InitStruct1);
  DMA_InitStruct1.DMA_PeripheralBaseAddr = (uint32_t)&(USART1->TDR);
  DMA_InitStruct1.DMA_MemoryBaseAddr = (uint32_t)outuart1;
  DMA_InitStruct1.DMA_DIR = DMA_DIR_PeripheralDST;
  DMA_InitStruct1.DMA_BufferSize = 0;
  DMA_InitStruct1.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
  DMA_InitStruct1.DMA_MemoryInc = DMA_MemoryInc_Enable;
  DMA_InitStruct1.DMA_PeripheralDataSize = DMA_PeripheralDataSize_Byte;
  DMA_InitStruct1.DMA_MemoryDataSize = DMA_MemoryDataSize_Byte;
  DMA_InitStruct1.DMA_Mode = DMA_Mode_Normal;
  DMA_InitStruct1.DMA_Priority = DMA_Priority_Medium;
  DMA_InitStruct1.DMA_M2M = DMA_M2M_Disable;
  DMA_Init(DMA1_Channel4, &DMA_InitStruct1);

USART_DMACmd(USART1, USART_DMAReq_Tx, ENABLE);

//for rx
USART_ITConfig(USART1, USART_IT_RXNE, ENABLE); 
NVIC_InitTypeDef NVIC_InitStructure;
NVIC_InitStructure.NVIC_IRQChannel = USART1_IRQn;		 
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;		
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;	
	NVIC_Init(&NVIC_InitStructure);				

}


void usart2_init(uint32_t baud)
{
GPIO_InitTypeDef GPIO_InitStructure;
USART_InitTypeDef USART_InitStructure;

inuart2line=inuart2len=0;

/* GPIOA clock enable */
RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA, ENABLE);
RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART2, ENABLE); 


/* GPIOA Configuration:  USART2 TX on PA2, RX on PA3 */
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_2;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP ;
	GPIO_Init(GPIOA, &GPIO_InitStructure); 

	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_3;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_Init(GPIOA, &GPIO_InitStructure); 


/* Connect USART2 pins to AF */ 
 	GPIO_PinAFConfig(GPIOA, GPIO_PinSource2, GPIO_AF_7);
 	GPIO_PinAFConfig(GPIOA, GPIO_PinSource3, GPIO_AF_7);
	USART_InitStructure.USART_BaudRate = baud;
	USART_InitStructure.USART_WordLength = USART_WordLength_8b;
	USART_InitStructure.USART_StopBits = USART_StopBits_1;
	USART_InitStructure.USART_Parity = USART_Parity_No;
	USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
	USART_InitStructure.USART_Mode = USART_Mode_Rx |USART_Mode_Tx;
	USART_Init(USART2, &USART_InitStructure);

	USART_Cmd(USART2, ENABLE); // enable USART2

  RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1, ENABLE);

//DMA usart2 TX = dma1 channel 7 
  DMA_StructInit(&DMA_InitStruct2);
  DMA_InitStruct2.DMA_PeripheralBaseAddr = (uint32_t)&(USART2->TDR);
  DMA_InitStruct2.DMA_MemoryBaseAddr = (uint32_t)outuart2;
  DMA_InitStruct2.DMA_DIR = DMA_DIR_PeripheralDST;
  DMA_InitStruct2.DMA_BufferSize = 0;
  DMA_InitStruct2.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
  DMA_InitStruct2.DMA_MemoryInc = DMA_MemoryInc_Enable;
  DMA_InitStruct2.DMA_PeripheralDataSize = DMA_PeripheralDataSize_Byte;
  DMA_InitStruct2.DMA_MemoryDataSize = DMA_MemoryDataSize_Byte;
  DMA_InitStruct2.DMA_Mode = DMA_Mode_Normal;
  DMA_InitStruct2.DMA_Priority = DMA_Priority_Medium;
  DMA_InitStruct2.DMA_M2M = DMA_M2M_Disable;
  DMA_Init(DMA1_Channel7, &DMA_InitStruct2);

USART_DMACmd(USART2, USART_DMAReq_Tx, ENABLE);

//for rx
USART_ITConfig(USART2, USART_IT_RXNE, ENABLE); 
NVIC_InitTypeDef NVIC_InitStructure;
NVIC_InitStructure.NVIC_IRQChannel = USART2_IRQn;		 
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;		
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;	
	NVIC_Init(&NVIC_InitStructure);				

}


void usart3_init(uint32_t baud)
{
GPIO_InitTypeDef GPIO_InitStructure;
USART_InitTypeDef USART_InitStructure;

inuart3line=inuart3len=0;

/* GPIOC clock enable */
RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOC, ENABLE);
RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART3, ENABLE); 


/* GPIOC Configuration:  USART3 TX on PC10, RX on PC11 */
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP ;
	GPIO_Init(GPIOC, &GPIO_InitStructure); 

	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_11;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_Init(GPIOC, &GPIO_InitStructure); 


/* Connect USART3 pins to AF */ 
 	GPIO_PinAFConfig(GPIOC, GPIO_PinSource10, GPIO_AF_7);
 	GPIO_PinAFConfig(GPIOC, GPIO_PinSource11, GPIO_AF_7);
	USART_InitStructure.USART_BaudRate = baud;
	USART_InitStructure.USART_WordLength = USART_WordLength_8b;
	USART_InitStructure.USART_StopBits = USART_StopBits_1;
	USART_InitStructure.USART_Parity = USART_Parity_No;
	USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
	USART_InitStructure.USART_Mode = USART_Mode_Rx |USART_Mode_Tx;
	USART_Init(USART3, &USART_InitStructure);

	USART_Cmd(USART3, ENABLE); // enable USART3

  RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1, ENABLE);

//DMA usart3 TX = dma1 channel 2 
  DMA_StructInit(&DMA_InitStruct3);
  DMA_InitStruct3.DMA_PeripheralBaseAddr = (uint32_t)&(USART3->TDR);
  DMA_InitStruct3.DMA_MemoryBaseAddr = (uint32_t)outuart3;
  DMA_InitStruct3.DMA_DIR = DMA_DIR_PeripheralDST;
  DMA_InitStruct3.DMA_BufferSize = 0;
  DMA_InitStruct3.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
  DMA_InitStruct3.DMA_MemoryInc = DMA_MemoryInc_Enable;
  DMA_InitStruct3.DMA_PeripheralDataSize = DMA_PeripheralDataSize_Byte;
  DMA_InitStruct3.DMA_MemoryDataSize = DMA_MemoryDataSize_Byte;
  DMA_InitStruct3.DMA_Mode = DMA_Mode_Normal;
  DMA_InitStruct3.DMA_Priority = DMA_Priority_Medium;
  DMA_InitStruct3.DMA_M2M = DMA_M2M_Disable;
  DMA_Init(DMA1_Channel2, &DMA_InitStruct3);

USART_DMACmd(USART3, USART_DMAReq_Tx, ENABLE);

//for rx
USART_ITConfig(USART3, USART_IT_RXNE, ENABLE); 
NVIC_InitTypeDef NVIC_InitStructure;
NVIC_InitStructure.NVIC_IRQChannel = USART3_IRQn;		 
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;		
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;	
	NVIC_Init(&NVIC_InitStructure);				

}


int _open(const char *pathname, int flags, mode_t mode)
{
int n,baud,echo;
#ifdef WITH_FAT 
if(pathname[0]=='/') return _open_fat(pathname,flags,mode);
#endif
#ifdef WITH_USB
if(!strncmp(pathname,"usb",3))
	{
	if(1!=sscanf(pathname,"usb %d",&n)) return -1;
	usbecho=n;
	uint8_t r=ttyacm_init();
	if(r) 
		{
		printf("ttyacm_init returned %d\n",r);
		return -1;
		}
	return 6;
	}
#endif
#ifdef WITH_ETHERNET
if(!strncmp(pathname,"udp",3)) //just formal, initialization has to be done separately
        {
        return 7;
        }       
#endif
if(strncmp(pathname,"usart",5)) return -1;
if(3!=sscanf(pathname,"usart %d %d %d",&n,&baud,&echo)) return -1;
switch(n){
case 1: 
	usart1_init(baud);
	usart1echo=echo;
	break;
case 2:
        usart2_init(baud);
	usart2echo=echo;
        break;
case 3:
        usart3_init(baud);
	usart3echo=echo;
        break;
default: return -1;
}
return n+2;
}

