/**
  ******************************************************************************
  * @file    usbd_msc_storage_sdcard.c
  * @author  MCD Application Team
  * @version V2.4.2
  * @date    11-December-2015
  * @brief   Memory management layer
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; COPYRIGHT 2015 STMicroelectronics</center></h2>
  *
  * Licensed under MCD-ST Liberty SW License Agreement V2, (the "License");
  * You may not use this file except in compliance with the License.
  * You may obtain a copy of the License at:
  *
  *        http://www.st.com/software_license_agreement_liberty_v2
  *
  * Unless required by applicable law or agreed to in writing, software 
  * distributed under the License is distributed on an "AS IS" BASIS, 
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
  *
  ******************************************************************************
  */ 

//adapted J.Pittner <jiri@pittnerovi.com> 2020

#include "sdcard.h"

/* Includes ------------------------------------------------------------------*/
#include "usbd_msc_storage_sdcard.h"


/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Extern function prototypes ------------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

//@@@@NOTE the STORAGE_BLK_NBR should not be fixed but obtained from the SD card
//@@@@presently this is set for 32G card
#define STORAGE_LUN_NBR                  1  
#define STORAGE_BLK_NBR                  67108864L
#define STORAGE_BLK_SIZ                  0x200


/* USB Mass storage Standard Inquiry Data */
int8_t  STORAGE_Inquirydata[] = {//36
  
  /* LUN 0 */
  0x00,		
  0x80,		
  0x02,		
  0x02,
  (STANDARD_INQUIRY_DATA_LEN - 5),
  0x00,
  0x00,	
  0x00,
  'P', 'i', 't', 't', 'n', 'e', 'r', ' ', /* Manufacturer : 8 bytes */
  'H', 'o', 'm', 'e', 'b', 'r', 'e', 'w', /* Product      : 16 Bytes */
  ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ',
  '0', '.', '0' ,'1',                     /* Version      : 4 Bytes */
}; 

/*******************************************************************************
* Function Name  : Read_Memory
* Description    : Handle the Read operation from the microSD card.
* Input          : None.
* Output         : None.
* Return         : None.
*******************************************************************************/
int8_t STORAGE_Init (uint8_t lun)
{
//we assume SDcard has been explicitely initialized before and FAT was unmounted
//printf("in STORAGE_Init\n");
if(STORAGE_BLK_SIZ!=SD_BLOCKSIZE)
	{
	printf("Block size mismatch between USB MSC and SD card\n");
	return 1;
	}
printf("NOTE: SD Card size assumed max %ld GB\n",STORAGE_BLK_NBR/1024/1024*(long)STORAGE_BLK_SIZ/1024);
  return (0);
}

/*******************************************************************************
* Function Name  : Read_Memory
* Description    : Handle the Read operation from the STORAGE card.
* Input          : None.
* Output         : None.
* Return         : None.
*******************************************************************************/
int8_t STORAGE_GetCapacity (uint8_t lun, uint32_t *block_num, uint16_t *block_size)
{
//printf("in STORAGE_GetCapacity \n");
//@@@TODO dynamically inquire the sdcard size
  *block_num  = STORAGE_BLK_NBR;
  *block_size = STORAGE_BLK_SIZ;
  return (0);
}

/*******************************************************************************
* Function Name  : Read_Memory
* Description    : Handle the Read operation from the STORAGE card.
* Input          : None.
* Output         : None.
* Return         : None.
*******************************************************************************/
int8_t  STORAGE_IsReady (uint8_t lun)
{
//printf("in STORAGE_IsReady \n");
  return (0);
}

/*******************************************************************************
* Function Name  : Read_Memory
* Description    : Handle the Read operation from the STORAGE card.
* Input          : None.
* Output         : None.
* Return         : None.
*******************************************************************************/
int8_t  STORAGE_IsWriteProtected (uint8_t lun)
{
  return  0;
}

/*******************************************************************************
* Function Name  : Read_Memory
* Description    : Handle the Read operation from the STORAGE card.
* Input          : None.
* Output         : None.
* Return         : None.
*******************************************************************************/
int8_t STORAGE_Read (uint8_t lun, 
                 uint8_t *buf, 
                 uint32_t blk_addr,                       
                 uint16_t blk_len)
{
//printf("in STORAGE_Read\n");
uint16_t i;
uint8_t r=0;
for(i=0; i<blk_len; ++i) 
	{
	r+= SD_readblock_a(blk_addr,buf);
	++blk_addr;
	buf+= SD_BLOCKSIZE;
	}
return r;
}
/*******************************************************************************
* Function Name  : Write_Memory
* Description    : Handle the Write operation to the STORAGE card.
* Input          : None.
* Output         : None.
* Return         : None.
*******************************************************************************/
int8_t STORAGE_Write (uint8_t lun, 
                  uint8_t *buf, 
                  uint32_t blk_addr,
                  uint16_t blk_len)
{
//printf("in STORAGE_Write\n");
uint16_t i;
uint8_t r=0;
for(i=0; i<blk_len; ++i) 
        {
        r+= SD_writeblock_a(blk_addr,buf);
        ++blk_addr;
        buf+= SD_BLOCKSIZE;
        }
return r;
}
/*******************************************************************************
* Function Name  : Write_Memory
* Description    : Handle the Write operation to the STORAGE card.
* Input          : None.
* Output         : None.
* Return         : None.
*******************************************************************************/
int8_t STORAGE_GetMaxLun (void)
{
//printf("in STORAGE_GetMaxLun \n");
  return (STORAGE_LUN_NBR - 1);
}



USBD_StorageTypeDef USBD_MSC_sdcard_fops =
{
  STORAGE_Init,
  STORAGE_GetCapacity,
  STORAGE_IsReady,
  STORAGE_IsWriteProtected,
  STORAGE_Read,
  STORAGE_Write,
  STORAGE_GetMaxLun,
  STORAGE_Inquirydata,
  
};
/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/

