/*
 * *      ttyacm.c  ... USB mass storage top level interface
 * *
 * *      Copyright (C) 2020 Jiri Pittner <jiri@pittnerovi.com>
 * *
 * *      This program is free software: you can redistribute it and/or modify
 * *      it under the terms of the GNU General Public License as published by
 * *      the Free Software Foundation, either version 3 of the License, or
 * *      (at your option) any later version.
 * *
 * *      This program is distributed in the hope that it will be useful,
 * *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 * *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * *      GNU General Public License for more details.
 * *      You should have received a copy of the GNU General Public License
 * *      along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * *
 * *
 * *      It employs STM32 HAL and USB library, which are under a different STM license
 *        and NOT included as a part of this program
 * */



#include "usbsd.h"
#include "stm32f37x.h"
#include "usbd_core.h"
#include "usbd_msc.h"
#include "usbd_msc_storage_sdcard.h"
#include "peripherals/stm32f37x_gpio.h"
#include "peripherals/stm32f37x_rcc.h"

extern USBD_DescriptorsTypeDef MSC_Desc;
USBD_HandleTypeDef hUSBDDevice2;

//NOTE: this STM stack does not support two different device classes simultaneously on one logical device
//can we have two USBD_HandleTypeDef devices on one physical device????@@@@@


//main initialization routine
uint8_t usbsd_init(uint8_t inithw)
{
if(inithw)
	{
	          /* Configure USB DM and DP pins */
          GPIO_InitTypeDef  GPIO_InitStruct;
          RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA, ENABLE);
          GPIO_InitStruct.GPIO_Pin = (GPIO_Pin_11 | GPIO_Pin_12);
          GPIO_InitStruct.GPIO_Mode = GPIO_Mode_AF;
          GPIO_InitStruct.GPIO_PuPd = GPIO_PuPd_NOPULL;
          GPIO_InitStruct.GPIO_Speed = GPIO_Speed_50MHz;
          GPIO_Init(GPIOA, &GPIO_InitStruct);
        GPIO_PinAFConfig(GPIOA, GPIO_PinSource11, GPIO_AF_14);
        GPIO_PinAFConfig(GPIOA, GPIO_PinSource12, GPIO_AF_14);
	RCC_USBCLKConfig(RCC_USBCLKSource_PLLCLK_1Div5); //do this after some other ll init as in my_usb_final_init()????@@@@@

        if(USBD_OK != USBD_Init(&hUSBDDevice2, &MSC_Desc, 0)) return 1;
        if(USBD_OK != USBD_RegisterClass(&hUSBDDevice2, USBD_MSC_CLASS)) return 2;
        if(USBD_OK != USBD_MSC_RegisterStorage(&hUSBDDevice2, &USBD_MSC_sdcard_fops)) return 3;
        if(USBD_OK != USBD_Start(&hUSBDDevice2)) return 5;

	}
return 0;
}

uint8_t usbsd_deinit(void)
{
MSC_BOT_DeInit (&hUSBDDevice2);
if(USBD_OK != USBD_Stop(&hUSBDDevice2)) return 5;
if(USBD_OK != USBD_DeInit(&hUSBDDevice2)) return 1;
return 0;
}
